Package: lapop
Type: Package
Title: Processing, Visualizing, and Labeling Americas Barometer Data
Version: 2.1.7
Authors@R: c(person(given = "Robert", family = "Vidigal", 
             role = c("cre","aut"), 
             email = "robert.vidigal@vanderbilt.edu", 
             comment = c(ORCID = "0000-0001-5713-119X")),
             person(given = "Luke", family = "Plutowski", 
             role = c("ctb", "aut"), 
             email = "luke.plutowski@vanderbilt.edu", 
             comment = c(ORCID = "0000-0001-8240-6722")))
Author: Robert Vidigal [cre, aut] (ORCID:
    <https://orcid.org/0000-0001-5713-119X>),
  Luke Plutowski [ctb, aut] (ORCID:
    <https://orcid.org/0000-0001-8240-6722>)
Date: 2026-08-04
Maintainer: Robert Vidigal <robert.vidigal@vanderbilt.edu>
Description: Labeling, weighting, and plotting data following custom style guidelines for use in reports, presentations, and social media posts. The Center for Global Democracy (formerly the Latin American Public Opinion Project) at Vanderbilt University is a leader in public survey research, best known for the Americas Barometer project. The publicly available data can be downloaded from: <https://www.vanderbilt.edu/lapop/data-access.php>.
URL: https://lapop-central.github.io/lapop/
Depends: R (>= 4.1.0)
Imports: ggplot2, ggtext, ggrepel, showtext, grid, gridtext, gridExtra,
        sf, sysfonts, systemfonts, svglite, dplyr, srvyr, survey,
        haven, stats, purrr, tibble, marginaleffects, stringr, zoo
VignetteBuilder: knitr
Suggests: readstata13, rio, rprojroot, knitr, rmarkdown, ggpattern,
        testthat (>= 3.0.0)
Language: en-US
Encoding: UTF-8
License: MIT + file LICENSE
RoxygenNote: 7.3.3
Config/testthat/edition: 3
Collate: 'bra23.R' 'cm23.R' 'globals.R' 'lapop-deprecated.R'
        'lapop_fonts.R' 'lapop_cc.R' 'lapop_cccomb.R' 'lapop_ccm.R'
        'lapop_coef.R' 'lapop_dumb.R' 'lapop_fonts_design.R'
        'lapop_hist.R' 'lapop_map.R' 'lapop_mline.R' 'lapop_mover.R'
        'lapop_save.R' 'lapop_stack.R' 'lapop_ts.R' 'lpr_cc.R'
        'lpr_ccm.R' 'lpr_ci.R' 'lpr_coef.R' 'lpr_data.R' 'lpr_dumb.R'
        'lpr_extract_notes.R' 'lpr_extract_ros.R' 'lpr_hist.R'
        'lpr_mline.R' 'lpr_mover.R' 'lpr_na_attributes.R' 'lpr_resc.R'
        'lpr_set_attr.R' 'lpr_set_ros.R' 'lpr_stack.R' 'lpr_ts.R'
        'world.R' 'ym23.R' 'zzz.R'
NeedsCompilation: no
Packaged: 2026-08-04 16:19:18 UTC; vidigar
Repository: CRAN
Date/Publication: 2026-08-04 17:50:30 UTC
Built: R 4.5.2; ; 2026-08-04 18:41:18 UTC; unix

# Generated by roxygen2: do not edit by hand

export(bra23)
export(cm23)
export(lapop_cc)
export(lapop_cccomb)
export(lapop_ccm)
export(lapop_coef)
export(lapop_db)
export(lapop_demog)
export(lapop_dumb)
export(lapop_fonts)
export(lapop_fonts_design)
export(lapop_hist)
export(lapop_map)
export(lapop_mline)
export(lapop_mover)
export(lapop_save)
export(lapop_sb)
export(lapop_stack)
export(lapop_ts)
export(lapop_tsmulti)
export(lpr_cc)
export(lpr_ccm)
export(lpr_ci)
export(lpr_coef)
export(lpr_data)
export(lpr_dumb)
export(lpr_extract_notes)
export(lpr_extract_ros)
export(lpr_hist)
export(lpr_mline)
export(lpr_mover)
export(lpr_na_attributes)
export(lpr_resc)
export(lpr_set_attr)
export(lpr_set_ros)
export(lpr_stack)
export(lpr_ts)
export(world)
export(ym23)
import(dplyr)
import(ggplot2)
import(ggtext)
import(grDevices)
import(grid)
import(gridtext)
import(haven)
import(marginaleffects)
import(purrr)
import(showtext)
import(srvyr)
import(stringr)
import(survey)
import(sysfonts)
import(systemfonts)
import(tibble)
importFrom(dplyr,filter)
importFrom(dplyr,left_join)
importFrom(dplyr,rename)
importFrom(ggrepel,geom_text_repel)
importFrom(ggtext,element_markdown)
importFrom(gridExtra,grid.arrange)
importFrom(sf,st_bbox)
importFrom(sf,st_drop_geometry)
importFrom(stats,Gamma)
importFrom(stats,ave)
importFrom(stats,confint)
importFrom(stats,gaussian)
importFrom(stats,na.omit)
importFrom(stats,poisson)
importFrom(stats,quasibinomial)
importFrom(stats,quasipoisson)
importFrom(stats,reorder)
importFrom(stats,setNames)
importFrom(stringr,str_wrap)
importFrom(svglite,svglite)
importFrom(zoo,na.approx)

# lapop R package

These are helper functions to wrangle labels and produce visualizations of 'AmericasBarometer' data following LAPOP Lab's editorial guidelines.

🔗 **Package website:** [https://lapop-central.github.io/lapop/](https://lapop-central.github.io/lapop/)

---

## 🛠️ Installation

To install the package in your console, run:

```r
devtools::install_github("lapop-central/lapop", 
                         force = TRUE, 
                         build_vignettes = TRUE)
```

---

## ⚙️ Workflow: 'AmericasBarometer' Variable & Value Labels

For the full online guide, see:

<a href="https://lapop-central.github.io/lapop/articles/lapop-r-labels.html" target="_blank">📖 LAPOP Data Guide for R Users</a>

### 1. Data Structure

'AmericasBarometer' datasets are distributed in Stata `.dta` format with multilingual metadata (question wording and response options) embedded as attributes. These support cross-national and longitudinal comparability.

### 2. Preferred Loading Method

Use:

```r
readstata13::read.dta13()
```

to preserve the full metadata structure.

Other methods such as `haven::read_dta()` or `rio::import()` may fail to import the STATA attributes.

### 3. Variable Labels (Question Wording)

- Stored in the `expansion.fields` attribute.
- Use `lpr_extract_notes()` to convert into a tidy data frame.
- Assign preferred language labels with `lpr_set_attr()` using the appropriate `noteid`.

### 4. Value Labels (Response Options)

- Stored in the `label.table` attribute.
- Use `lpr_set_ros()` to assign these response labels in English, Spanish, or Portuguese.

---

## 🎨 Workflow: 'AmericasBarometer' Data Visualization

<a href="https://lapop-central.github.io/lapop/articles/lapop-visualization.html" target="_blank">📖 LAPOP Visualization Guide</a>

1. Load the package in R:

   ```r
   library(lapop)
   ```

2. LAPOP Lab fonts is automatically loaded, yet you can also manually if needed:

   ```r
   lapop_fonts()
   ```

3. Apply the 'AmericasBarometer' design effects with:

   ```r
   lpr_data()
   ```

4. Choose the appropriate `lpr` graph type:
   - Histograms: `lpr_hist()`
   - Cross-country comparison: `lpr_cc()`
   - Time series: `lpr_ts()`
   - Breakdown by covariates: `lpr_mover()`  

<p>5. Store the output in an R object.</p>

   - File names: .csv and graphics files should have the same name. Their names should be in the following standard format: CountryYear/ts_DVcode(s)_IVcode(s)_graphtype.extension.
   - Examples:
      
      - mex21_countfair1_hist.csv
      - hnd_b4_ts.svg
      - ab23_vic1ext_pais_cc.svg
   - There will be some cases that do not easily fit this standard. Use your best judgment.

6. Use the corresponding `lapop` plotting function to produce the visualization:

   - Examples: `lapop_hist()`, `lapop_cc()`, `lapop_ts()`, etc.

<p>7. Export the figure to your machine with:</p>

   ```r
   lapop_save()
   ```
---

## 🤝 Workflow: Contributing to the `lapop` R Package

1. **Fork** the repository and clone it to your local machine.
2. **Create a new branch** for your feature or fix.
3. Add your new function in the `R/` folder.
4. Document the function with roxygen2 comments.
5. Run `devtools::document()` to generate `.Rd` files in `man/` and update `NAMESPACE`.
6. Commit your changes and push the branch to your fork.
7. Submit a **pull request** to the main repository.
8. If you find a bug, please consider contributing to the lapop package — we spent all our money on coffee and data cleaning.

---

## ----fig setup, include=FALSE-------------------------------------------------

knitr::opts_chunk$set(echo = TRUE, message = FALSE, warning = FALSE)
knitr::opts_chunk$set(fig.width = 12, fig.height = 6)

## ----INSTALL LPR, eval = FALSE------------------------------------------------
# # Install the LAPOP Lab package:
# devtools::install_github("https://github.com/lapop-central/lapop",
#                          force=TRUE,
#                          build_vignettes = TRUE)

## ----LOAD LPR-----------------------------------------------------------------
library(lapop)
# Note: no need for lapop_fonts() 
# because we are not plotting figures

## ----LOAD DATA CHUNK, eval = FALSE--------------------------------------------
# # READSTATA13 PACKAGE (RECOMMENDED FOR LABELS)
# data1 <- read.dta13("./BRA 2023 LAPOP AmericasBarometer (v1.0s).dta")
# names(attributes(data1))
# 
# # HAVEN PACKAGE (RECOMMENDED FOR PLOTTING)
# data2 <- read_dta("./BRA 2023 LAPOP AmericasBarometer (v1.0s).dta")
# names(attributes(data2))
# 
# # RIO PACKAGE (NOT RECOMMENDED)
# data3 <- import("./BRA 2023 LAPOP AmericasBarometer (v1.0s).dta")
# names(attributes(data3))

## ----QWORDING, eval = TRUE, warnings = F--------------------------------------
data(bra23)

# Extracting attributes from expasion.fields
names(attributes(bra23))
varlabels = attr(bra23, "expansion.fields")

## ----STREAMLINED APPROACH, eval = TRUE, warnings = F--------------------------
# Base R approach:
# The base R version uses lapply to iterate over the list and returns NULL when 
# the condition isn't met, then Filter(Negate(is.null), ...) removes those NULLs.
head(
  Filter(Negate(is.null), lapply(attr(bra23, "expansion.fields"), 
  function(x) {
    if (x[1] == "ing4") list(note_id = x[2], note_value = x[3])})), n = 3)

# purrr package approach
#  The purrr version uses map() to do the same, and also remove NULL results. 
tail(
  purrr::map_dfr(attr(bra23, "expansion.fields"), ~ {
  if (.x[1] == "ing4") 
    data.frame(
      note_id = .x[2],
      note_value = .x[3],
      stringsAsFactors = FALSE
    )
})
)


## ----NOTES EXTRACT, eval = TRUE, warnings = F---------------------------------
# Extract AB notes with lpr function
BRA23notes <- lpr_extract_notes(bra23)
head(BRA23notes, n = 10) # columns names
table(BRA23notes$note_id) # note information available!

## ----SET ATTRIBUTES, eval = TRUE, warnings = F, warning=F---------------------
# English
bra23<-lpr_set_attr(bra23, BRA23notes, verbose = F,
                    noteid = "note2", 
                    attribute_name = "qwording_en") 
# Spanish
bra23<-lpr_set_attr(bra23, BRA23notes, verbose = F,
                    noteid = "note1", 
                    attribute_name = "qwording_es")

# Portuguese
bra23<-lpr_set_attr(bra23, BRA23notes, verbose = F,
                    noteid = "note3", 
                    attribute_name = "qwording_pt") 

# Printing languages
attr(bra23$ing4, "qwording_en") # English
attr(bra23$ing4, "qwording_es") # Spanish
attr(bra23$ing4, "qwording_pt") # Portuguese

## ----ROs, eval = TRUE, warnings = F-------------------------------------------
bra23 <- lpr_set_ros(bra23) # Default English

bra23 <- lpr_set_ros(bra23, lang_id = "es", 
                     attribute_name = "respuestas") # Spanish

bra23 <- lpr_set_ros(bra23, lang_id = "pt", 
                     attribute_name = "ROsLabels_pt") # Portuguese

# Printing ROs
attr(bra23$ing4, "roslabel") # English
attr(bra23$ing4, "respuestas") # Spanish
attr(bra23$ing4, "ROsLabels_pt") # Portuguese

# 1. Full CRAN check
unlink("lapop.Rcheck", recursive = TRUE, force = TRUE)
unlink("lapop_2.1.5.tar.gz", force = TRUE)

devtools::document()
Sys.setenv(TMPDIR = tempdir())
devtools::check(args = "--as-cran") # REQUIRED

# 2. Unit tests
devtools::test() # ✅ Strongly Recommended

# 3. Platform checks
urlchecker::url_check()
urlchecker::url_update()

# 4. (Optional) Test coverage
#detach(package:lapop)
#covr::package_coverage() # 🔶 Optional but helpful

# 5. (Optional) Spell check
#spelling::spell_check_package() # 🔶 Optional but helpful
#spelling::update_wordlist()

# 6. Build vignettes and manual
devtools::build_vignettes()
#devtools::build(manual = T) # ✅ Manual PDF if submitting offline

# 7. Build tar.gz source package
devtools::build() # ✅ REQUIRED for submission

# 8. (Optional) Build pkgdown site
pkgdown::clean_site();
pkgdown::build_site() # ❌ Not needed for CRAN

# Final reminders
cat("Check DESCRIPTION, NEWS.md, cran-comments.md\n")

# 1. Full CRAN check
unlink("lapop.Rcheck", recursive = TRUE, force = TRUE)
unlink("lapop_2.1.6.tar.gz", force = TRUE)

devtools::document()
Sys.setenv(TMPDIR = tempdir())
devtools::check(args = "--as-cran") # REQUIRED

# 2. Unit tests
devtools::test() # ✅ Strongly Recommended

# 3. Platform checks
urlchecker::url_check()
urlchecker::url_update()

# 4. (Optional) Test coverage
#detach(package:lapop)
#covr::package_coverage() # 🔶 Optional but helpful

# 5. (Optional) Spell check
#spelling::spell_check_package() # 🔶 Optional but helpful
#spelling::update_wordlist()

# 6. Build vignettes and manual
devtools::build_vignettes()
#devtools::build(manual = T) # ✅ Manual PDF if submitting offline

# 7. Build tar.gz source package
devtools::build() # ✅ REQUIRED for submission

# 8. (Optional) Build pkgdown site
pkgdown::clean_site();
pkgdown::build_site() # ❌ Not needed for CRAN

# Final reminders
cat("Check DESCRIPTION, NEWS.md, cran-comments.md\n")

## ----fig setup, include=FALSE-------------------------------------------------

knitr::opts_chunk$set(echo = TRUE, message = FALSE, warning = FALSE)
knitr::opts_chunk$set(fig.width = 9, fig.height = 5)


## ----lapop package------------------------------------------------------------
library(lapop)

## ----load data----------------------------------------------------------------
# Single-country Single-year AmericasBarometer 2023 Brazil
data(bra23)

# AmericasBarometer 2006-2023 Brazil Merge
data(cm23)

# AmericasBarometer 2023 Year Merge
data(ym23)

## ----weight datasets, eval = TRUE---------------------------------------------
# Single-Country Single-Year
bra23w <- lpr_data(bra23, wt = TRUE)

print(bra23w)

# Single-Country Multi-Year
cm23w <- lpr_data(cm23)

print(cm23w)

# Multi-Country Single-Year
ym23w <- lpr_data(ym23)

print(ym23w)

## ----lapop fonts, eval=F------------------------------------------------------
# lapop::lapop_fonts() # Re-run font registration explicitly if needed

## ----time series, eval = TRUE, fig.alt = "Support for democracy decline a decade ago and remains comparatively low in Brazil"----
fig1.1_data <- lpr_ts(cm23w, outcome = "ing4", rec = c(5, 7), use_wave = TRUE)

lapop_ts(fig1.1_data,
         ymin = 50,
         ymax = 80,
         main_title = "Support for democracy decline a decade ago and remains comparatively low in Brazil",
         subtitle = "% who support democracy")

## ----cross country, eval = TRUE, fig.alt = "In many countries, only about one in two adults support democracy"----
fig1.2_data <- lpr_cc(ym23w, outcome = "ing4", rec = c(5, 7))

lapop_cc(fig1.2_data,
         main_title = "In many countries, only about one in two adults support democracy",
         subtitle = "% who support democracy")

## ----mline, eval = TRUE, warnings=F, fig.alt = "Trust in executives has declined to a level similar to other political institutions"----
fig2.1_data <- lpr_mline(cm23w,
                         outcome = c("b13", "b21", "b31"),
                         rec = c(5, 7), rec2 = c(5, 7), rec3 = c(5, 7))

# Changing legend for readibility
fig2.1_data$varlabel <- ifelse(fig2.1_data$varlabel == "b13", "National Legislature", 
                          ifelse(fig2.1_data$varlabel=="b21", "Political Parties",
                                 "Supreme Court"))

lapop_mline(fig2.1_data,
            main_title = "Trust in executives has declined to a level similar to other \npolitical institutions",
            subtitle = "% who trust...")

## ----ccm, warnings = FALSE, eval = TRUE, fig.alt = "The public typically reports more trust in the armed forces than the police, but levels vary"----

fig2.3_data <- lpr_ccm(ym23w,
                       outcome_vars = c("b12", "b18"),
                       rec = c(5, 7),
                       rec2 = c(5, 7))

# Changing legend for readibility
fig2.3_data$var <- ifelse(fig2.3_data$var == "b12", "Armed Forces", "National Police")

lapop_ccm(fig2.3_data,
          main_title = "The public typically reports more trust in the armed forces than the police,\nbut levels vary",
          subtitle = "% who trust...")


## ----mover, eval = TRUE, fig.alt = "Satisfaction with democracy is significantly lower among women, those with higher educational attainment, and the middle class"----
library(dplyr)

ym23w$variables <- ym23w$variables %>%
    mutate(
    edrer = as.numeric(edre),
    gender = as.numeric(q1tc_r)
  ) %>%
  mutate(edrer = case_when(
    edrer <= 2 ~ "None/Primary",
    edrer %in% c(3, 4) ~ "Secondary",
    edrer > 4 ~ "Superior",
    TRUE ~ NA_character_
  ), 
  edrer = factor(edrer, levels = c("None/Primary", "Secondary", "Superior")),
  gender = ifelse(q1tc_r == 3, NA, q1tc_r)) %>%
  mutate(gender = case_when(
    gender == 1 ~ "Men",
    gender == 2 ~ "Women",
     TRUE ~ NA_character_
  ))

# Changing legend for readibility 
attributes(ym23w$variables$gender)$label <- "Gender"
attributes(ym23w$variables$edrer)$label <- "Education"
attributes(ym23w$variables$wealth)$label <- "Wealth"

fig_mover <- lpr_mover(ym23w,
                       outcome = "pn4",
                       grouping_vars = c("gender", "edrer", "wealth"),
                       rec = c(1, 2)) 

lapop_mover(fig_mover,
            main_title = "Satisfaction with democracy is significantly lower among women,\nthose with higher educational attainment, and the middle class",
            subtitle = "% who are satisfied with democracy")

## ----histogram, eval = TRUE, fig.alt = "On average in the LAC region, one in three say voting is the best way to influence change"----
spot32_data <- lpr_hist(ym23w, outcome = "vb21n")

# Translating to English
spot32_data$cat <- c("Vote", "Run for\noffice", 
                     "Protest", 
                     "Participate\nin local orgs.", 
                     "Other", 
                     "Change is\nimpossible")

lapop_hist(spot32_data,
           main_title = "On average in the LAC region, one in three say voting is the best way \nto influence change",
           subtitle = "How do you believe you can best influence change?")

## ----dumbell, eval = TRUE, fig.alt = "Among those with emigration intentions, the percentage who say they are very likely to emigrate increased in Nicaragua and Guatemala"----
fig3.5_data <- lpr_dumb(ym23w,
                        outcome = "q14f",
                        rec = c(1, 1),
                        over = c(2018, 2023),
                        xvar = "pais",
                        ttest = TRUE)

# Select Countries
fig3.5_data <- fig3.5_data[c(1:3,5:6), ]

lapop_dumb(fig3.5_data,
           main_title = "Among those with emigration intentions, the percentage who say they are \nvery likely to emigrate increased in Nicaragua and Guatemala",
           subtitle = "% who say it is very likely they will emigrate",
           source = "LAPOP Lab, AmericasBarometer 2018/19 and 2023")

## ----stack, eval= TRUE, fig.alt = "Nicaragua has the highest percentage of individuals with migration intentions, while Haiti has the lowest"----
attributes(ym23w$variables$q14f)$label <- "Migration intentions"

fig3.8_data <- lpr_stack(ym23w,
                         outcome = "q14f",
                         xvar = "pais",
                         order = "hi-lo")

lapop_stack(fig3.8_data,
            xvar = "xvar_label",
            source = "LAPOP Lab, AmericasBarometer 2023",
            main_title = "Nicaragua has the highest percentage of individuals with migration intentions,\nwhile Haiti has the lowest")

## ----reverse, eval = TRUE, warnings=F-----------------------------------------
# For data.frames
cm23$ing4r <- lpr_resc(cm23$ing4, reverse = TRUE, map=TRUE)

# For LPR_DATA() objects
cm23w$variables$ing4r <- lpr_resc(cm23w$variables$ing4, reverse = TRUE, map=TRUE)

## ----fig setup, include=FALSE-------------------------------------------------

knitr::opts_chunk$set(echo = TRUE, message = FALSE, warning = FALSE)
knitr::opts_chunk$set(fig.width = 9, fig.height = 5)


## ----lapop package------------------------------------------------------------
library(lapop)

## ----load data----------------------------------------------------------------
# Single-country Single-year AmericasBarometer 2023 Brazil
data(bra23)

# AmericasBarometer 2006-2023 Brazil Merge
data(cm23)

# AmericasBarometer 2023 Year Merge
data(ym23)

## ----weight datasets, eval = TRUE---------------------------------------------
# Single-Country Single-Year
bra23w <- lpr_data(bra23, wt = TRUE)

print(bra23w)

# Single-Country Multi-Year
cm23w <- lpr_data(cm23)

print(cm23w)

# Multi-Country Single-Year
ym23w <- lpr_data(ym23)

print(ym23w)

## ----lapop fonts, eval=F------------------------------------------------------
# lapop::lapop_fonts() # Re-run font registration explicitly if needed

## ----time series, eval = TRUE, fig.alt = "Support for democracy decline a decade ago and remains comparatively low in Brazil"----
fig1.1_data <- lpr_ts(cm23w, outcome = "ing4", rec = c(5, 7), use_wave = TRUE)

lapop_ts(fig1.1_data,
         ymin = 50,
         ymax = 80,
         main_title = "Support for democracy decline a decade ago and remains comparatively low in Brazil",
         subtitle = "% who support democracy")

## ----cross country, eval = TRUE, fig.alt = "In many countries, only about one in two adults support democracy"----
fig1.2_data <- lpr_cc(ym23w, outcome = "ing4", rec = c(5, 7))

lapop_cc(fig1.2_data,
         main_title = "In many countries, only about one in two adults support democracy",
         subtitle = "% who support democracy")

## ----mline, eval = TRUE, warnings=F, fig.alt = "Trust in executives has declined to a level similar to other political institutions"----
fig2.1_data <- lpr_mline(cm23w,
                         outcome = c("b13", "b21", "b31"),
                         rec = c(5, 7), rec2 = c(5, 7), rec3 = c(5, 7))

# Changing legend for readibility
fig2.1_data$varlabel <- ifelse(fig2.1_data$varlabel == "b13", "National Legislature", 
                          ifelse(fig2.1_data$varlabel=="b21", "Political Parties",
                                 "Supreme Court"))

lapop_mline(fig2.1_data,
            main_title = "Trust in executives has declined to a level similar to other \npolitical institutions",
            subtitle = "% who trust...")

## ----ccm, warnings = FALSE, eval = TRUE, fig.alt = "The public typically reports more trust in the armed forces than the police, but levels vary"----

fig2.3_data <- lpr_ccm(ym23w,
                       outcome_vars = c("b12", "b18"),
                       rec1 = c(5, 7),
                       rec2 = c(5, 7))

# Changing legend for readibility
fig2.3_data$var <- ifelse(fig2.3_data$var == "b12", "Armed Forces", "National Police")

lapop_ccm(fig2.3_data,
          main_title = "The public typically reports more trust in the armed forces than the police,\nbut levels vary",
          subtitle = "% who trust...")


## ----mover, eval = TRUE, fig.alt = "Satisfaction with democracy is significantly lower among women, those with higher educational attainment, and the middle class"----
library(dplyr)

ym23w$variables <- ym23w$variables %>%
    mutate(
    edrer = as.numeric(edre),
    gender = as.numeric(q1tc_r)
  ) %>%
  mutate(edrer = case_when(
    edrer <= 2 ~ "None/Primary",
    edrer %in% c(3, 4) ~ "Secondary",
    edrer > 4 ~ "Superior",
    TRUE ~ NA_character_
  ), 
  edrer = factor(edrer, levels = c("None/Primary", "Secondary", "Superior")),
  gender = ifelse(q1tc_r == 3, NA, q1tc_r)) %>%
  mutate(gender = case_when(
    gender == 1 ~ "Men",
    gender == 2 ~ "Women",
     TRUE ~ NA_character_
  ))

# Changing legend for readibility 
attributes(ym23w$variables$gender)$label <- "Gender"
attributes(ym23w$variables$edrer)$label <- "Education"
attributes(ym23w$variables$wealth)$label <- "Wealth"

fig_mover <- lpr_mover(ym23w,
                       outcome = "pn4",
                       grouping_vars = c("gender", "edrer", "wealth"),
                       rec = c(1, 2)) 

lapop_mover(fig_mover,
            main_title = "Satisfaction with democracy is significantly lower among women,\nthose with higher educational attainment, and the middle class",
            subtitle = "% who are satisfied with democracy")

## ----histogram, eval = TRUE, fig.alt = "On average in the LAC region, one in three say voting is the best way to influence change"----
spot32_data <- lpr_hist(ym23w, outcome = "vb21n")

# Translating to English
spot32_data$cat <- c("Vote", "Run for\noffice", 
                     "Protest", 
                     "Participate\nin local orgs.", 
                     "Other", 
                     "Change is\nimpossible")

lapop_hist(spot32_data,
           main_title = "On average in the LAC region, one in three say voting is the best way \nto influence change",
           subtitle = "How do you believe you can best influence change?")

## ----dumbell, eval = TRUE, fig.alt = "Among those with emigration intentions, the percentage who say they are very likely to emigrate increased in Nicaragua and Guatemala"----
fig3.5_data <- lpr_dumb(ym23w,
                        outcome = "q14f",
                        rec = c(1, 1),
                        over = c(2018, 2023),
                        xvar = "pais",
                        ttest = TRUE)

# Select Countries
fig3.5_data <- fig3.5_data[c(1:3,5:6), ]

lapop_dumb(fig3.5_data,
           main_title = "Among those with emigration intentions, the percentage who say they are \nvery likely to emigrate increased in Nicaragua and Guatemala",
           subtitle = "% who say it is very likely they will emigrate",
           source = "LAPOP Lab, AmericasBarometer 2018/19 and 2023")

## ----stack, eval= TRUE, fig.alt = "Nicaragua has the highest percentage of individuals with migration intentions, while Haiti has the lowest"----
attributes(ym23w$variables$q14f)$label <- "Migration intentions"

fig3.8_data <- lpr_stack(ym23w,
                         outcome = "q14f",
                         xvar = "pais",
                         order = "hi-lo")

lapop_stack(fig3.8_data,
            xvar = "xvar_label",
            source = "LAPOP Lab, AmericasBarometer 2023",
            main_title = "Nicaragua has the highest percentage of individuals with migration intentions,\nwhile Haiti has the lowest")

## ----reverse, eval = TRUE, warnings=F-----------------------------------------
# For data.frames
cm23$ing4r <- lpr_resc(cm23$ing4, reverse = TRUE, map=TRUE)

# For LPR_DATA() objects
cm23w$variables$ing4r <- lpr_resc(cm23w$variables$ing4, reverse = TRUE, map=TRUE)

#######################################

# LAPOP Multiple Cross-Country Bar Graph Pre-Processing #

#######################################

#' LAPOP Grouped Bar Graph Pre-Processing
#'
#' This function creates dataframes which can then be input in lapop_ccm for
#' comparing values for multiple variables across countries with a bar graph
#' using LAPOP formatting.
#'
#' @param data A survey object.  The data that should be analyzed.
#' @param outcome_vars Character vector.  Outcome variable(s) of interest to be plotted
#' across country (or other x variable). Max of 3 (three) variables.
#' @param xvar Character string. Outcome variables are broken down by this variable. You can set
#' xvar to "wave" or "year" for cross-time comparisons. Default: pais_lab.
#' @param rec1,rec2,rec3 Numeric. The minimum and maximum values of the outcome variable that
#' should be included in the numerator of the percentage.  For example, if the variable
#' is on a 1-7 scale and rec1 is c(5, 7), the function will show the percentage who chose
#' an answer of 5, 6, 7 out of all valid answers.  Can also supply one value only,
#' to produce the percentage that chose that value out of all other values.
#' Default: c(1, 1).
#' @param ci_level Numeric. Confidence interval level for estimates.  Default: 0.95
#' @param mean Logical.  If TRUE, will produce the mean of the variable rather than
#' rescaling to percentage.  Default: FALSE.
#' @param filesave Character.  Path and file name to save the dataframe as csv.
#' @param cfmt Character. Changes the format of the numbers displayed above the bars.
#' Uses sprintf string formatting syntax. Default is whole numbers for percentages
#' and tenths place for means.
#' @param sort Character. On what value the bars are sorted.
#' Options are "y" (default; for the value of the first outcome variable), "xv" (for
#' the underlying values of the x variable), "xl" (for the labels of the x variable,
#' i.e., alphabetical).
#' @param order Character.  How the bars should be sorted.  Options are "hi-lo"
#' (default) or "lo-hi".
#' @param ttest Logical.  If TRUE, will conduct pairwise t-tests for difference
#' of means between all outcomes vs. all x-vars and save them in attr(x,
#' "t_test_results"). Default: FALSE.
#' @param keep_nr Logical.  If TRUE, will convert "don't know" (missing code .a)
#' and "no response" (missing code .b) into valid data (value = 99) and use them
#' in the denominator when calculating percentages.  The default is to examine
#' valid responses only.  Default: FALSE.
#'
#' @return Returns a data frame, with data formatted for visualization by lapop_ccm()
#'
#' @examples
#'
#' require(lapop); data(ym23)
#'
#' # Set Survey Context on a small cross-country subset
#' ym23_small <- subset(ym23, pais %in% c(1, 15, 17))
#' ym23lpr <- lpr_data(ym23_small)
#'
#' # Multiple outcomes over countries
#' lpr_ccm(ym23lpr,
#' outcome_vars = c("b12", "b18"),
#' rec1 = c(1, 3),
#' rec2 = c(5, 7))
#'
#' # Multiple outcomes over years
#' \donttest{
#' lpr_ccm(ym23lpr,
#' outcome_vars = c("b12", "b18"),
#' xvar = "wave",
#' rec1 = c(1, 3),
#' rec2 = c(5, 7),
#' ttest = TRUE)
#' }
#'
#'@export
#'@import dplyr
#'@import srvyr
#'
#'@author Luke Plutowski, \email{luke.plutowski@@vanderbilt.edu} & Robert Vidigal, \email{robert.vidigal@@vanderbilt.edu}


lpr_ccm <- function(data,
                    outcome_vars,
                    xvar = "pais_lab",
                    rec1 = c(1, 1),
                    rec2 = c(1, 1),
                    rec3 = c(1, 1),
                    ci_level = 0.95,
                    mean = FALSE,
                    filesave = "",
                    cfmt = "",
                    sort = "y",
                    order = "hi-lo",
                    ttest = FALSE,
                    keep_nr = FALSE) {

  if (length(rec1) == 1) {
    rec1 = c(rec1, rec1)
  }
  if (length(rec2) == 1) {
    rec2 = c(rec2, rec2)
  }
  if (length(rec3) == 1) {
    rec3 = c(rec3, rec3)
  }

  # Map rec arguments to outcome variables
  rec_list <- list(rec1, rec2, rec3)
  rec_map <- purrr::map2(outcome_vars, rec_list[1:length(outcome_vars)], ~ list(var = .x, rec = .y))

  # Handle NA recoding if keep_nr is TRUE
  if (keep_nr) {
    data <- data %>%
      mutate(across(all_of(outcome_vars), ~ case_when(
        na_tag(.) %in% c("a", "b") ~ 99,
        TRUE ~ as.numeric(.)
      )))
  }

  # Process each outcome variable with its respective rec
  ccm <- purrr::map_dfr(rec_map, function(mapping) {
    outcome <- mapping$var
    rec <- mapping$rec

    temp <- data %>%
      drop_na(!!sym(xvar)) %>%
      group_by(pais = as_factor(!!sym(xvar))) %>%
      {
        if (mean) {
          summarize(.,
                    prop = survey_mean(!!sym(outcome),
                                       na.rm = TRUE,
                                       vartype = "ci",
                                       level = ci_level)) %>%
            mutate(proplabel = if (cfmt != "") {
              sprintf(cfmt, prop)
            } else {
              sprintf("%.1f", prop)
            })
        } else {
          summarize(.,
                    prop = survey_mean(between(!!sym(outcome), rec[1], rec[2]),
                                       na.rm = TRUE,
                                       vartype = "ci",
                                       level = ci_level) * 100) %>%
            mutate(proplabel = if (cfmt != "") {
              sprintf(cfmt, round(prop))
            } else {
              sprintf("%.0f%%", round(prop))
            })
        }
      } %>%
      filter(prop != 0) %>%
      rename(lb = prop_low, ub = prop_upp) %>%
      ungroup() %>%
      mutate(var = outcome)

    temp
  })
  # Sorting logic
  ccm = ccm %>%
    {
      if (sort == "y") {
        group_by(., var) %>%
          mutate(rank = rank(-prop)) %>%
          arrange(match(var, unique(var)[1]),
                  if (order == "hi-lo") rank else desc(rank)) %>%
          select(-rank)
      } else if (sort == "xl") {
          arrange(., if (order == "hi-lo") desc(as.character(pais)) else as.character(pais))
      } else if (sort == "xv") {
        arrange(., if (order == "hi-lo") desc(match(pais, levels(pais))) else match(pais, levels(pais)))
      } else {
        .
      }
    }

  # Perform pairwise t-tests if requested
  if (ttest) {
    # Estimate standard error for each row using the confidence intervals
    ccm <- ccm %>%
      mutate(se = (ub - lb) / (2 * 1.96))

    # Initialize an empty data frame to store t-test results
    t_test_results <- data.frame(test = character(),
                                 diff = numeric(),
                                 ttest = numeric(),
                                 pval = numeric(),
                                 stringsAsFactors = FALSE)

    # Perform pairwise t-tests for each combination of rows
    for(i in 1:(nrow(ccm) - 1)) {
      for(j in (i + 1):nrow(ccm)) {
        # Extract values for the two rows being compared
        # adding a comment
        prop1 <- ccm$prop[i]
        se1 <- ccm$se[i]
        prop2 <- ccm$prop[j]
        se2 <- ccm$se[j]

        # Calculate difference, t-statistic, and degrees of freedom
        diff <- prop1 - prop2
        t_stat <- diff / sqrt(se1^2 + se2^2)
        df <- (se1^2 + se2^2)^2 / ((se1^2)^2 / (nrow(data) - 1) + (se2^2)^2 / (nrow(data) - 1))

        # Calculate p-value
        p_value <- 2 * pt(-abs(t_stat), df)

        # Store results in a data frame
        t_test_results <- rbind(t_test_results,
                                data.frame(test = paste(ccm$pais[i], ccm$var[i], "vs",
                                                        ccm$pais[j], ccm$var[j]),
                                           diff = round(diff, 3),
                                           ttest = round(t_stat, 3),
                                           pval = round(p_value, 3)))
        attr(ccm, "t_test_results") <- t_test_results
      }
    }
  }


  # Save the results to a file if specified
  if (filesave != "") {
    write.csv(ccm, filesave, row.names = FALSE)
  }

  return(ccm)
}

######################################################

# LAPOP Histogram Graph Pre-Processing #

######################################################

#' LAPOP Bar/Histogram Graphs
#'
#' This function creates dataframes which can then be input in lapop_hist for
#' showing a bar graph using LAPOP formatting.
#'
#' @param data A survey object.  The data that should be analyzed.
#' @param outcome Character. Outcome variable of interest.
#' @param filesave Character.  Path and file name to save the dataframe as csv.
#' @param cfmt Character. Changes the format of the numbers displayed above the bars.
#' Uses sprintf string formatting syntax. Default is whole numbers.
#' @param sort Character. On what value the bars are sorted.
#' Options are "y" (for the value of the outcome variable), "xv" (default; for
#' the underlying values of the x variable), "xl" (for the labels of the x variable,
#' i.e., alphabetical).
#' @param order Character.  How the bars should be sorted.  Options are "hi-lo"
#' or "lo-hi" (default).
#' @param keep_nr Logical.  If TRUE, will convert "don't know" (missing code .a)
#' and "no response" (missing code .b) into valid data (value = 99).
#' The default is to examine valid responses only.  Default: FALSE.
#'
#' @return Returns a data frame, with data formatted for visualization by lapop_hist
#'
#' @examples
#'
#'require(lapop); data(bra23)
#'
#' # Set Survey Context: single country and year (requires wt = T)
#' bra23lpr <- lpr_data(bra23, wt = TRUE)
#'
#' lpr_hist(bra23lpr,
#' outcome = "ing4",
#' sort = "xv",
#' order = "hi-lo")
#'
#'@export
#'@import dplyr
#'@import srvyr
#'
#'@author Shashwat Dhar \email{shashwat.dhar@@vanderbilt.edu} & Luke Plutowski, \email{luke.plutowski@@vanderbilt.edu}

lpr_hist <- function(data,
                     outcome,
                     filesave = "",
                     cfmt = "",
                     sort = "xv",
                     order = "lo-hi",
                     keep_nr = FALSE) {

  # If keep_nr is TRUE, convert don't knows (NA(a)) and no answers (NA(b)) to
  # non-NA data (a value of 99).
  if (keep_nr) {
    data = data %>%
      mutate(!!sym(outcome) := case_when(
        na_tag(!!sym(outcome)) %in% c("a", "b") ~ 99,
        TRUE ~ as.numeric(!!sym(outcome))
      ))
  }

  # Create the data frame
  hist_df = data %>%
    drop_na(outcome) %>%
    group_by(across(outcome)) %>%  #
    summarise(n = n()) %>%
    rename(cat = 1) %>%
    mutate(
      prop = prop.table(n) * 100,
      proplabel = if (cfmt != "") {
        sprintf(cfmt, round(prop))
      } else {
        sprintf("%.0f%%", round(prop))
      },
      cat = (haven::as_factor(cat))
    ) %>%
    select(-n)

  # Apply sorting
  if (sort == "y") {
    hist_df = hist_df %>%
      arrange(if (order == "lo-hi") prop else desc(prop))
  }

  if (sort == "xv") {
    hist_df = hist_df %>%
      arrange(if (order == "lo-hi") match(cat, levels(cat)) else desc(match(cat, levels(cat))))
  }

  if (sort == "xl") {
    hist_df = hist_df %>%
      arrange(if (order == "lo-hi") as.character(cat) else desc(as.character(cat)))
  }

  if (filesave != "") {
    write.csv(hist_df, filesave)
  }

  # Return the data frame
  return(hist_df)
}

#######################################

# LAPOP Cross-Country Bar Graphs #

#######################################

#' @include lapop_fonts.R
NULL

#' LAPOP Cross-Country Bar Graphs
#'
#' This function creates bar graphs for comparing values across countries using LAPOP formatting.
#'
#' @param data Data Frame. Dataset to be used for analysis.  The data frame should have columns
#' titled vallabel (values of x-axis variable (e.g. pais); character vector), prop (outcome variable; numeric),
#' proplabel (text of outcome variable; character), lb (lower bound of estimate; numeric),
#' and ub (upper bound of estimate; numeric). Default: None (must be supplied).
#' @param vallabel,outcome_var,label_var,lower_bound,upper_bound Character, numeric, character,
#' numeric, numeric. Each component of the plot data can be manually specified in case
#' the default columns in the data frame should not be used (if, for example, the values for a given
#' variable were altered and stored in a new column). x
#' @param ymin,ymax Numeric.  Minimum and maximum values for y-axis. Default: 0 to 100.
#' @param highlight Character.  Country of interest.  Will highlight (make darker) that country's bar.
#' Input must match entry in "vallabel" exactly. Default: None.
#' @param sort Character. Method of sorting bars.  Options: "hi-lo" (highest to lowest y value), "lo-hi" (lowest to highest),
#' "alpha" (alphabetical by vallabel/x-axis label). Default: Order of data frame.
#' @param main_title Character.  Title of graph.  Default: None.
#' @param source_info Character.  Information on dataset used (country, years, version, etc.),
#' which is added to the bottom-left corner of the graph. Default: LAPOP ("Source: LAPOP Lab" will be printed).
#' @param subtitle Character.  Describes the values/data shown in the graph, e.g., "percentage of Mexicans who say...)".
#' Default: None.
#' @param lang Character.  Changes default subtitle text and source info to either Spanish or English.
#' Will not translate input text, such as main title or variable labels.  Takes either "en" (English)
#' or "es" (Spanish).  Default: "en".
#' @param color_scheme Character.  Color of bars.  Takes hex number, beginning with "#".
#' Default: #784885.
#' @param label_size Numeric.  Size of text for data labels (percentages above bars).  Default: 5.
#' @param max_countries Numeric. Threshold for automatic x-axis label rotation. When the number of unique
#' country labels exceeds this value, labels will be rotated for better readability. Default: 20.
#' @param label_angle Numeric. Angle (in degrees) to rotate x-axis labels when max_countries is exceeded. Default: 0.
#'
#' @return Returns an object of class \code{ggplot}, a ggplot figure showing
#' average values of some variables across multiple countries.
#'
#' @examples
#'\donttest{
#' require(lapop); lapop_fonts()
#'
#' df <- data.frame(
#' vallabel = c("PE", "CO", "BR", "PN", "GT", "DO", "MX", "BO", "EC"),
#' prop      = c(36.1, 19.3, 16.6, 13.3, 13.0, 11.1,  9.5,  9.0,  8.1),
#' proplabel = c("36%" ,"19%" ,"17%" ,"13%" ,"13%" ,"11%" ,"10%", "9%", "8%"),
#' lb        = c(34.9, 18.1, 15.4, 12.1, 11.8,  9.9,  8.3,  7.8,  6.9),
#' ub        = c(37.3, 20.5, 17.8, 14.5, 14.2, 12.3, 10.7, 10.2,  9.3)
#' )
#' lapop_cc(df,
#'          main_title = "Normalization of Intimate Partner Violence in LAC Countries",
#'          subtitle = "% who say domestic violence is private matter",
#'          source_info = "LAPOP Lab, AmericasBarometer 2021",
#'          highlight = "PE",
#'          ymax = 50)
#'}
#'@export
#'@import ggplot2
#'@import ggtext
#'@import sysfonts
#'@import showtext
#'
#'@author Luke Plutowski, \email{luke.plutowski@@vanderbilt.edu} & Robert Vidigal, \email{robert.vidigal@@vanderbilt.edu}

lapop_cc <- function(data, outcome_var = data$prop, lower_bound = data$lb, vallabel = data$vallabel,
                     upper_bound = data$ub, label_var = data$proplabel,
                     ymin = 0,
                     ymax = 100,
                     lang = "en",
                     highlight = "",
                     main_title = "",
                     source_info = "LAPOP",
                     subtitle = "",
                     sort = "",
                     color_scheme = "#784885",
                     label_size = 5,  # Default size
                     max_countries = 30,
                     label_angle = 0) {  # Removed label_adjust

  # Check if we need to rotate labels based on number of unique countries
  rotate_labels <- length(unique(data$vallabel)) > max_countries
  # Adjust label size if rotation is needed
  current_label_size <- ifelse(rotate_labels, (label_size * 0.5), label_size)


  if(all(highlight != "")){
    data$hl_var = factor(ifelse(vallabel %in% highlight, 0, 1), labels = c("hl", "other"))
    fill_values = c(paste0(color_scheme, "47"), paste0(color_scheme, "20"))
  }
  else{
    data$hl_var = factor("other")
    fill_values = paste0(color_scheme, "47")
  }

  if(sort == "hi-lo"){
    data = data[order(-data$prop),]
  } else if(sort == "lo-hi"){
    data = data[order(data$prop),]
  } else if(sort == "alpha"){
    data = data[order(data$vallabel),]
  }

  ci_text = ifelse(lang == "es",
                   paste0(" <span style='color:", color_scheme, "; font-size:18pt'> \u0131\u2014\u0131</span> ",
                          "<span style='color:#585860; font-size:13pt'>95% intervalo de confianza </span>"),
                   ifelse(lang == "fr",
                          paste0(" <span style='color:", color_scheme, "; font-size:18pt'> \u0131\u2014\u0131</span> ",
                                 "<span style='color:#585860; font-size:13pt'>Intervalle de confiance de 95% </span>"),
                          paste0(" <span style='color:", color_scheme, "; font-size:18pt'> \u0131\u2014\u0131</span> ",
                                 "<span style='color:#585860; font-size:13pt'>95% confidence </span>",
                                 "<span style='color:#585860'>interval</span>")))

  update_geom_defaults("text", list(family = "inter")) # roboto

  # Create base plot
   cc <- ggplot(data=data, aes(x=factor(vallabel, levels = vallabel), y=prop, fill = hl_var)) +
    geom_bar(stat="identity", color = color_scheme, width = 0.6) +
    geom_text(aes(label=label_var, y = upper_bound), vjust= -0.5,
              size=current_label_size, fontface = "bold", color = color_scheme) +
    geom_errorbar(aes(ymin=lower_bound, ymax=upper_bound), width = 0.15, color = color_scheme, linetype = "solid") +
    scale_fill_manual(breaks = "other",
                      values = fill_values,
                      labels = paste0(" <span style='color:#585860; font-size:13pt'> ",
                                      subtitle,
                                      "<span style='color:#FFFFFF00'>-----------</span>",
                                      ci_text),
                      na.value = paste0(color_scheme, "90")) +
    scale_y_continuous(limits = c(ymin, ymax), expand = expansion(mult = 0.002)) +
    labs(title=main_title,
         y = "",
         x = "",
         caption = paste0(ifelse(lang == "es" & source_info == "LAPOP", "Fuente: LAPOP Lab",
                                 ifelse(lang == "en" & source_info == "LAPOP", "Source: LAPOP Lab",
                                        source_info)))) +
    theme(text = element_text(size = 14, family = "inter"), # roboto
          plot.title = element_text(size = 18, family = "inter", face = "bold"), # nunito
          plot.caption = element_text(size = 10.5, vjust = 2, hjust = 0, family = "inter", color="#585860"), # nunito
          panel.background = element_blank(),
          panel.border = element_blank(),
          axis.line.x = element_line(linewidth = 0.6, linetype = "solid", colour = "#dddddf"),
          axis.text = element_text(size = ifelse(rotate_labels, 10, 14), color = "#585860", face = "bold"),
          axis.text.y = element_blank(),
          axis.ticks = element_blank(),
          legend.position = "top",
          legend.title = element_blank(),
          legend.justification='left',
          legend.margin = margin(t=0, b=0, l=0, r=0),
          legend.text = element_markdown(family = "inter-light")) # nunito-light

  # Apply label rotation if needed
  if(rotate_labels) {
   cc <- cc + theme(axis.text.x = element_text(angle = label_angle,
                                              hjust = 0.5, vjust = 1))

    # Adjust plot margins to accommodate rotated labels
     cc <-  cc + theme(plot.margin = margin(t = 10, r = 10, b = max(10, current_label_size*5), l = 10))
  }

  return(cc)
}

######################################################

# LAPOP "Dumbbell" Graph Pre-Processing #

######################################################

#' LAPOP Dumbbell Graphs
#'
#' This function creates dataframes which can then be input in lapop_dumb for
#' comparing means of a variable across countries and two waves using LAPOP formatting.

#' @param data A survey object.  The data that should be analyzed.
#' @param outcome Outcome variable(s) of interest to be plotted across countries
#' and waves, supplied as a character string or vector of strings.
#' @param xvar Character. The grouping variable to be plotted
#' along the x-axis (technically, the vertical axis for lapop_dumb). Usually
#' country (pais). Default: "pais".
#' @param over Numeric. A vector of values for "wave" that specify which two
#' waves should be included in the plot.
#' @param rec Numeric. The minimum and maximum values of the outcome variable that
#' should be included in the numerator of the percentage.  For example, if the variable
#' is on a 1-7 scale and rec is c(5, 7), the function will show the percentage who chose
#' an answer of 5, 6, 7 out of all valid answers.  Can also supply one value only,
#' to produce the percentage that chose that value out of all other values.
#' Default: c(1, 1).
#' @param ci_level Numeric. Confidence interval level for estimates.  Default: 0.95
#' @param mean Logical.  If TRUE, will produce the mean of the variable rather than
#' recoding to percentage.  Default: FALSE.
#' @param filesave Character.  Path and file name to save the dataframe as csv.
#' @param cfmt Character. Changes the format of the numbers displayed above the bars.
#' Uses sprintf string formatting syntax. Default is whole numbers for percentages
#' and tenths place for means.
#' @param sort Character. On what value the bars are sorted.
#' Options are "prop1" (for the value of the outcome variable in wave 1), "prop2"
#' (default; for the value of the outcome variable in wave 2), "xv" (for
#' the underlying values of the x variable), "xl" (for the labels of the x variable,
#' i.e., alphabetical), and "diff" (for the difference between the outcome between
#' the two waves).
#' @param order Character.  How the bars should be sorted.  Options are "hi-lo"
#' (default) or "lo-hi".
#' @param ttest Logical.  If TRUE, will conduct pairwise t-tests for difference
#' of means between all pais-wave combinations and save them in attr(x,
#' "t_test_results"). Default: FALSE.
#' @param keep_nr Logical.  If TRUE, will convert "don't know" (missing code .a)
#' and "no response" (missing code .b) into valid data (value = 99) and use them
#' in the denominator when calculating percentages.  The default is to examine
#' valid responses only.  Default: FALSE.
#'
#' @return Returns a data frame, with data formatted for visualization by lapop_dumb()
#'
#' @examples
#'
#' require(lapop); data(cm23)
#'
#' # Set Survey Context
#' cm23lpr <- lpr_data(cm23)
#'
#' # Single outcome over years
#' lpr_dumb(cm23lpr,
#' outcome = "ing4",
#' rec = c(5, 7),
#' over = c("2018/19", "2023"),
#' sort = "diff")
#'
#' # Multiple outcomes over years
#' lpr_dumb(cm23lpr,
#' outcome=c("b13","b21", "b31"),
#' rec=c(5,7),
#' over=c("2018/19", "2023"))
#'
#'@export
#'@import dplyr
#'@import srvyr
#'@import tibble
#'
#'@author Luke Plutowski, \email{luke.plutowski@@vanderbilt.edu} & Robert Vidigal, \email{robert.vidigal@@vanderbilt.edu}

lpr_dumb <- function(data,
                     outcome,
                     xvar = "pais",
                     over,
                     rec = c(1, 1),
                     ci_level = 0.95,
                     mean = FALSE,
                     filesave = "",
                     cfmt = "",
                     sort = "prop2",
                     order = "hi-lo",
                     ttest = FALSE,
                     keep_nr = FALSE) {

  if (length(rec) == 1) rec <- c(rec, rec)

  if (length(outcome) > 1) {
    results_list <- lapply(outcome, function(out) {
      tmp <- data
      if (keep_nr) {
        tmp <- tmp %>%
          mutate(!!sym(out) := case_when(
            na_tag(!!sym(out)) %in% c("a", "b") ~ 99,
            TRUE ~ as.numeric(!!sym(out))
          ))
      }

      wave1 <- tmp %>%
        filter(wave == over[1]) %>%
        group_by(wave1 = as.character(as_factor(wave))) %>%
        {
          if (mean) {
            summarize(.,
                      prop1 = survey_mean(!!sym(out), na.rm = TRUE, vartype = "ci", level = ci_level)) %>%
              mutate(proplabel1 = if (cfmt != "") sprintf(cfmt, prop1) else sprintf("%.1f", prop1))
          } else {
            summarize(.,
                      prop1 = survey_mean(between(!!sym(out), rec[1], rec[2]), na.rm = TRUE,
                                          vartype = "ci", level = ci_level) * 100) %>%
              mutate(proplabel1 = if (cfmt != "") sprintf(cfmt, round(prop1)) else sprintf("%.0f%%", round(prop1)))
          }
        } %>%
        rename(lb1 = prop1_low, ub1 = prop1_upp)

      wave2 <- tmp %>%
        filter(wave == over[2]) %>%
        group_by(wave2 = as.character(as_factor(wave))) %>%
        {
          if (mean) {
            summarize(.,
                      prop2 = survey_mean(!!sym(out), na.rm = TRUE, vartype = "ci", level = ci_level)) %>%
              mutate(proplabel2 = if (cfmt != "") sprintf(cfmt, prop2) else sprintf("%.1f", prop2))
          } else {
            summarize(.,
                      prop2 = survey_mean(between(!!sym(out), rec[1], rec[2]), na.rm = TRUE,
                                          vartype = "ci", level = ci_level) * 100) %>%
              mutate(proplabel2 = if (cfmt != "") sprintf(cfmt, round(prop2)) else sprintf("%.0f%%", round(prop2)))
          }
        } %>%
        rename(lb2 = prop2_low, ub2 = prop2_upp)

      full <- merge(wave1, wave2, by = character(0))  # no common vars to merge
      full$pais <- attr(tmp[[out]], "label") %||% out
      return(full)
    })

    dumb <- bind_rows(results_list) %>%
      relocate(pais, wave1, prop1, proplabel1, wave2, prop2, proplabel2)

  } else {
    out <- outcome
    if (keep_nr) {
      data <- data %>%
        mutate(!!sym(out) := case_when(
          na_tag(!!sym(out)) %in% c("a", "b") ~ 99,
          TRUE ~ as.numeric(!!sym(out))
        ))
    }

    wave1 <- data %>%
      drop_na(!!sym(xvar)) %>%
      filter(wave == over[1]) %>%
      group_by(pais = as_factor(!!sym(xvar)),
               wave1 = as.character(as_factor(wave))) %>%
      {
        if (mean) {
          summarize(.,
                    prop1 = survey_mean(!!sym(out), na.rm = TRUE, vartype = "ci", level = ci_level)) %>%
            mutate(proplabel1 = if (cfmt != "") sprintf(cfmt, prop1) else sprintf("%.1f", prop1))
        } else {
          summarize(.,
                    prop1 = survey_mean(between(!!sym(out), rec[1], rec[2]), na.rm = TRUE,
                                        vartype = "ci", level = ci_level) * 100) %>%
            mutate(proplabel1 = if (cfmt != "") sprintf(cfmt, round(prop1)) else sprintf("%.0f%%", round(prop1)))
        }
      } %>%
      rename(lb1 = prop1_low, ub1 = prop1_upp)

    wave2 <- data %>%
      drop_na(!!sym(xvar)) %>%
      filter(wave == over[2]) %>%
      group_by(pais = as_factor(!!sym(xvar)),
               wave2 = as.character(as_factor(wave))) %>%
      {
        if (mean) {
          summarize(.,
                    prop2 = survey_mean(!!sym(out), na.rm = TRUE, vartype = "ci", level = ci_level)) %>%
            mutate(proplabel2 = if (cfmt != "") sprintf(cfmt, prop2) else sprintf("%.1f", prop2))
        } else {
          summarize(.,
                    prop2 = survey_mean(between(!!sym(out), rec[1], rec[2]), na.rm = TRUE,
                                        vartype = "ci", level = ci_level) * 100) %>%
            mutate(proplabel2 = if (cfmt != "") sprintf(cfmt, round(prop2)) else sprintf("%.0f%%", round(prop2)))
        }
      } %>%
      rename(lb2 = prop2_low, ub2 = prop2_upp)

    dumb <- merge(wave1, wave2, by = "pais")
  }

  dumb = dumb %>%
    {
      if (sort == "prop1") {
        if (order == "hi-lo") {
          arrange(., desc(prop1))
        } else if (order == "lo-hi") {
          arrange(., prop1)
        }
      } else if (sort == "prop2") {
        if (order == "hi-lo") {
          arrange(., desc(prop2))
        } else if (order == "lo-hi") {
          arrange(., prop2)
        }
      } else if (sort == "xv") {
        if (order == "hi-lo") {
          arrange(., desc(match(pais, levels(pais))))
        } else if (order == "lo-hi") {
          arrange(., match(pais, levels(pais)))
        }
      } else if (sort == "diff") {
        if (order == "hi-lo") {
          mutate(., diff = prop2 - prop1) %>%
            arrange(., desc(diff))
        } else if (order == "lo-hi") {
          mutate(., diff = prop2 - prop1) %>%
            arrange(., diff)
        }
      } else if (sort == "xl") {
        if (order == "hi-lo") {
          arrange(., desc(as.character(xvar)))
        } else if (order == "lo-hi") {
          arrange(., as.character(xvar))
        } else {
          .  # Return unchanged
        }
      }
    }


  if (ttest) {
    # Compute standard errors
    t_test_results <- dumb %>%
      mutate(se1 = (ub1 - lb1) / (2 * 1.96),
             se2 = (ub2 - lb2) / (2 * 1.96))

    # Initialize an empty dataframe for storing test results
    t_test_results_df <- data.frame(test = character(),
                                    diff = numeric(),
                                    ttest = numeric(),
                                    pval = numeric(),
                                    stringsAsFactors = FALSE)

    # Within-country t-tests: Compare prop1 vs. prop2 for each country
    for (i in 1:nrow(t_test_results)) {
      diff <- round(t_test_results$prop1[i] - t_test_results$prop2[i], 3)
      t_stat <- round(diff / sqrt(t_test_results$se1[i]^2 + t_test_results$se2[i]^2), 3)
      df <- (t_test_results$se1[i]^2 + t_test_results$se2[i]^2)^2 /
        ((t_test_results$se1[i]^4 / (nrow(data) - 1)) + (t_test_results$se2[i]^4 / (nrow(data) - 1)))
      p_value <- round(2 * pt(-abs(t_stat), df), 3)

      t_test_results_df <- rbind(t_test_results_df,
                                 data.frame(test = paste(t_test_results$pais[i], t_test_results$wave1[i], "vs",
                                                         t_test_results$pais[i], t_test_results$wave2[i]),
                                            diff = diff,
                                            ttest = t_stat,
                                            pval = p_value))
    }

    # Pairwise comparisons across all rows for prop1
    for (i in 1:(nrow(t_test_results) - 1)) {
      for (j in (i + 1):nrow(t_test_results)) {
        diff <- round(t_test_results$prop1[i] - t_test_results$prop1[j], 3)
        t_stat <- round(diff / sqrt(t_test_results$se1[i]^2 + t_test_results$se1[j]^2), 3)
        df <- (t_test_results$se1[i]^2 + t_test_results$se1[j]^2)^2 /
          ((t_test_results$se1[i]^4 / (nrow(data) - 1)) + (t_test_results$se1[j]^4 / (nrow(data) - 1)))
        p_value <- round(2 * pt(-abs(t_stat), df), 3)

        t_test_results_df <- rbind(t_test_results_df,
                                   data.frame(test = paste(t_test_results$pais[i], t_test_results$wave1[i], "vs",
                                                           t_test_results$pais[j], t_test_results$wave1[j]),
                                              diff = diff,
                                              ttest = t_stat,
                                              pval = p_value))
      }
    }

    # Pairwise comparisons across all rows for prop2
    for (i in 1:(nrow(t_test_results) - 1)) {
      for (j in (i + 1):nrow(t_test_results)) {
        diff <- round(t_test_results$prop2[i] - t_test_results$prop2[j], 3)
        t_stat <- round(diff / sqrt(t_test_results$se2[i]^2 + t_test_results$se2[j]^2), 3)
        df <- (t_test_results$se2[i]^2 + t_test_results$se2[j]^2)^2 /
          ((t_test_results$se2[i]^4 / (nrow(data) - 1)) + (t_test_results$se2[j]^4 / (nrow(data) - 1)))
        p_value <- round(2 * pt(-abs(t_stat), df), 3)

        t_test_results_df <- rbind(t_test_results_df,
                                   data.frame(test = paste(t_test_results$pais[i], t_test_results$wave2[i], "vs",
                                                           t_test_results$pais[j], t_test_results$wave2[j]),
                                              diff = round(diff, 3),
                                              ttest = round(t_stat, 3),
                                              pval = round(p_value, 3)))
      }
    }

    # Store the results as an attribute
    attr(dumb, "t_test_results") <- t_test_results_df
  }


  if (filesave != "") write.csv(dumb, filesave, row.names = FALSE)
  return(dumb)
}

#######################################

# LAPOP Data Processing #

#######################################

#' LAPOP Data Processing
#'
#' This function takes LAPOP datasets and adds survey features such as sampling
#' design effects, outputting a svy_tbl object that can then be analyzed using
#' lpr_ wrangling commands.
#'
#' @param data_path The path for a AmericasBarometer data or a an existing dataframe.
#' @param wt Logical.  If TRUE, use `wt` (weights only for single-country single-year data)
#' instead of `weight1500` (the default weights for multiple-country and multiple-year data).
#' Default: FALSE.
#'
#' @return Returns a svy_tbl object
#'
#' @examples
#' \donttest{
#' data(bra23)
#' data(cm23)
#'
#' bra23w <- lpr_data(bra23, wt = TRUE)
#' cm23w <- lpr_data(cm23)
#' }
#'
#'@export
#'@import srvyr
#'@import haven
#'@import dplyr
#'
#'@author Luke Plutowski, \email{luke.plutowski@@vanderbilt.edu} & Robert Vidigal, \email{robert.vidigal@@vanderbilt.edu}

lpr_data = function (data_path, wt = FALSE)
{
  # Check if data_path is a path or a data frame
  if (is.character(data_path) && file.exists(data_path)) {
    # If it's a path, read the data
    data <- haven::read_dta(data_path)
    #data <- readstata13::read.dta13(data_path)
  } else if (is.data.frame(data_path)) {
    # If it's already a data frame, use it directly
    data <- data_path
  } else {
    stop("data_path must be a valid file path or a data frame.")
  }

  country_codes_numbers <- c(`1` = "MX", `2` = "GT", `3` = "SV", `4` = "HN",
                     `5` = "NI", `6` = "CR", `7` = "PA", `8` = "CO", `9` = "EC",
                     `10` = "BO", `11` = "PE", `12` = "PY", `13` = "CL", `14` = "UY",
                     `15` = "BR", `17` = "AR", `21` = "DO", `22` = "HT", `23` = "JM",
                     `25` = "TT", `26` = "BZ", `27` = "SR", `28` = "BS", `30` = "GD",
                     `40` = "US", `41` = "CA")

  country_codes_names <- c(
    "Mexico" = "MX", "M\u00e9xico" = "MX",
    "Guatemala" = "GT",
    "El Salvador" = "SV",
    "Honduras" = "HN",
    "Nicaragua" = "NI",
    "Costa Rica" = "CR",
    "Panama" = "PA", "Panam\u00e1" = "PA",
    "Colombia" = "CO",
    "Ecuador" = "EC",
    "Bolivia" = "BO",
    "Peru" = "PE", "Per\u00fa" = "PE",
    "Paraguay" = "PY",
    "Chile" = "CL",
    "Uruguay" = "UY",
    "Brazil" = "BR", "Brasil" = "BR",
    "Argentina" = "AR",
    "Dominican Republic" = "DO", "Rep\u00fablica Dominicana" = "DO",
    "Haiti" = "HT", "Hait\u00ed" = "HT",
    "Jamaica" = "JM",
    "Trinidad and Tobago" = "TT", "Trinidad y Tobago" = "TT",
    "Belize" = "BZ", "Belice" = "BZ",
    "Suriname" = "SR", "Surinam" = "SR",
    "Bahamas" = "BS",
    "Grenada" = "GD",
    "United States" = "US", "Estados Unidos" = "US",
    "Canada" = "CA", "Canad\u00e1" = "CA"
  )

  if (is.factor(data$pais)) {
    # Assume factor of country names
    data$pais_lab <- country_codes_names[as.character(data$pais)]
  } else {
    # Assume numeric codes
    data$pais_lab <- country_codes_numbers[as.character(data$pais)]
  }

  # ensure strata present
  if (!"strata" %in% names(data)) {
    stop("Stratum variable 'strata' not found in dataset.")
  } else {
    data <- data[!is.na(data$upm), ]
  }
  # Check if the required weight variable is present
  if (wt == TRUE) {
    if (!"wt" %in% names(data)) stop("Weight variable 'wt' not found in dataset.")
    datalpr <- data %>% as_survey(ids = upm, strata = strata,
                                weights = wt, nest = TRUE)
  } else {
    if (!"weight1500" %in% names(data)) stop("Weight variable 'weight1500' not found in dataset.")
    datalpr <- data %>% as_survey(ids = upm, strata = strata,
                                weights = weight1500, nest = TRUE)
  }

  return(datalpr)
}

test_that("lapopWelcomeMessage includes package metadata", {
  msg <- lapop:::lapopWelcomeMessage("lapop")

  expect_type(msg, "character")
  expect_length(msg, 1)
  expect_match(msg, "Version:")
  expect_match(msg, "browseVignettes\\('lapop'\\)")
})

test_that("lapop_fonts registers bundled fonts", {
  expect_invisible(lapop_fonts())

  families <- sysfonts::font_families()
  expect_true("inter" %in% families)
  expect_true("inter-light" %in% families)
})

test_that("lapop_save writes svg and png outputs", {
  fig <- ggplot2::ggplot(
    data.frame(x = c("A", "B"), y = c(1, 2)),
    ggplot2::aes(x = x, y = y)
  ) +
    ggplot2::geom_col()

  svg_file <- tempfile(fileext = ".svg")
  png_file <- tempfile(fileext = ".png")

  expect_identical(lapop_save(fig, svg_file, format = "svg"), svg_file)
  expect_true(file.exists(svg_file))
  expect_gt(file.info(svg_file)$size, 0)

  expect_identical(lapop_save(fig, png_file, format = "png"), png_file)
  expect_true(file.exists(png_file))
  expect_gt(file.info(png_file)$size, 0)
})

test_that("lapop_map returns a ggplot object for simple input", {
  skip_if_not_installed("sf")

  map_data <- data.frame(
    code = c("US", "BR", "MX"),
    score = c(37, 52, 61),
    stringsAsFactors = FALSE
  )

  p <- lapop_map(
    data = map_data,
    pais_lab = "code",
    outcome = "score",
    survey = "AmericasBarometer"
  )

  expect_s3_class(p, "ggplot")
})

test_that("lpr_data maps country codes and returns a survey object", {
  toy <- data.frame(
    pais = factor(c("Brazil", "Mexico")),
    upm = c(1, 2),
    strata = c(10, 20),
    weight1500 = c(1.5, 0.5),
    stringsAsFactors = TRUE
  )

  survey_tbl <- lpr_data(toy)

  expect_s3_class(survey_tbl, "tbl_svy")
  expect_identical(survey_tbl$variables$pais_lab, c("BR", "MX"))
})

test_that("lpr_resc rescales and flips labelled vectors", {
  x <- haven::labelled(
    c(1, 2, 3, NA),
    labels = c(Low = 1, Medium = 2, High = 3)
  )

  resc <- lpr_resc(x, min = 0, max = 100)
  flipped <- lpr_resc(x, only_flip = TRUE)

  expect_equal(unname(resc[1:3]), c(0, 50, 100))
  expect_equal(as.numeric(flipped[1:3]), c(3, 2, 1))
  expect_named(attr(flipped, "labels"), c("Low", "Medium", "High"))
})

test_that("lpr_set_attr propagates notes and respects overwrite", {
  dat <- data.frame(
    q1 = 1:2,
    q1_1 = 3:4,
    q2 = 5:6
  )

  notes <- data.frame(
    variable_name = c("q1", "q2"),
    note_id = c("qtext_en", "qtext_en"),
    note_value = c("Question one", "Question two"),
    stringsAsFactors = FALSE
  )

  out <- lpr_set_attr(
    data = dat,
    notes = notes,
    noteid = "qtext_en",
    attribute_name = "qwording_en"
  )

  expect_identical(attr(out$q1, "qwording_en"), "Question one")
  expect_identical(attr(out$q1_1, "qwording_en"), "Question one")
  expect_identical(attr(out$q2, "qwording_en"), "Question two")

  attr(dat$q2, "qwording_en") <- "Keep me"
  out_no_overwrite <- lpr_set_attr(
    data = dat,
    notes = notes,
    noteid = "qtext_en",
    attribute_name = "qwording_en",
    overwrite = FALSE
  )

  expect_identical(attr(out_no_overwrite$q2, "qwording_en"), "Keep me")
})

test_that("lpr_set_ros stores formatted response options", {
  dat <- data.frame(ing4 = c(1, 2, 1))
  attr(dat, "label.table") <- list(
    ing4_en = c("Strongly support" = 1, "Support" = 2, "Missing" = 1000)
  )

  out <- lpr_set_ros(dat, lang_id = "en", attribute_name = "roslabel")

  expect_identical(
    attr(out$ing4, "roslabel"),
    "Response Options: (1) Strongly support (2) Support"
  )
})

#######################################

#   LAPOP Extract Response Options    #

#######################################

#' Extract Response Option (RO) values and texts for all variables into a tidy table.
#'
#' Works with:
#'  (a) dataset-level dictionaries (e.g., attr(data, "label.table") is a list keyed by "<VAR>_<lang>"), or
#'  (b) per-variable attributes (e.g., attr(data[[VAR]], "levels") or factor levels).
#'
#' @param data A data.frame read with readstata13/haven/etc.
#' @param lang_id Language code used in label table names ("en", "es", "pt").
#'   If `NULL` or `""`, auto-detect per variable (dataset-level only). Ignored for per-variable `levels`.
#' @param include_special Logical; if FALSE, drop codes >= 1000 when codes are numeric. Default FALSE.
#' @param restrict_to_present Logical; if TRUE, keep only codes that appear in the data. Default TRUE.
#' @param one_row_per_var Logical; if TRUE, return one row per variable_name with concatenated ROs. Default FALSE.
#' @param pair_sep String used to separate each "(value) answer_text" pair when collapsing. Default " | ".
#' @param attr_name Name of the attribute that stores RO info. Default "label.table".
#'
#' @return
#' If `one_row_per_var = FALSE`: tibble with columns `variable_name`, `value`, `answer_text`.
#' If `one_row_per_var = TRUE`: tibble with columns `variable_name`, `answer_text` (collapsed pairs).
#'
#' @examples
#' toy <- data.frame(
#'   ing4 = c(1L, 2L, 1L),
#'   b12 = c(1L, 2L, NA_integer_)
#' )
#' attr(toy, "label.table") <- list(
#'   ing4_pt = c("Apoia muito" = 1L, "Apoia" = 2L, "NS/NR" = 1000L),
#'   b12_pt = c("Muito" = 1L, "Algo" = 2L, "NS/NR" = 1000L)
#' )
#'
#' lpr_extract_ros(toy, lang_id = "pt")
#' lpr_extract_ros(toy, lang_id = "pt", one_row_per_var = TRUE)
#' 
#'@author Robert Vidigal, \email{robert.vidigal@@vanderbilt.edu}
#'
#' @export
#' @import tibble
lpr_extract_ros <- function(data,
                            lang_id = "en",
                            include_special = FALSE,
                            restrict_to_present = TRUE,
                            one_row_per_var = FALSE,
                            pair_sep = " | ",
                            attr_name = "label.table") {

  # ---------- helpers ----------
  is_nonempty_string <- function(x) !is.null(x) && nzchar(x)

  resolve_label_name <- function(VAR, lang_id, lbl_names) {
    # explicit lang
    if (is_nonempty_string(lang_id)) {
      nm <- paste0(VAR, "_", lang_id)
      return(if (nm %in% lbl_names) nm else NA_character_)
    }
    # exact match
    if (VAR %in% lbl_names) return(VAR)
    # candidates
    candidates <- grep(paste0("^", VAR, "(_[A-Za-z0-9]+)?$"), lbl_names, value = TRUE)
    if (length(candidates) == 1) return(candidates)
    if (length(candidates) > 1) {
      pref <- paste0(VAR, "_en")
      if (pref %in% candidates) return(pref)
      warning(sprintf(
        "Multiple label tables found for '%s': %s. Using '%s'.",
        VAR, paste(candidates, collapse = ", "), candidates[1]
      ))
      return(candidates[1])
    }
    NA_character_
  }

  build_df_from_named_codes <- function(VAR, x) {
    # Expect names = labels, values = codes (numeric/integer)
    tibble::tibble(
      variable_name     = VAR,
      value       = suppressWarnings(as.integer(unname(x))),
      answer_text = as.character(names(x))
    )
  }

  build_df_from_named_labels <- function(VAR, x) {
    # names = codes (as character), values = labels
    tibble::tibble(
      variable_name     = VAR,
      value       = suppressWarnings(as.integer(names(x))),
      answer_text = as.character(unname(x))
    )
  }

  build_df_from_levels <- function(VAR, levs) {
    # Fallback when we only have plain levels (no numeric codes available)
    tibble::tibble(
      variable_name     = VAR,
      value       = seq_along(levs),          # positional codes
      answer_text = as.character(levs)
    )
  }

  # ---------- main pathway selection ----------
  dict_at_data <- attr(data, attr_name)
  dict_is_list <- !is.null(dict_at_data) && is.list(dict_at_data)

  vars <- names(data)

  out <- lapply(vars, function(VAR) {
    # Path A: dataset-level dictionary list
    if (dict_is_list) {
      label_table_name <- resolve_label_name(VAR, lang_id, names(dict_at_data))
      if (!is.na(label_table_name)) {
        lt <- dict_at_data[[label_table_name]]
        if (is.null(lt)) return(tibble::tibble(variable_name = VAR,value = NA_integer_,answer_text = NA_character_ ))

        # Try common shapes:
        if (!is.null(names(lt)) && (is.numeric(lt) || is.integer(lt))) {
          df <- build_df_from_named_codes(VAR, lt)
        } else if (!is.null(names(lt)) && is.character(lt)) {
          df <- build_df_from_named_labels(VAR, lt)
        } else {
          warning(sprintf("Unrecognized label table format for '%s'. Skipping.", label_table_name))
          return(tibble::tibble(variable_name = VAR,value = NA_integer_,answer_text = NA_character_))
        }

      } else {
        # No table for this VAR in dataset-level dict; try per-variable attribute below
        df <- NULL
      }

      # If we got df from dataset-level path, continue post-processing
      if (!is.null(df)) {
        # Filter special codes if numeric
        if (!include_special && is.numeric(df$value)) {
          df <- dplyr::filter(df, .data$value < 1000)
        }

        if (restrict_to_present) {
          vals <- unique(stats::na.omit(data[[VAR]]))
          suppressWarnings(vals <- as.integer(vals))
          df <- dplyr::filter(df, .data$value %in% vals)
        }

        return(df)
      }
    }

    # Path B: per-variable attribute or factor levels
    x <- data[[VAR]]
    x_attr <- attr(x, attr_name)

    if (!is.null(x_attr)) {
      # Several possible shapes:
      if (!is.null(names(x_attr)) && (is.numeric(x_attr) || is.integer(x_attr))) {
        df <- build_df_from_named_codes(VAR, x_attr)
      } else if (!is.null(names(x_attr)) && is.character(x_attr)) {
        df <- build_df_from_named_labels(VAR, x_attr)
      } else if (is.character(x_attr) && attr_name == "levels") {
        df <- build_df_from_levels(VAR, x_attr)
      } else {
        # Last resort: if it's a plain character vector, treat as levels
        if (is.character(x_attr)) {
          df <- build_df_from_levels(VAR, x_attr)
        } else {
          warning(sprintf("Attribute '%s' for '%s' has unsupported format; skipping.", attr_name, VAR))
          return(tibble::tibble(variable_name = VAR,value = NA_integer_,answer_text = NA_character_))
        }
      }
    } else {
      # If attr not present but variable is a factor and attr_name == "levels", use factor levels
      if (attr_name == "levels" && is.factor(x)) {
        df <- build_df_from_levels(VAR, levels(x))
      } else {
        return(tibble::tibble(   variable_name = VAR,   value = NA_integer_,   answer_text = NA_character_ ))
      }
    }

    # Post-processing for per-variable path
    # include_special: we only apply if values are numeric and comparable
    if (!include_special && is.numeric(df$value)) {
      df <- dplyr::filter(df, .data$value < 1000)
    } else if (!include_special && !is.numeric(df$value)) {
      # can't apply the >=1000 rule without numeric codes
      # (silently skip; could message if you prefer)
      df <- df
    }

    if (restrict_to_present) {
      vals <- unique(stats::na.omit(data[[VAR]]))
      suppressWarnings(vals <- as.integer(vals))
      if (all(is.na(vals))) {
        # If we couldn't coerce present values to numeric (e.g., character factors),
        # we skip this filter to avoid dropping everything.
        df <- df
      } else {
        df <- dplyr::filter(df, .data$value %in% vals)
      }
    }

    df
  })

  out <- dplyr::bind_rows(out)

  if (nrow(out) == 0) {
    warning("No ROS found; returning NA placeholders for all variables.")
    out <- tibble(
      variable_name = vars,
      value = NA_integer_,
      answer_text = NA_character_
    )
  }

  out <- dplyr::arrange(out, .data$variable_name, suppressWarnings(as.numeric(.data$value)))

  if (one_row_per_var) {
    out <- out |>
      dplyr::mutate(.pair = paste0("(", .data$value, ") ", .data$answer_text)) |>
      dplyr::group_by(.data$variable_name) |>
      dplyr::summarise(answer_text = paste(.data$.pair, collapse = pair_sep), .groups = "drop") |>
      dplyr::arrange(.data$variable_name)
  }

  out
}

world_sf <- NULL

.lapop_try_autoload_fonts <- function(pkg) {
   tryCatch(
      {
         .lapop_register_fonts(pkg = pkg)
         showtext::showtext_auto(enable = TRUE)
         "LAPOP fonts loaded successfully: Inter and Inter Light."
      },
      error = function(e) NULL
   )
}

.onAttach <- function(lib, pkg, ...){
   font_message <- .lapop_try_autoload_fonts(pkg)
   packageStartupMessage(lapopWelcomeMessage(pkg))
   if (!is.null(font_message)) {
      packageStartupMessage(font_message)
   }
}

# Package startup message
lapopWelcomeMessage <- function(pkg){
   paste("\n",
    "LAPOP Lab R package. Version: ", utils::packageVersion(pkg), " loaded successfully\n",
    "For Americasbarometer datasets, please visit https://www.vanderbilt.edu/lapop/", "\n\n",
    "For documentation and vignettes, run: browseVignettes('lapop') or ?lapop", "\n",
    "To report issues or suggestions, please visit: https://github.com/lapop-central/lapop/issues", "\n",
    "Contact: <robert.vidigal@vanderbilt.edu>\n", sep="")
}

.onLoad <- function(libname, pkgname) {

  rda_path <- system.file("data", "world.rda", package = pkgname)
  if (identical(rda_path, "")) {
    stop("The bundled `world.rda` file could not be found.")
  }

  # Load into a private environment
  env <- new.env(parent = emptyenv())
  load(rda_path, envir = env)   # loads an object named "world"

  # Store the sf object in the package namespace without polluting the global env.
  assign("world_sf", env$world, envir = asNamespace(pkgname))

  invisible(TRUE)
}

#######################################

# LAPOP Set Response Options Language #

#######################################

#' Set Response Option (ROS) labels for variables in AmericasBarometer datasets
#'
#' This function extracts formatted response option labels for AmericasBarometer
#' variables, using label tables stored as attributes. The labels are formatted
#' with their corresponding numeric codes and can be pulled in multiple languages.
#'
#' @param data A data frame loaded using readstata13 containing label.table attributes.
#' @param lang_id Language identifier for the labels ("en" for English,
#'        "es" for Spanish, "pt" for Portuguese). Default is "en" (English).
#' @param attribute_name The name of the attribute where the formatted response
#'        options string will be stored. Default is "roslabel".
#'
#' @return The input data frame with response option labels added to variables as attributes
#'
#' @details
#' The function looks for label tables stored as attributes of the data frame,
#' with names following the pattern "VARNAME_lang_id" (e.g., "ing4_en" for English
#' labels of variable ing4). Each label table should be a named numeric vector where
#' names are the response labels and values are the corresponding codes.
#'
#' Special codes (values >= 1000) are excluded from the response options string.
#'
#' @examples
#'
#' require(lapop); data(bra23)
#'
#' # Apply the function
#' bra23 <- lpr_set_ros(bra23) # Default English
#' bra23 <- lpr_set_ros(bra23, lang_id = "es", attribute_name = "respuestas") # Spanish
#' bra23 <- lpr_set_ros(bra23, lang_id = "pt", attribute_name = "ROsLabels_pt") # Portuguese
#'
#' # View the resulting attribute
#' attr(bra23$ing4, "roslabel")
#' attr(bra23$ing4, "respuestas")
#' attr(bra23$ing4, "ROsLabels_pt")
#'
#' @export

# Define the flexible label formatting function
lpr_set_ros <- function(data, lang_id = "en", attribute_name = "roslabel") {
  for (VAR in names(data)) {
    label_table_name <- paste0(VAR, "_", lang_id)

    # Extract available language codes from label names (e.g., "_pt", "_es", "_en")
    matches <- regexpr("(?<=_)[a-z]{2}$", label_table_name, perl = TRUE)
    lang_suffixes <- regmatches(label_table_name, matches)
    lang_suffixes <- lang_suffixes[matches != -1] # names that do not end in a valid 2-letter language code, c
    # causing regexpr() to return -1 and regmatches() to insert NA when coerced to a character vector.
    lang_suffixes <- lang_suffixes[!is.na(lang_suffixes) & lang_suffixes != ""] # Clean those NAs

    # Warn if requested lang_id not found
    if (!(lang_id %in% lang_suffixes)) {
      warning(sprintf(
        "Language '%s' not found in label tables. Available options: %s",
        lang_id, paste(unique(lang_suffixes), collapse = ", ")
      ))
      return(data)
    }

    # Check if the label table exists for the given language
    if (label_table_name %in% names(attr(data, "label.table"))) {
      label_table <- attr(data, "label.table")[[label_table_name]]
      valid_labels <- label_table[label_table < 1000]  # Exclude special codes

      formatted <- sapply(seq_along(valid_labels), function(i) {
        paste0("(", valid_labels[i], ") ", names(valid_labels)[i])
      })

      # Determine the prefix depending on language
      prefix <- switch(lang_id,
                       en = "Response Options: ",
                       es = "Opciones de Respuesta: ",
                       pt = "Alternativas de Resposta: ",
                       "")  # fallback

      attr(data[[VAR]], attribute_name) <- paste0(prefix, paste(formatted, collapse = " "))
    }
  }

  return(data)
}

########################################

# LAPOP Regression Coefficients Graph Pre-Processing

########################################

#' LAPOP Regression Coefficients Graph Pre-Processing
#'
#' This function creates a data frame which can then be input in lapop_coef() for
#' plotting regression coefficients graph using LAPOP formatting.
#'
#' @param data Survey design data from lpr_data() output.
#' @param outcome Dependent variable for the svyglm regression model. (e.g., "outcome_name"). Only one variable allowed.
#' @param xvar Vector of independent variables for the svyglm regression model (e.g., "xvar1+xvar2+xvar3" and so on). Multiple variables are allowed.
#' @param interact Interaction terms (e.g., "xvar1`*`xvar2 + xvar3`:`xvar4"). Supports `:` and `*` operators for interacting variables. Optional, default is NULL.
#' @param model Model family object for glm. Default is gaussian regression (i.e., "linear"). For a logit model, use model="binomial"
#' @param estimate Character. Graph either the coefficients (i.e., `coef`) or the change in probabilities (i.e., `contrast`). Default is "coef."
#' @param vlabs Character. Rename variable labels to be displayed in the graph produced by lapop_coef(). For instance, vlabs=c("old_varname" = "new_varname").
#' @param omit Character. Do not display coefficients for these independent variables. Default is to display all variables included in the model. To omit any variables you need to include the raw "varname" in the omit argument.
#' @param filesave Character. Path and file name with csv extension to save the dataframe output.
#' @param replace Logical. Replace the dataset output if it already exists. Default is FALSE.
#' @param level Numeric. Set confidence level in numeric values; default is 95 percent.
#'
#' @return Returns a data frame, with data formatted for visualization by lapop_coef
#'
#' @examples
#'
#' require(lapop); data(bra23)
#'
#' # Set Survey Context
#' bra23lpr <- lpr_data(bra23, wt = TRUE)
#'
#' # Example 1: Linear model
#' lpr_coef(data = bra23lpr,
#'  outcome = "ing4",
#'  xvar = "wealth+idio2",
#'  model = "linear",
#'  estimate = "coef")
#'
#' # Example 2: Logit model with contrasts
#' lpr_coef(data = bra23lpr,
#'  outcome = "fs2",
#'  xvar = "wealth+idio2",
#'  model = "binomial",
#'  estimate = "contrast")
#'
#' # Example 3: Interactive linear model
#' lpr_coef(data = bra23lpr,
#'  outcome = "ing4",
#'  xvar = "wealth+idio2",
#'  interact = "wealth*idio2",
#'  model = "linear",
#'  estimate = "coef")
#'
#' # Example 4: Interactive logit model
#' lpr_coef(data = bra23lpr,
#'          outcome = "fs2",
#'          xvar = "wealth+idio2",
#'          interact = "wealth*idio2",
#'          model = "binomial",
#'          estimate = "contrast")
#'
#'@export
#'@import dplyr
#'@import srvyr
#'@import marginaleffects
#'@importFrom stats Gamma gaussian poisson quasibinomial quasipoisson
#'
#'@author Robert Vidigal, \email{robert.vidigal@@vanderbilt.edu}

lpr_coef <- function(
    outcome  = NULL,
    xvar     = NULL,
    interact = NULL,
    model    = "linear",
    data     = NULL,
    estimate = c("coef"),
    vlabs    = NULL,
    omit     = NULL,
    filesave = NULL,
    replace  = FALSE,
    level    = 95
) {
  # Initialize an empty data frame
  coef_data <- data.frame()

  # Mapper for model family
  model <- model
  if (is.character(model)) {
    model <- switch(tolower(model),
                  "linear"        = gaussian(),
                  "gaussian"      = gaussian(),
                  "identity"      = gaussian(),
                  "binomial"      = quasibinomial(), # keep strict binomial (warns on non-integer)
                  "logit"         = quasibinomial(),
                  "logistic"      = quasibinomial(),
                  "probit"        = quasibinomial(link = "probit"),
                  "poisson"       = poisson(),
                  "quasipoisson"  = quasipoisson(),
                  "gamma"         = Gamma(link = "log"),
                  stop("Unknown model to git: ", model)
    )
  }

  # Basic checks
  if (is.null(outcome) || is.null(xvar) || is.null(model) || is.null(data)) {
    stop("Error: One or more required inputs (outcome, xvar, model, data) are NULL.")
  }

  # Process interaction terms
  if (!is.null(interact)) {
    # Normalize interaction syntax: convert * to : to avoid redundancy
    interact_clean <- gsub("\\*", ":", interact)
    interact_str   <- paste(interact_clean, collapse = "+")
    full_formula   <- paste(outcome, "~", paste(xvar, interact_str, sep = "+"))
  } else {
    full_formula   <- paste(outcome, "~", xvar)
  }

  # Fit model
  formula <- as.formula(full_formula)
  svyglm_object <- tryCatch({
    survey::svyglm(formula, design = data, family = model)
  }, error = function(e) {
    stop("Error in svyglm: ", e$message)
  })

  # Estimate type validation
  if (estimate == "") {
    stop("You need to define the type of estimate: coef or contrast")
  }

  if (estimate == "contrast") {
    # Get predictors from model
    predictors <- attr(svyglm_object$terms, "term.labels")
    if (length(predictors) == 0) {
      stop("No valid predictor variables found in the model.")
    }

    # Loop over predictors to compute marginal effects
    for (Term in predictors) {
      dataset <- svyglm_object$data
      if (!Term %in% colnames(dataset)) {
        next # Skip if term isn't in the data directly (e.g., interaction)
      }

      comparison_type <- if (is.factor(dataset[[Term]])) "reference" else "minmax"

      term_data <- tryCatch({
        term_result <- suppressWarnings(
          as.data.frame(
            avg_comparisons(
              svyglm_object,
              variables = setNames(list(comparison_type), Term)
            )
          )
        )

        term_result <- term_result %>%
          rename(
            varlabel = contrast,
            coef     = estimate,
            ub       = conf.high,
            lb       = conf.low
          ) %>%
          mutate(
            varterm   = Term,
            varlabel  = paste(Term, varlabel, sep = ": "),
            pvalue    = round(p.value, 4),
            proplabel = round(coef, 2)
          )

        term_result
      }, error = function(e) {
        data.frame(
          varterm   = NA,
          varlabel  = NA,
          coef      = NA,
          lb        = NA,
          ub        = NA,
          pvalue    = NA,
          proplabel = NA
        )
      })

      coef_data <- bind_rows(coef_data, term_data)
    }

    # Clean and relabel
    if (!is.null(omit)) {
      coef_data <- coef_data %>% filter(!varterm %in% omit)
    }
    if (!is.null(vlabs)) {
      coef_data <- coef_data %>%
        mutate(varlabel = ifelse(varterm %in% names(vlabs), vlabs[varterm], varlabel))
    }

    coef_data <- coef_data %>%
      select(varlabel, coef, lb, ub, pvalue, proplabel)

  } else {
    # Coefficient-based summary
    level <- level / 100
    coef_raw <- summary(svyglm_object)$coefficients

    if (nrow(coef_raw) == 0) {
      stop("No coefficients returned from model. Check for convergence issues or empty formula.")
    }

    coef_data <- as.data.frame(coef_raw) %>%
      mutate(Term = rownames(coef_raw)) %>%
      select(Term, everything()) %>%
      mutate(
        lb         = Estimate - qt(1 - (1 - level) / 2, df = svyglm_object$df.residual) * `Std. Error`,
        ub         = Estimate + qt(1 - (1 - level) / 2, df = svyglm_object$df.residual) * `Std. Error`,
        `Pr(>|t|)` = as.numeric(format(`Pr(>|t|)`, scientific = FALSE, digits = 3)),
        pvalue     = round(`Pr(>|t|)`, 3),
        proplabel  = round(Estimate, 3)
      )

    if (!is.null(omit)) {
      coef_data <- coef_data %>% filter(!Term %in% omit)
    }

    coef_data <- coef_data %>%
      rename(varlabel = Term, coef = Estimate) %>%
      select(varlabel, coef, lb, ub, pvalue, proplabel)

    if (!is.null(vlabs)) {
      coef_data <- coef_data %>%
        mutate(varlabel = ifelse(varlabel %in% names(vlabs), vlabs[varlabel], varlabel))
    }
  }

  # File output
  if (!is.null(filesave)) {
    if (dir.exists(filesave)) {
      stop("You provided a directory for 'filesave'. Please provide a valid file path, including the file name.")
    }
    if (file.exists(filesave) && !replace) {
      stop("File already exists. Use replace = TRUE to overwrite.")
    } else {
      write.csv(coef_data, file = filesave, row.names = FALSE)
      message("File saved to ", filesave)
    }
  } else {
    message("CSV output file not created.")
  }

  return(coef_data)
}

##############################################################

# LAPOP Set Variable Attributes from AmericasBarometer Notes #

##############################################################

#' Set Variable Attributes from AmericasBarometer Notes (with propagation)
#'
#' Applies notes stored in a data frame as attributes to variables in `data`.
#' If a variable has expanded children (e.g., vb20_1, vb20_2), the attribute
#' is propagated to all of them by default.
#'
#' @param data data.frame with variables to annotate
#' @param notes data.frame with columns variable_name, note_id, note_value
#' @param noteid character scalar; which note_id to use (e.g., "qtext_en")
#' @param attribute_name character scalar; attribute name to set (e.g., "qwording_en")
#' @param verbose logical; if TRUE it prints all variables notes available but not found in data
#' @param propagate logical; if TRUE, also set on <varname>_* children.
#' Useful for nominal variables or multiple response options variables. Default TRUE.
#' @param overwrite logical; if FALSE, do not overwrite existing attribute on a variable. Default TRUE.
#'
#' @return The input data frame with attributes applied (i.e., question wording)
#'
#' @export

lpr_set_attr <- function(data,
                         notes,
                         noteid = character(),
                         attribute_name = character(),
                         verbose = FALSE,
                         propagate = TRUE,
                         overwrite = TRUE) {

  # basic checks
  req_cols <- c("variable_name", "note_id", "note_value")
  miss <- setdiff(req_cols, names(notes))
  if (length(miss)) stop("`notes` is missing columns: ",
                         paste(miss, collapse=", "))

  note_subset <- notes[notes$note_id == noteid & !is.na(notes$note_value),
                       c("variable_name","note_value")]
  if (!nrow(note_subset)) return(data)

  # escape regex metacharacters in base var names
  .escape <- function(x) gsub("([\\^\\$\\.\\|\\(\\)\\[\\]\\{\\}\\+\\*\\?\\\\-])",
                              "\\\\\\1", x, perl = TRUE)

  for (i in seq_len(nrow(note_subset))) {
    base_var <- as.character(note_subset$variable_name[i])
    text_label <- note_subset$note_value[i]

    if (propagate) {
      # match exact varname base or base_...
      pat <- paste0("^", .escape(base_var), "($|_)")
      targets <- grep(pat, names(data), value = TRUE, perl = TRUE)
    } else {
      targets <- intersect(base_var, names(data))
    }

    if (!length(targets) & verbose == TRUE) {
      message(
        sprintf(
          "Variable %s not found in data (no base or expanded matches) but notes are available.",
          base_var
        )
      )
      next
    }

    for (nm in targets) {
      if (!overwrite && !is.null(attr(data[[nm]], attribute_name))) next
      attr(data[[nm]], attribute_name) <- text_label
    }
  }

  data
}

#' bra23: Single-country Single-year Dataset
#'
#' A dataset containing the AmericasBarometer Brazil 2023 survey round.
#'
#' @format A data frame
#' \describe{
#' \item{ing4}{Support for Democracy}
#' \item{b12}{Trust in Armed Forces}
#' \item{b13}{Trust in the National Legislature}
#' \item{b21}{Trust in Political Parties}
#' \item{b31}{Trust in the Supreme Court of Justice}
#' \item{fs2}{Food security}
#' \item{idio2}{Personal economic situation}
#' \item{wealth}{Wealth}
#' \item{wave}{Survey round year for regional or multi-country data}
#' \item{pais}{Country of survey}
#' \item{year}{Survey round year for single-country data}
#' \item{upm}{Primary Sampling Unit}
#' \item{strata}{Stratification}
#' \item{wt}{Country-specific post stratification weight}
#' }
#' @source LAPOP AmericasBarometer (https://www.vanderbilt.edu/lapop/)
#' @export
bra23 <- load("data/bra23.rda")

#######################################

# LAPOP Multi-Line Time Series Graph Pre-Processing #

#######################################

#' LAPOP Multi-Line Time Series Graph Pre-Processing
#'
#' This function creates a dataframe which can then be input in lapop_mline for
#' to show a time series plot with multiple lines.  If one "outcome" variable and an
#' `xvar` variable is supplied, the function produces the values of a single outcome
#' variable, broken down by a secondary variable, across time.  If multiple outcome
#' variables (up to four) are supplied, it will show means/percentages of those
#' variables across time (essentially, it allows you to do lpr_ts for multiple variables).
#'
#' @param data A survey object.  The data that should be analyzed.
#' @param outcome Character vector.  Outcome variable(s) of interest to be plotted
#' across time.  If only one value is provided, the graph will show the outcome
#' variable, over time, broken down by a secondary variable (x-var).
#' If more than one value is supplied, the graph will show each outcome variable
#' across time (no secondary variable).
#' @param rec,rec2,rec3,rec4 Numeric. The minimum and maximum values of the outcome
#' variable that should be included in the numerator of the percentage.
#' For example, if the variable is on a 1-7 scale and rec is c(5, 7), the
#' function will show the percentage who chose an answer of 5, 6, 7 out of
#' all valid answers.  Can also supply one value only, to produce the percentage
#' that chose that value out of all other values. Default: c(1, 1).
#' @param xvar Character. Variable on which to break down the outcome variable.
#' In other words, the line graph will produce multiple lines for each value of
#' xvar (technically, it is the z-variable, not the x variable, which is year/wave).
#' Ignored if multiple outcome variables are supplied.
#' @param use_wave Logical.  If TRUE, will use "wave" for the x-axis; otherwise,
#' will use "year".  Default: FALSE.
#' @param use_cat Logical. If TRUE, will show the percentages of category values
#' of a single variable; otherwise will show percentages of the range of values
#' from rec(). Default FALSE.
#' @param ci_level Numeric. Confidence interval level for estimates.  Default: 0.95
#' @param mean Logical.  If TRUE, will produce the mean of the variable rather than
#' rescaling to percentage.  Default: FALSE.
#' @param filesave Character.  Path and file name to save the dataframe as csv.
#' @param cfmt Character. changes the format of the numbers displayed above the bars.
#' Uses sprintf string formatting syntax. Default is whole numbers for percentages
#' and tenths place for means.
#' @param ttest Logical.  If TRUE, will conduct pairwise t-tests for difference
#' of means between all individual x levels and save them in attr(x,
#' "t_test_results"). Default: FALSE.
#' @param keep_nr Logical.  If TRUE, will convert "don't know" (missing code .a)
#' and "no response" (missing code .b) into valid data (value = 99) and use them
#' in the denominator when calculating percentages.  The default is to examine
#' valid responses only.  Default: FALSE.
#'
#' @return Returns a data frame, with data formatted for visualization by lapop_mline
#'
#' @examples
#'\donttest{
#'require(lapop); data(ym23)
#'
#'# Set Survey Context
#'ym23lpr <- lpr_data(ym23)
#'
#' # Single Variable by Country and Year
#' lpr_mline(ym23lpr,
#' outcome = "ing4",
#' rec = c(5, 7),
#' xvar = "pais",
#' use_wave = TRUE)
#'
#' # Multiple Variables
#' lpr_mline(ym23lpr,
#' outcome = c("b12", "b18"),
#' rec = c(5, 7),
#' rec2 = c(1, 2),
#' rec3 = c(5, 7),
#' use_wave = TRUE)
#'
#' # Binary Single Variable by Category
#' lpr_mline(ym23lpr,
#' outcome = "pn4",
#' use_cat = TRUE,
#' use_wave = TRUE)
#'
#' # Recode Categorical Variable (max 4-categories)
#' lpr_mline(ym23lpr,
#' outcome = "ing4",
#' rec = c(5, 7),
#' use_cat = TRUE,
#' use_wave = TRUE)
#'}
#'@export
#'@import dplyr
#'@import srvyr
#'
#'@author Luke Plutowski, \email{luke.plutowski@@vanderbilt.edu} & Robert Vidigal, \email{robert.vidigal@@vanderbilt.edu}
lpr_mline <- function(data,
                      outcome,
                      rec = c(1, 1),
                      rec2 = c(1, 1),
                      rec3 = c(1, 1),
                      rec4 = c(1, 1),
                      xvar,
                      use_wave = FALSE,
                      use_cat = FALSE,
                      ci_level = 0.95,
                      mean = FALSE,
                      filesave = "",
                      cfmt = "",
                      ttest = FALSE,
                      keep_nr = FALSE) {

  # 1. Input validation
  if (!all(outcome %in% names(data$variables))) {
    missing_vars <- setdiff(outcome, names(data$variables))
    stop(paste("Variable(s) not found in data:", paste(missing_vars, collapse = ", ")))
  }

  if (length(rec) == 1) rec <- c(rec, rec)
  if (length(rec2) == 1) rec2 <- c(rec2, rec2)
  if (length(rec3) == 1) rec3 <- c(rec3, rec3)
  if (length(rec4) == 1) rec4 <- c(rec4, rec4)

  # 2. Store variable labels for all outcomes
  var_labels <- sapply(outcome, function(x) {
    lbl <- attr(data$variables[[x]], "label")
    if (is.null(lbl)) x else lbl
  })

  # 3. Handle categorical case
  if (use_cat) {
    if (length(outcome) > 1) stop("When use_cat=TRUE, only one outcome variable should be provided")

    # Get value labels for categories
    val_labels <- attr(data$variables[[outcome]], "labels")

    # Get categories from rec range or data
    categories <- if (!identical(rec, c(1,1))) {
      rec[1]:rec[2]
    } else {
      sort(unique(na.omit(data$variables[[outcome]])))
    }

    if (!keep_nr) categories <- categories[categories != 99]
    if (length(categories) > 4) stop("Variable has more than 4 categories. Consider recoding.")

    # Create rec_list based on categories
    rec_list <- lapply(categories, function(x) c(x, x))
    outcome_rep <- rep(outcome, length(categories))
    rec <- rec_list[[1]]
    if (length(rec_list) > 1) rec2 <- rec_list[[2]]
    if (length(rec_list) > 2) rec3 <- rec_list[[3]]
    if (length(rec_list) > 3) rec4 <- rec_list[[4]]
  }

  # 4. Process data
  if (length(outcome) > 1 || use_cat) {
    rec_list <- if (use_cat) list(rec, rec2, rec3, rec4)[1:length(categories)]
else list(rec, rec2, rec3, rec4)[1:length(outcome)]

result_list <- lapply(seq_along(if (use_cat) categories else outcome), function(i) {
  temp <- lpr_ts(data = data,
                 outcome = if (use_cat) outcome else outcome[i],
                 rec = rec_list[[i]],
                 use_wave = use_wave,
                 mean = mean,
                 cfmt = cfmt,
                 keep_nr = keep_nr)

  if (use_cat) {
    temp$category <- categories[i]
  } else {
    temp$varlabel <- var_labels[i]
  }
  temp
})

mline <- bind_rows(result_list)

# Handle waves
if (nrow(mline) > 0) {
  all_waves <- unique(mline$wave)
  join_by <- if (use_cat) c("wave", "category") else c("wave", "varlabel")

  full_grid <- expand.grid(
    wave = all_waves,
    if (use_cat) unique(mline$category) else unique(mline$varlabel),
    stringsAsFactors = FALSE
  )
  names(full_grid)[2] <- if (use_cat) "category" else "varlabel"

  mline <- full_grid %>%
    left_join(mline, by = join_by) %>%
    arrange(if (use_cat) category else varlabel, wave)

  # Filter combined waves
  if (any(c("2016","2017") %in% mline$wave)) mline <- filter(mline, wave != "2016/17")
  if (any(c("2018","2019") %in% mline$wave)) mline <- filter(mline, wave != "2018/19")
}

# Apply category labels if use_cat
if (use_cat && exists("val_labels")) {
  mline <- mline %>%
    mutate(varlabel = ifelse(
      category %in% val_labels,
      names(val_labels)[match(category, val_labels)],
      as.character(category)
    ))
}

} else {
  # Single outcome case
  if (keep_nr) {
    data <- data %>%
      mutate(!!outcome := case_when(
        na_tag(!!outcome) == "a" | na_tag(!!outcome) == "b" ~ 99,
        TRUE ~ as.numeric(!!outcome)
      ))
  }

  mline <- data %>%
    drop_na(!!sym(xvar)) %>%
    group_by(varlabel = as_factor(!!sym(xvar)),
             wave = if (use_wave) as.character(as_factor(wave)) else year) %>%
    {
      if (mean) {
        summarize(., prop = survey_mean(!!sym(outcome),
                                        na.rm = TRUE,
                                        vartype = "ci",
                                        level = ci_level)) %>%
          mutate(proplabel = sprintf(if (cfmt != "") cfmt else "%.1f", prop))
      } else {
        summarize(., prop = survey_mean(between(!!sym(outcome), rec[1], rec[2]),
                                        na.rm = TRUE,
                                        vartype = "ci",
                                        level = ci_level) * 100) %>%
          mutate(proplabel = sprintf(if (cfmt != "") cfmt else "%.0f%%", round(prop)))
      }
    } %>%
    filter(prop != 0) %>%
    rename(lb = prop_low, ub = prop_upp)
}

  # 5. T-tests if requested
  if (ttest && nrow(mline) > 1) {
    mline <- mline %>%
      mutate(se = (ub - lb) / (2 * 1.96))

    t_test_results <- combn(1:nrow(mline), 2, function(pair) {
      i <- pair[1]; j <- pair[2]
      diff <- mline$prop[i] - mline$prop[j]
      se_diff <- sqrt(mline$se[i]^2 + mline$se[j]^2)
      t_stat <- diff/se_diff
      df <- (mline$se[i]^2 + mline$se[j]^2)^2 /
        ((mline$se[i]^4 + mline$se[j]^4)/(nrow(mline)-1))
      p_value <- 2 * pt(-abs(t_stat), df)

      data.frame(
        test = paste(mline$varlabel[i], mline$wave[i], "vs",
                     mline$varlabel[j], mline$wave[j]),
        diff = round(diff, 3),
        t_stat = round(t_stat, 3),
        p_value = round(p_value, 3),
        stringsAsFactors = FALSE
      )
    }, simplify = FALSE) %>% bind_rows()

    attr(mline, "t_test_results") <- t_test_results
  }

  # 6. Save if requested
  if (filesave != "") {
    write.csv(mline, filesave, row.names = FALSE)
  }

  return(mline)
  }

########################################

# LAPOP Stacked Bar Graph Pre-Processing

########################################

#' LAPOP Stacked Bar Graph Pre-Processing
#'
#' This function creates dataframes which can then be input in lapop_stack() for
#' plotting variables categories with a stacked bar graph using LAPOP formatting.
#'
#' @param data  The data that should be analyzed. It requires a survey object from lpr_data() function.
#' @param outcome Vector of variables be plotted.
#' @param xvar Character. Outcome variable will be broken down by this variable. Default is NULL
#' @param sort Character. On what value the bars are sorted: the x or the y.
#' Options are "y" (default; for the value of the outcome variable), "xv" (for
#' the underlying values of the x variable), "xl" (for the labels of the x variable,
#' i.e., alphabetical).
#' @param order Character. How the bars should be sorted. Options are "hi-lo"
#' (default) or "lo-hi".
#' @param filesave Character. Path and file name to save the dataframe as csv.
#' @param keep_nr Logical. If TRUE, will convert "don't know" (missing code .a)
#' and "no response" (missing code .b) into valid data (value = 99) and use them
#' in the denominator when calculating percentages.  The default is to examine
#' valid responses only.  Default: FALSE.
#'
#' @return Returns a data frame, with data formatted for visualization by lapop_stack
#'
#' @examples
#'\donttest{
#' require(lapop); data(ym23)
#'
#' # Set Survey Context
#' ym23lpr<-lpr_data(ym23)
#'
#' # Multiple outcomes stacked
#' lpr_stack(data = ym23lpr,
#' outcome = c("b12", "b18"))
#'
#' # Single outcome over years
#' lpr_stack(data = ym23lpr,
#' outcome = "q14f",
#' xvar="year")
#'}
#'
#'@export
#'@import dplyr
#'@import srvyr
#'@import purrr
#'@import haven
#'
#'@author Robert Vidigal, \email{robert.vidigal@@vanderbilt.edu}

# # -----------------------------------------------------------------------
# LPR_STACK
# # -----------------------------------------------------------------------
lpr_stack <- function(data,
                      outcome,
                      xvar = NULL,
                      sort = "y",
                      order = "hi-lo",
                      filesave = "",
                      keep_nr = FALSE) {

  # Helper function to handle a single variable
  process_outcome <- function(data, outcome_var) {
    outcome_sym <- sym(outcome_var)

    # Handle `keep_nr` logic
    if (keep_nr) {
      data <- data %>%
        mutate(!!outcome_sym := case_when(
          na_tag(!!outcome_sym) %in% c("a", "b") ~ 99,
          TRUE ~ as.numeric(!!outcome_sym)
        ))
    }

    # Perform proportion calculations
    stack <- data %>%
      drop_na(!!outcome_sym) %>%
      {
        if (!is.null(xvar)) {
          group_by(., xvar_label = as_factor(!!sym(xvar)), vallabel = as_factor(!!outcome_sym))
        } else {
          group_by(., vallabel = as_factor(!!outcome_sym))
        }
      } %>%
      summarise(
        prop = survey_mean(proportion = TRUE),
        .groups = "drop"
      ) %>%
      mutate(
        varlabel = attributes(data$variables[[outcome_var]])$label,
        prop = prop * 100,
        proplabel = sprintf("%.0f%%", prop)
      ) %>%
      {
        if (!is.null(xvar)) {
          select(., varlabel, vallabel, xvar_label, prop, proplabel)
        } else {
          select(., varlabel, vallabel, prop, proplabel)
        }
      }

    # Sorting logic: prioritize sorting by `xvar_label` if provided
    stack <- stack %>%
      {
        if (!is.null(xvar)) {
          arrange(., xvar_label, desc(prop)) # First sort by xvar_label, then by prop
        } else {
          if (sort == "y") {
            if (order == "hi-lo") {
              arrange(., desc(prop))
            } else if (order == "lo-hi") {
              arrange(., prop)
            } else {
              .
            }
          } else if (sort == "xv") {
            if (order == "hi-lo") {
              arrange(., desc(vallabel))
            } else if (order == "lo-hi") {
              arrange(., vallabel)
            } else {
              .
            }
          } else if (sort == "xl") {
            if (order == "hi-lo") {
              arrange(., desc(as.character(vallabel)))
            } else if (order == "lo-hi") {
              arrange(., as.character(vallabel))
            } else {
              .
            }
          } else {
            .
          }
        }
      }

    return(stack)
  }

  # Apply the purrr helper function to all outcomes and combine the results
  results <- map_dfr(outcome, ~ process_outcome(data, .x))

  # Save to file if required
  if (filesave != "") {
    write.csv(results, filesave, row.names = FALSE)
  }

  return(results)
}

#######################################

# LAPOP Cross-Country Bar Graphs #

#######################################

#' @include lapop_fonts.R
NULL

#' LAPOP Cross-Country Bar Graphs
#'
#' This function creates bar graphs for comparing values across countries using LAPOP formatting.
#' @param data Data Frame. Dataset to be used for analysis.  The data frame should have columns
#' titled pais (values of x-axis variable (usually pais); character vector), prop (outcome variable; numeric),
#' proplabel (text of outcome variable; character), lb (lower bound of estimate; numeric),
#'  ub (upper bound of estimate; numeric), and var (labels of secondary variables; character).
#'  Default: None (must be supplied).
#' @param pais,outcome_var,label_var,lower_bound,upper_bound,var Character, numeric, character,
#' numeric, numeric, character. Each component of the plot data can be manually specified in case
#' the default columns in the data frame should not be used (if, for example, the values for a given
#' variable were altered and stored in a new column).
#' @param ymin,ymax Numeric.  Minimum and maximum values for y-axis. Default: 0 to 100.
#' @param sort Character. Method of sorting bars.  Options: "var1" (highest to lowest on variable 1),
#' "var2" (highest to lowest on variable 2), "var3" (highest to lowest on variable 3),
#' "alpha" (alphabetical along x-axis/pais). Default: Order of data frame.
#' @param main_title Character.  Title of graph.  Default: None.
#' @param source_info Character.  Information on dataset used (country, years, version, etc.),
#' which is added to the end of "Source: " in the bottom-left corner of the graph.
#' Default: None (only "Source: " will be printed).
#' @param subtitle Character.  Describes the values/data shown in the graph, e.g., "percentage of Mexicans who say...)".
#' Default: None.
#' @param y_label Character.  Y-axis label.
#' @param x_label Character.  X-axis label.
#' @param highlight Character.  Country of interest.  Will highlight (make darker) that country's bar.
#' Input must match entry in "vallabel" exactly. Default: None.
#' @param lang Character.  Changes default subtitle text and source info to either Spanish or English.
#' Will not translate input text, such as main title or variable labels.  Takes either "en" (English)
#' or "es" (Spanish).  Default: "en".
#' @param color_scheme Character.  Color of bars.  Takes hex number, beginning with "#".
#' Default: "#784885", "#008381", "#C74E49".
#' @param label_size Numeric.  Size of text for data labels (percentages above bars).  Default: 4.
#' @param text_position Numeric.  Amount that text above error bars should be offset (to avoid overlap).  Default: 0.7
#'
#' @return Returns an object of class \code{ggplot}, a ggplot figure showing
#' average values of some variables across multiple countries.
#'
#' @examples
#' \donttest{
#' require(lapop); lapop_fonts()
#'
#' df <- data.frame(pais = c(rep("HT", 2), rep("PE", 2), rep("HN", 2), rep("CO", 2),
#'              rep("UY", 2), rep("CR", 2), rep("EC", 2), rep("CL", 2),
#'               rep("BR", 2), rep("BO", 2), rep("JA", 2), rep("PN", 2)),
#'               var = rep(c("countfair1", "countfair3"), 3),
#'               prop = c(30, 38, 40, 49, 57, 33, 80, 54, 30, 43, 61, 42,
#'                        38, 54, 74, 61, 50, 34, 48, 34, 72, 41, 58, 57),
#'               proplabel = c("30%", "38%", "40%", "49%", "57%", "33%",
#'                             "80%", "54%", "30%", "43%", "61%", "42%",
#'                             "38%", "54%", "74%", "61%", "50%", "34%",
#'                             "48%", "34%", "72%", "41%", "58%", "57%"),
#'               lb = c(27, 35, 37, 46, 54, 30, 77, 51, 27, 40, 58, 39,
#'                      35, 51, 71, 58, 47, 31, 45, 31, 69, 38, 55, 54),
#'               ub = c(33, 41, 43, 52, 60, 36, 83, 57, 33, 46, 64, 45,
#'                      41, 57, 77, 64, 53, 37, 51, 37, 75, 44, 61, 60))
#'
#' lapop_ccm(df, sort = "var", source_info = ", AmericasBarometer")
#'}
#'@export
#'@import ggplot2
#'@import dplyr
#'@import ggtext
#'@import sysfonts
#'@import showtext
#'
#'@author Luke Plutowski, \email{luke.plutowski@@vanderbilt.edu} & Robert Vidigal, \email{robert.vidigal@@vanderbilt.edu}

lapop_ccm <- function(data,
                      pais = data$pais, outcome_var = data$prop,
                      lower_bound = data$lb, upper_bound = data$ub,
                      label_var = data$proplabel, var = data$var,
                      ymin = 0,
                      ymax = 100,
                      lang = "en",
                      main_title = "",
                      source_info = "",
                      subtitle = "",
                      sort = "",
                      y_label = "",
                      x_label = "",
                      highlight = "",
                      color_scheme = c("#784885", "#008381", "#C74E49"),
                      label_size = 4,
                      text_position = 0.7) {

  fill_colors = paste0(color_scheme, "52")

  # Define highlight logic
  if (highlight != "") {
    data$hl_var <- ifelse(data$pais == highlight, "hl", "other")
  } else {
    data$hl_var <- "other"
  }

  # Compute numeric alpha values (no warning!)
  data$alpha_value <- ifelse(data$hl_var == "hl", 0.6, 0.32)

  # Add language-specific var label customization
  if (lang == "es") {
    data$var <- ifelse(data$var == unique(data$var)[length(unique(data$var))],
                       paste0(data$var,
                              "<span style='color:#FFFFFF00'>-------</span>",
                              "<span style='color:#585860; font-size:18pt'> \u0131\u2014\u0131</span>",
                              "<span style='color:#585860; font-size:13pt'>95% intervalo de confianza </span>"),
                       data$var)
    data$var <- factor(data$var, levels = unique(data$var))
  } else if (lang == "fr") {
    data$var <- ifelse(data$var == unique(data$var)[length(unique(data$var))],
                       paste0(data$var,
                              "<span style='color:#FFFFFF00'>-------</span>",
                              "<span style='color:#585860; font-size:18pt'> \u0131\u2014\u0131</span>",
                              "<span style='color:#585860; font-size:13pt'>Intervalle de confiance de 95% </span>"),
                       data$var)
    data$var <- factor(data$var, levels = unique(data$var))
  } else {
    data$var <- ifelse(data$var == unique(data$var)[length(unique(data$var))],
                       paste0(data$var,
                              "<span style='color:#FFFFFF00'>-------</span>",
                              "<span style='color:#585860; font-size:18pt'> \u0131\u2014\u0131</span>",
                              "<span style='color:#585860; font-size:13pt'>95% confidence interval </span>"),
                       data$var)
    data$var <- factor(data$var, levels = unique(data$var))
  }

  # Sorting logic
  if (sort == "var1") {
    data <- data %>%
      group_by(var) %>%
      mutate(rank = rank(-prop)) %>%
      arrange(var, rank)
  } else if (sort == "var2") {
    data <- data %>%
      group_by(var) %>%
      mutate(rank = rank(-prop)) %>%
      arrange(match(var, unique(var)[2]), rank)
  } else if (sort == "var3") {
    data <- data %>%
      group_by(var) %>%
      mutate(rank = rank(-prop)) %>%
      arrange(match(var, unique(var)[3]), rank)
  } else if (sort == "alpha") {
    data <- data[order(data$pais), ]
  }

  # Apply font
  update_geom_defaults("text", list(family = "inter")) # roboto

  # Create ggplot
  ggplot(data = data,
         aes(x = factor(pais, levels = unique(pais)),
             y = prop,
             fill = var,
             color = var)) +
    geom_bar(aes(alpha = alpha_value), position = "dodge", stat = "identity", width = 0.7) +
    geom_text(aes(label = label_var, y = upper_bound, group = var),
              position = position_dodge(width = text_position),
              vjust = -0.5, size = label_size, fontface = "bold", show.legend = FALSE) +
    geom_errorbar(aes(ymin = lower_bound, ymax = upper_bound),
                  width = 0.15,
                  position = position_dodge(width = 0.7), linetype = "solid", show.legend = FALSE) +
    scale_fill_manual(values = fill_colors) +
    scale_color_manual(values = color_scheme) +
    scale_y_continuous(limits = c(ymin, ymax), expand = expansion(mult = 0.002)) +
    labs(title = main_title,
         y = y_label,
         x = x_label,
         caption = paste0(ifelse(lang == "es", "Fuente: LAPOP Lab", "Source: LAPOP Lab"),
                          source_info)) +
    { if (subtitle != "") labs(subtitle = subtitle) } +
    { if (x_label != "") theme(axis.title.x = element_text(margin = margin(b = 10, t = 10))) } +
    theme(text = element_text(size = 14, family = "inter"), # roboto
          plot.title = element_text(size = 18, family = "inter", face = "bold"), # nunito
          plot.caption = element_text(size = 10.5, vjust = 2, hjust = 0, family = "inter", color = "#585860"), # nunito
          panel.background = element_blank(),
          panel.border = element_blank(),
          axis.line.x = element_line(linewidth = 0.6, linetype = "solid", colour = "#dddddf"),
          axis.text = element_text(size = 14, color = "#585860", face = "bold"),
          axis.text.y = element_blank(),
          axis.ticks = element_blank(),
          legend.position = "top",
          legend.title = element_blank(),
          legend.justification = 'left',
          legend.margin = margin(t = 0, b = 0, l = 0),
          legend.text = ggtext::element_markdown(family = "inter-light")) + guides(alpha = "none") # nunito-light
}

######################################################

# LAPOP "Multiple-Over" Breakdown Graph Pre-Processing #

######################################################

#' LAPOP "Multiple-Over" Breakdown Graphs
#'
#' This function creates a dataframe which can then be input in lapop_mover() for
#' comparing means across values of secondary variable(s) using LAPOP formatting.
#'
#' @param data A survey object. The data that should be analyzed.
#' @param outcome Character. Outcome variable(s) of interest to be plotted across secondary
#' variable(s).
#' @param grouping_vars A character vector specifying one or more grouping variables.
#' For each variable, the function calculates the average of the outcome variable,
#' broken down by the distinct values within the grouping variable(s).
#' @param rec Numeric. The minimum and maximum values of the frst outcome variable that
#' should be included in the numerator of the percentage.  For example, if the variable
#' is on a 1-7 scale and rec is c(5, 7), the function will show the percentage who chose
#' an answer of 5, 6, 7 out of all valid answers.  Can also supply one value only,
#' to produce the percentage that chose that value out of all other values.
#' Default: c(1, 1).
#' @param rec2 Numeric. Similar to 'rec' for the second outcome. Default: c(1, 1).
#' @param rec3 Numeric.  Similar to 'rec' for the third outcome. Default: c(1, 1).
#' @param rec4 Numeric.  Similar to 'rec' for the fourth outcome. Default: c(1, 1).
#' @param ci_level Numeric. Confidence interval level for estimates.  Default: 0.95
#' @param mean Logical.  If TRUE, will produce the mean of the variable rather than
#' recoding to percentage.  Default: FALSE.
#' @param filesave Character.  Path and file name to save the dataframe as csv.
#' @param cfmt Changes the format of the numbers displayed above the bars.
#' Uses sprintf string formatting syntax. Default is whole numbers for percentages
#' and tenths place for means.
#' @param ttest Logical.  If TRUE, will conduct pairwise t-tests for difference
#' of means between all individual year-xvar levels and save them in attr(x,
#' "t_test_results"). Default: FALSE.
#' @param keep_nr Logical.  If TRUE, will convert "don't know" (missing code .a)
#' and "no response" (missing code .b) into valid data (value = 99) and use them
#' in the denominator when calculating percentages.  The default is to examine
#' valid responses only.  Default: FALSE.
#'
#' @return Returns a data frame, with data formatted for visualization by lapop_mover
#'
#' @examples
#'\donttest{
#' require(lapop); data(ym23)
#'
#' # Set SUrvey Context
#' ym23lpr<-lpr_data(ym23)
#'
#' # Single DV
#' lpr_mover(data = ym23lpr,
#'  outcome = "ing4",
#'  grouping_vars = c("q1tc_r", "edre"),
#'  rec = c(5, 7), ttest = FALSE)
#'
#' # Multiple DV
#' lpr_mover(data = ym23lpr,
#' outcome = c("ing4", "pn4"),
#' grouping_vars = c("q1tc_r", "edre"),
#' rec = c(5, 7), rec2 = c(1, 2),
#' ttest = FALSE)
#'
#' # Single DV X Single IV
#' lpr_mover(data = ym23lpr,
#' outcome="ing4",
#' grouping_vars="pn4",
#' rec=c(5,7),
#' ttest = FALSE)
#'
#' # Multiple DV X Single IV
#' lpr_mover(data = ym23lpr,
#' outcome=c("ing4", "pn4"),
#' grouping_vars="edre",
#' rec=c(5,7), rec2=c(1,2),
#' ttest = FALSE)
#'
#' # Multiple DV X Multiple IV
#' lpr_mover(data = ym23lpr,
#' outcome=c("ing4", "pn4"),
#' grouping_vars=c("edre", "q1tc_r"),
#' rec=c(5,7), rec2=c(1,2),
#' ttest = FALSE)
#'}
#'
#'@export
#'@import dplyr
#'@import srvyr
#'@import purrr
#'@import haven
#'
#'@author Luke Plutowski, \email{luke.plutowski@@vanderbilt.edu} && Robert Vidigal, \email{robert.vidigal@@vanderbilt.edu}

lpr_mover <- function(data,
                      outcome,
                      grouping_vars,
                      rec = list(c(1, 1)),
                      rec2 = c(1, 1),
                      rec3 = c(1, 1),
                      rec4 = c(1, 1),
                      ci_level = 0.95,
                      mean = FALSE,
                      filesave = "",
                      cfmt = "",
                      ttest = FALSE,
                      keep_nr = FALSE) {

  if (keep_nr) {
    data <- data %>%
      mutate(across(all_of(outcome), ~ case_when(
        na_tag(.) == "a" | na_tag(.) == "b" ~ 99,
        TRUE ~ as.numeric(.)
      )))
  }

  rec_list <- list(rec, rec2, rec3, rec4)
  rec_list <- rec_list[seq_along(outcome)] # Ensure only as many rec values as outcomes

  # Function to calculate means/proportions for a single outcome and grouping variable
  calculate_means <- function(data, outcome_var, grouping_var, rec_range, single_outcome) {
    if (!(grouping_var %in% names(data$variables))) {
      stop(paste("Grouping variable", grouping_var, "not found in data."))
    }
    if (!(outcome_var %in% names(data$variables))) {
      stop(paste("Outcome variable", outcome_var, "not found in data."))
    }

    data %>%
      filter(!is.na(.data[[grouping_var]])) %>%
      group_by(vallabel = haven::as_factor(.data[[grouping_var]])) %>%
      {
        if (mean) {
          summarize(.,
                    prop = survey_mean(.data[[outcome_var]],
                                       na.rm = TRUE,
                                       vartype = "ci",
                                       level = ci_level)
          ) %>%
            mutate(proplabel = sprintf("%.1f", prop))
        } else {
          summarize(.,
                    prop = survey_mean(between(.data[[outcome_var]], rec_range[1], rec_range[2]),
                                       na.rm = TRUE,
                                       vartype = "ci",
                                       level = ci_level) * 100
          ) %>%
            mutate(proplabel = sprintf("%.0f%%", round(prop)))
        }
      } %>%
      mutate(
        outcome = if (!is.null(attributes(data$variables[[outcome_var]])$label)) {
          attributes(data$variables[[outcome_var]])$label
        } else {
          outcome_var
        },
        varlabel = if (single_outcome) {
          if (!is.null(attributes(data$variables[[grouping_var]])$label)) {
            attributes(data$variables[[grouping_var]])$label
          } else {
            grouping_var
          }
        } else {
          paste(grouping_var, outcome_var, sep = " x ")
        },
        vallabel = as.character(vallabel)
      ) %>%
      rename(lb = prop_low, ub = prop_upp) %>%
      select(outcome, varlabel, vallabel, prop, proplabel, lb, ub)
  }

  single_outcome <- length(outcome) == 1

  # Apply function to each combination of outcome and grouping variable
  mover <- map_dfr(grouping_vars, function(gvar) {
    map2_dfr(outcome, rec_list, ~ calculate_means(data, .x, gvar, .y, single_outcome))
  })

  if (filesave != "") {
    write.csv(mover, filesave)
  }

  # Conduct pairwise t-tests if requested
  if (ttest) {
    mover <- mover %>%
      mutate(se = (ub - lb) / (2 * 1.96))

    t_test_results <- data.frame(
      outcome = character(),
      varlabel = character(),
      test = character(),
      diff = numeric(),
      ttest = numeric(),
      pval = numeric(),
      stringsAsFactors = FALSE
    )

    outcomes <- unique(mover$outcome)
    for (oc in outcomes) {
      mover_subset <- mover %>% filter(outcome == oc)

      varlabels <- unique(mover_subset$varlabel)
      for (vl in varlabels) {
        group_subset <- mover_subset %>% filter(varlabel == vl)

        for (i in 1:(nrow(group_subset) - 1)) {
          for (j in (i + 1):nrow(group_subset)) {
            prop1 <- group_subset$prop[i]
            se1 <- group_subset$se[i]
            prop2 <- group_subset$prop[j]
            se2 <- group_subset$se[j]

            diff <- prop1 - prop2
            t_stat <- diff / sqrt(se1^2 + se2^2)
            df <- (se1^2 + se2^2)^2 / ((se1^2)^2 / (nrow(data) - 1) + (se2^2)^2 / (nrow(data) - 1))

            p_value <- 2 * pt(-abs(t_stat), df)

            t_test_results <- rbind(t_test_results,
                                    data.frame(outcome = oc,
                                               varlabel = vl,
                                               test = paste(group_subset$vallabel[i], "vs", group_subset$vallabel[j]),
                                               diff = round(diff, 3),
                                               ttest = round(t_stat, 3),
                                               pval = round(p_value, 3)))
          }
        }
      }
    }

    attr(mover, "t_test_results") <- t_test_results
  }

  return(mover)
}

#######################################

# LAPOP Multiple-Over/Breakdown Graph #

#######################################

#' @rdname lapop-deprecated
#' @section \code{lapop_demog}:
#' For \code{lapop_demog}, use \code{\link{lapop_mover}}.
#'
#' @export

lapop_demog <- function(data,
                        lang = "en",
                        main_title = "",
                        subtitle = "",
                        source_info = "",
                        rev_values = FALSE,
                        rev_variables = FALSE,
                        subtitle_h_just = 0,
                        ymin = 0,
                        ymax = 100,
                        x_lab_angle = 90,
                        color_scheme = c("#7030A0", "#00ADA9", "#3CBC70", "#7EA03E", "#568424", "#ACB014")){
  .Deprecated("lapop_mover")
  lapop_mover(data = data,
  lang = lang,
  main_title = main_title,
  subtitle = subtitle,
  qword = NULL,
  source_info = source_info,
  rev_values = rev_values,
  rev_variables = rev_variables,
  subtitle_h_just = subtitle_h_just,
  ymin = ymin,
  ymax = ymax,
  x_lab_angle = x_lab_angle,
  color_scheme = color_scheme)
}

#' @include lapop_fonts.R
NULL

#' LAPOP Multiple-Over/Breakdown Graphs
#'
#' This function shows the values of an outcome variable for subgroups of a secondary variable, using LAPOP formatting.
#'
#' @param data Data Frame. Dataset to be used for analysis.  The data frame should have columns
#' titled varlabel (name(s)/label(s) of secondary variable(s); character), vallabel (names/labels of values for secondary variable; character),
#' prop (outcome variable value; numeric), proplabel (text of outcome variable value; character),
#' lb (lower bound of estimate; numeric), and ub (upper bound of estimate; numeric).
#' Default: None (must be provided).
#' @param ymin,ymax Numeric.  Minimum and maximum values for y-axis. Defaults: 0 and 100.
#' @param main_title Character.  Title of graph.  Default: None.
#' @param source_info Character.  Information on dataset used (country, years, version, etc.),
#' which is added to the bottom-left corner of the graph. Default: LAPOP ("Source: LAPOP Lab" will be printed).
#' @param subtitle Character.  Describes the values/data shown in the graph, e.g., "Percent who agree that...".
#' Default: None.
#' @param qword Character.  Describes the question wording shown in the graph, e.g., "Do you agree that...".
#' Default: NULL.
#' @param lang Character.  Changes default subtitle text and source info to either Spanish or English.
#' Will not translate input text, such as main title or variable labels.  Takes either "en" (English)
#' or "es" (Spanish).  Default: "en".
#' @param color_scheme Character.  Color of data points and text for each secondary variable.  Allows up to 6 values.
#' Takes hex numbers, beginning with "#".
#' Default: c("#784885", "#008381", "#c74e49", "#2d708e", "#a43d6a")
#' (purple, teal, green, olive, sap green, pea soup).
#' @param subtitle_h_just Numeric.  Move the subtitle/legend text left (negative numbers) or right (positive numbers).
#' Ranges from -100 to 100.  Default: 0.
#' @param x_lab_angle Numeric.  Angle/orientation of the value labels.  Default: 90.
#' @param rev_variables Logical.  Should the order of the variables be reversed?  Default: FALSE.
#' @param rev_values Logical.  Should the order of the values for each variable be reversed?  Default: FALSE.
#' @return Returns an object of class \code{ggplot}, a ggplot figure showing
#' average values of some variable broken down by one or more secondary variables
#' (commonly, demographic variables).
#'
#' @examples
#'\donttest{
#' df <- data.frame(varlabel = c(rep("Gender", 2), rep("Age", 6),
#'                               rep("Education", 4), rep("Wealth", 5)),
#'                  vallabel = c("Women", "Men", "18-25", "26-35", "36-45",
#'                               "46-55", "56-65", "66+", "  None", "Primary",
#'                               "Secondary", "Post-Sec.", "Low", "2",
#'                               "3", "4", "High"),
#'                  prop = c(20, 22, 21, 24, 22, 21, 17, 15, 20, 18, 21, 25, 21,
#'                           21, 21, 21, 22),
#'                  proplabel = c("20%", "22%", "21%", "24%", "22%", "21%",
#'                                "17%", "15%", "20%", "18%", "21%", "25%",
#'                                "21%", "21%", "21%", "21%", "22%"),
#'                  lb = c(19, 21, 20, 23, 21, 20, 15, 13, 16, 17, 20, 24, 20,
#'                         20, 20, 20, 21),
#'                  ub = c(21, 23, 22, 25, 23, 22, 19, 17, 24, 19, 22, 26, 22,
#'                         22, 22, 22, 23))
#'require(lapop); lapop_fonts
#' lapop_mover(df,
#'             main_title = paste0("More educated, men, and younger individuals",
#'                                 " in the LAC region are the\nmost likely",
#'                                   " to be crime victims"),
#'             subtitle = "% victim of a crime", qword = "",
#'             source_info = "Source: LAPOP Lab, AmericasBarometer",
#'             ymin = 0,
#'             ymax = 40)
#'}
#'@export
#'@import ggplot2
#'@import stringr
#'@importFrom ggtext element_markdown
#'@importFrom stats setNames
#'@import showtext
#'@importFrom stringr str_wrap
#'
#'@author Luke Plutowski, \email{luke.plutowski@@vanderbilt.edu} & Robert Vidigal, \email{robert.vidigal@@vanderbilt.edu}

lapop_mover <- function(data,
                        lang = "en",
                        main_title = "",
                        subtitle = "",
                        qword = NULL,
                        source_info = "LAPOP",
                        rev_values = FALSE,
                        rev_variables = FALSE,
                        subtitle_h_just = 0,
                        ymin = 0,
                        ymax = 100,
                        x_lab_angle = 90,
                        color_scheme = c("#784885", "#008381", "#c74e49", "#2d708e", "#a43d6a")) {

  data$varlabel = factor(data$varlabel, levels = unique(data$varlabel))
  data$order = 1:nrow(data)
  data$order = factor(data$order, levels = unique(data$order))
  mycolors = color_scheme[seq_along(unique(data$varlabel))]

  ci_text = ifelse(lang == "es",
                   paste0(" <span style='color:#585860; font-size:18pt'>\u0131\u2014\u0131 </span>",
                          "<span style='color:#585860; font-size:13pt'>95% intervalo de confianza </span>"),
                   ifelse(lang == "fr",
                          paste0(" <span style='color:#585860; font-size:18pt'>\u0131\u2014\u0131 </span>",
                                 "<span style='color:#585860; font-size:13pt'>Intervalle de confiance de 95% </span>"),
                          paste0(" <span style='color:#585860; font-size:18pt'> \u0131\u2014\u0131 </span> ",
                                 "<span style='color:#585860; font-size:13pt'>95% confidence interval</span>")))

  update_geom_defaults("text", list(family = "inter")) # roboto

  # Build per-facet vertical line positions based on the number of values in each varlabel
  vline_df <- do.call(rbind, lapply(split(data, data$varlabel), function(df) {
    n <- nrow(df)
    data.frame(
      varlabel = unique(df$varlabel)[1],
      xint = if (n > 1) seq(0.5, n - 0.5, by = 1) else numeric(0)
    )
  }))

  # Create a color map based on unique levels in varlabel
  color_map <- setNames(mycolors[1:length(unique(data$varlabel))], unique(data$varlabel))

  # Modify the facet labeller function to use colors
  grid_labeller <- function(x) {
    # Wrap the labels at a specified width
    wrapped_labels <- str_wrap(x, width = 12)

    # Apply color from the color_map based on the variable level
    color <- color_map[x]

    # Return the wrapped label with color applied
    wrapped_labels <- paste0("<span style='color:", color, "'>", wrapped_labels, "</span>")
    return(wrapped_labels)
  }

  ggplot(data, aes(x = order, y = prop, color = factor(varlabel), label = proplabel)) +
    geom_point(alpha = 0.47, key_glyph = "point") +
    facet_grid(. ~ varlabel,
               scales = "free_x",
               space = "free_x",
               labeller = labeller(varlabel = grid_labeller)) +
    geom_errorbar(aes(ymin = lb, ymax = ub), width = 0.2, show.legend = FALSE) +
    geom_text(aes(y = ub), fontface = "bold", size = 5, vjust = -0.8, show.legend = FALSE) +
    scale_color_manual(values = mycolors,
                       labels = paste0("<span style='color:#585860; font-size:13pt'> ",
                                       subtitle,
                                       "<span style='color:#FFFFFF00'>-----------</span>",
                                       ci_text),
                       guide = guide_legend(override.aes = list(shape = 16,
                                                                color = c("black", rep("white", length(unique(data$varlabel)) - 1)),
                                                                fill = c("black", rep("white", length(unique(data$varlabel)) - 1))))) +
    scale_y_continuous(limits = c(ymin, ymax),
                       breaks = seq(ymin, ymax, ifelse(ymax - ymin <= 50, 5, 10)),
                       expand = c(0, 0)) +
    scale_x_discrete(
      labels = function(x) stringr::str_wrap(data$vallabel[match(x, data$order)], width = 12),
      expand = expansion(add = 0.5)
    ) +
    geom_vline(
      data = vline_df,
      aes(xintercept = xint),
      color = "#dddddf",
      linewidth = 0.5,
      show.legend = FALSE) +
    labs(title = main_title,
         y = "",
         x = " ",
         caption = paste0(ifelse(lang == "es" & source_info == "LAPOP", "Fuente: LAPOP Lab",
                                 ifelse(lang == "en" & source_info == "LAPOP", "Source: LAPOP Lab",
                                        source_info)))) +
    theme(text = element_text(size = 14, family = "inter"), # roboto
          plot.title = element_text(size = 17, family = "inter", face = "bold"), # nunito
          plot.caption = element_text(size = 10.5, hjust = 0, vjust = 2, family = "inter", color = "#585860"), # nunito
          plot.subtitle = element_text(size = 14, family = "inter", color = "#585860"), # nunito
          panel.grid.major.x = element_blank(),
          panel.grid.major.y = element_line(linewidth = 0.5, color = "#dddddf"),
          panel.background = element_rect(fill = "white"),
          panel.border = element_rect(linetype = "solid", color = "#dddddf", fill = NA, linewidth = 1),
          axis.text.y = element_blank(),
          axis.text.x = element_text(angle = x_lab_angle, vjust = 0.5),
          axis.ticks = element_blank(),
          axis.text = element_text(size = 14, family = "inter", color = "#585860"), # roboto
          legend.position = "top",
          legend.title = element_blank(),
          legend.justification = 'left',
          legend.margin = margin(0, 0, -5, 0 - subtitle_h_just),
          legend.text = element_markdown(family = "inter-light"), # nunito
          legend.key = element_blank(),
          strip.text = element_markdown(size = 14),
          strip.background = element_blank()
    )
}

#' bra23: Single-country Single-year Dataset
#'
#' A dataset containing the World Map geometry for lapop_map() function.
#'
#' @format A data frame
#' \describe{
#' \item{pais}{Country name}
#' \item{pais_lab}{Country ISO2 code}
#' \item{geometry}{Polygon with geometry}
#' }
#' @source LAPOP AmericasBarometer (https://www.vanderbilt.edu/lapop/)
#' @export
world <- load("data/world.rda")

#######################################

# LAPOP Stacked Bar Graph #

#######################################

#' @rdname lapop-deprecated
#' @section \code{lapop_sb}:
#' For \code{lapop_sb}, use \code{\link{lapop_stack}}.
#'
#' @export

lapop_sb <- function(data, outcome_var = data$prop, prop_labels = data$proplabel,
                        var_labels = data$varlabel, value_labels = data$vallabel,
                        lang = "en",
                        main_title = "",
                        subtitle = "",
                        source_info = "",
                        rev_values = FALSE,
                        rev_variables = FALSE,
                        hide_small_values = TRUE,
                        order_bars = FALSE,
                        subtitle_h_just = 0,
                        color_scheme = c("#2D708E", "#1F9689", "#00ADA9", "#21A356", "#568424", "#ACB014")){
  .Deprecated("lapop_stack")
  lapop_stack(data = data, outcome_var = outcome_var, prop_labels = prop_labels,
              var_labels = var_labels, value_labels = value_labels,
              lang = lang,
              main_title = main_title,
              subtitle = subtitle,
              source_info = source_info,
              rev_values = rev_values,
              rev_variables = rev_variables,
              hide_small_values = hide_small_values,
              order_bars = order_bars,
              subtitle_h_just = subtitle_h_just,
              color_scheme = color_scheme)
}


#' @include lapop_fonts.R
NULL

#' LAPOP Stacked Bar Graphs
#'
#' This function shows a stacked bar graph using LAPOP formatting.
#'
#' @param data Data Frame. Dataset to be used for analysis.  The data frame should have columns
#' titled varlabel (name(s)/label(s) of variable(s) of interest; character), vallabel (names/labels of values for each variable; character),
#' prop (outcome variable value; numeric), and proplabel (text of outcome variable value; character).
#' Default: None (must be provided).
#' @param outcome_var,prop_labels,var_labels,value_labels Numeric, character, character, character.
#' Each component of the data to be plotted can be manually specified in case
#' the default columns in the data frame should not be used (if, for example, the values for a given
#' variable were altered and stored in a new column).
#' @param xvar Character. Column name to group the plots by. This should match a column name in the dataset.
#' Default: NULL (no grouping).
#' @param main_title Character.  Title of graph.  Default: None.
#' @param source_info Character.  Information on dataset used (country, years, version, etc.),
#' which is added to the end of "Source: " in the bottom-left corner of the graph.
#' Default: LAPOP ("Source: LAPOP Lab" will be printed).
#' @param subtitle Character.  Describes the values/data shown in the graph, e.g., "Percent who support...".
#' Default: None.
#' @param lang Character.  Changes default subtitle text and source info to either Spanish or English.
#' Will not translate input text, such as main title or variable labels.  Takes either "en" (English)
#' or "es" (Spanish).  Default: "en".
#' @param color_scheme Character.  Color of data bars for each value.  Allows up to 6 values.
#' Takes hex numbers, beginning with "#".
#' Default: c("#2D708E", "#008381", "#C74E49", "#784885", "#a43d6a","#202020")
#' (navy blue, turquoise, teal, green, sap green, pea soup).
#' @param subtitle_h_just Numeric.  Move the subtitle/legend text left (negative numbers) or right (positive numbers).
#' Ranges from -100 to 100.  Default: 0.
#' @param fixed_aspect_ratio Logical.  Should the aspect ratio be set to a specific value (0.35)?
#' This prevents bars from stretching vertically to fit the plot area.  Set to false when you have
#' a large number of bars (> 10).  Default: TRUE.
#' @param rev_variables Logical.  Should the order of the variables be reversed?  Default: FALSE.
#' @param rev_values Logical.  Should the order of the values for each variable be reversed?  Default: FALSE.
#' @param hide_small_values Logical.  Should labels for categories with less than 5 percent be hidden?  Default: TRUE.
#' @param order_bars Logical.  Should categories be placed in descending order for each bar?  Default: FALSE.
#' showing the distributions of multiple categorical variables.
#' @param legendnrow Numeric.  How many rows for legend labels. Default: 1.
#' @return Returns an object of class \code{ggplot}, a ggplot stacked bar graph
#'
#' @examples
#' \donttest{
#'df <- data.frame(varlabel = c(rep("Politicians can\nidentify voters", 5),
#'                              rep("Wealthy can\nbuy results", 5),
#'                              rep("Votes are\ncounted correctly", 5)),
#'                 vallabel = rep(c("Always", "Often", "Sometimes",
#'                                  "Never", "Other"), 3),
#'                 prop = c(36, 10, 19, 25, 10, 46, 10, 23, 11, 10, 35,
#'                          10, 32, 13, 10),
#'                 proplabel = c("36%", "10%", "19%", "25%", "10%", "46%",
#'                               "10%", "23%", "11%", "10%", "35%", "10%",
#'                               "32%", "13%", "10%"))
#'require(lapop); lapop_fonts()
#'lapop_stack(df,
#'         main_title = "Trust in key features of the electoral process is low in Latin America",
#'         subtitle = "% believing it happens:",
#'         source_info = "Source: LAPOP Lab, AmericasBarometer 2019")
#'}
#'@export
#'@import ggplot2
#'@import ggtext
#'@import showtext
#'@importFrom stats reorder

#'
#'@author Luke Plutowski, \email{luke.plutowski@@vanderbilt.edu} & Robert Vidigal, \email{robert.vidigal@@vanderbilt.edu}

lapop_stack <- function(data,
                        outcome_var = data$prop,
                        prop_labels = data$proplabel,
                        var_labels = data$varlabel,
                        value_labels = data$vallabel,
                        xvar = NULL,
                        lang = "en",
                        main_title = "",
                        subtitle = "",
                        source_info = "LAPOP",
                        rev_values = FALSE,
                        rev_variables = FALSE,
                        hide_small_values = TRUE,
                        order_bars = FALSE,
                        subtitle_h_just = 0,
                        fixed_aspect_ratio = TRUE,
                        legendnrow = 1,
                        color_scheme = c("#2D708E", "#008381", "#C74E49", "#784885", "#a43d6a","#202020")){

  # Ensure proper data types
  if(!inherits(var_labels, "character") & !inherits(var_labels, "factor")){
    var_labels = as.character(var_labels)
    data$varlabels = as.character(data$varlabel)
  }
  if(!inherits(value_labels, "character") & !inherits(value_labels, "factor")){
    value_labels = as.character(value_labels)
    data$vallabel = as.character(data$vallabel)
  }

  # Create a new data frame for plotting
  plot_data <- data.frame(
    var_labels = var_labels,
    value_labels = value_labels,
    outcome_var = outcome_var,
    prop_labels = prop_labels
  )

  # Add grouping variable if provided
  if (!is.null(xvar)) {
    if (xvar %in% colnames(data)) {
      plot_data$group_var <- data[[xvar]]
    } else {
      warning(paste("Column", xvar, "not found in data. Ignoring grouping."))
      xvar <- NULL
    }
  }

  # Set up colors
  mycolors = rev(color_scheme[seq_along(unique(value_labels))])

  # Handle value label ordering
  if(rev_values == TRUE){
    plot_data$value_labels = factor(plot_data$value_labels, levels = unique(plot_data$value_labels))
  } else{
    plot_data$value_labels = factor(plot_data$value_labels, levels = rev(unique(plot_data$value_labels)))
  }

  # Determine x-axis positions
  if (!is.null(xvar)) {
    # When grouped, create a combined label of var_label and group_var
    plot_data$combined_label <- plot_data$group_var

    # Handle variable ordering
    if (rev_variables) {
      positions <- rev(unique(plot_data$combined_label))
    } else {
      positions <- unique(plot_data$combined_label)
    }

    # For x-axis label display
    plot_data$x_display <- plot_data$combined_label
  } else {
    # When not grouped, use var_labels directly
    if (rev_variables) {
      positions <- rev(unique(plot_data$var_labels))
    } else {
      positions <- unique(plot_data$var_labels)
    }

    # For x-axis label display
    plot_data$x_display <- plot_data$var_labels
  }

  update_geom_defaults("text", list(family = "inter")) # roboto

  # Handle ordering of bars if requested
  if(order_bars == TRUE){
    if (!is.null(xvar)) {
      # With grouping, order within each group
      plot_data$x_display <- factor(plot_data$x_display)

      # Create an ordering function that works with grouped data
      plot <- ggplot(plot_data, aes(y = outcome_var, x = x_display,
                                    fill = reorder(value_labels, outcome_var), label = prop_labels))
    } else {
      # Without grouping, order as before
      plot_data$var_labels <- factor(plot_data$var_labels, levels = unique(plot_data$var_labels))
      plot_data$x_display <- plot_data$var_labels

      plot <- ggplot(plot_data, aes(y = outcome_var, x = x_display,
                                    fill = reorder(value_labels, outcome_var), label = prop_labels))
    }

    # Generate the ordered plot
    plot +
      geom_bar(position = "stack", stat = "identity", width = 0.6) +
      geom_text(aes(label = ifelse(outcome_var >= 5, prop_labels, NA)),
                position = position_stack(vjust = 0.5), color = "#FFFFFF",
                fontface = "bold", size = 5, na.rm=T) +
      ggrepel::geom_text_repel(aes(label = ifelse(outcome_var < 5 & hide_small_values == FALSE, prop_labels, NA)),
                               position = position_stack(vjust = 0.5),
                               color = "#FFFFFF", segment.color = 'transparent',
                               fontface = "bold", size = 4, family = "inter", # roboto
                               direction = "y",
                               force_pull = 0.2, force = 5, na.rm=T) +
      coord_flip() +
      scale_fill_manual(values = mycolors, guide = guide_legend(reverse = TRUE, nrow = legendnrow), na.translate = FALSE) +
      scale_x_discrete(limits = positions, expand = c(0, 0)) +
      scale_y_continuous(expand = c(0.02, 0)) +
      labs(title = main_title,
           y = "",
           x = " ",
           caption = paste0(ifelse(lang == "es" & source_info == "LAPOP", "Fuente: LAPOP Lab",
                                   ifelse(lang == "en" & source_info == "LAPOP", "Source: LAPOP Lab",
                                          source_info))),
           subtitle = subtitle) +
      theme(text = element_text(size = 14, family = "inter"), # roboto
            plot.title = element_text(size = 17, family = "inter", face = "bold"), # nunito
            plot.caption = element_text(size = 10.5, hjust = 0.02, vjust = 2, family = "inter", color="#585860"), # nunito
            plot.subtitle = element_text(size = 14, family = "inter-light", color="#585860"), # nunito-light
            axis.title.y = element_blank(),
            axis.text.x = element_blank(),
            axis.text.y = element_text(margin=margin(r=0)),
            axis.ticks = element_blank(),
            axis.text = element_text(size = 14, family = "inter", color = "#585860", margin=margin(r=5)), # roboto
            panel.background = element_rect(fill = "white"),
            panel.grid = element_blank(),
            legend.position = "top",
            plot.title.position = "plot",
            plot.caption.position = "plot",
            legend.text = element_text(family = "inter", color = "#585860"), # roboto
            legend.title = element_blank(),
            legend.justification='left',
            legend.key.size = unit(1, "line"),
            legend.margin = margin(t=5,b=5, 0, subtitle_h_just)) +
      {if(fixed_aspect_ratio)theme(aspect.ratio = 0.35)}
  } else {
    # Create the standard plot without ordering bars
    ggplot(plot_data, aes(fill = value_labels, y = outcome_var, x = x_display, label = prop_labels)) +
      geom_bar(position = "stack", stat = "identity", width = 0.6) +
      geom_text(aes(label = ifelse(outcome_var >= 5, prop_labels, NA)),
                position = position_stack(vjust = 0.5), color = "#FFFFFF",
                fontface = "bold", size = 5, na.rm=T) +
      ggrepel::geom_text_repel(aes(label = ifelse(outcome_var < 5 & hide_small_values == FALSE, prop_labels, NA)),
                               position = position_stack(vjust = 0.5),
                               color = "#FFFFFF", segment.color = 'transparent',
                               fontface = "bold", size = 4, family = "inter", # nunito
                               direction = "y",
                               force_pull = 0.2, force = 5, na.rm=T) +
      coord_flip() +
      scale_fill_manual(values = mycolors, guide=guide_legend(reverse = TRUE, nrow = legendnrow)) +
      scale_x_discrete(limits = positions, expand = c(0, 0)) +
      scale_y_continuous(expand = c(0.02, 0)) +
      labs(title = main_title,
           y = "",
           x = " ",
           caption = paste0(ifelse(lang == "es" & source_info == "LAPOP", "Fuente: LAPOP Lab",
                                   ifelse(lang == "en" & source_info == "LAPOP", "Source: LAPOP Lab",
                                          source_info))),
           subtitle = subtitle) +
      theme(text = element_text(size = 14, family = "inter"), # roboto
            plot.title = element_text(size = 17, family = "inter", face = "bold"), # nunito
            plot.caption = element_text(size = 10.5, hjust = 0, vjust = 2, family = "inter-light", color="#585860"), # roboto
            plot.subtitle = element_text(size = 14, family = "inter-light", color="#585860"), # nunito
            axis.title.y = element_blank(),
            axis.text.x = element_blank(),
            axis.text.y = element_text(margin=margin(r=0)),
            axis.ticks = element_blank(),
            axis.text = element_text(size = 14, family = "inter", color = "#585860", margin=margin(r=5)), # roboto
            panel.background = element_rect(fill = "white"),
            panel.grid = element_blank(),
            legend.position = "top",
            plot.title.position = "plot",
            plot.caption.position = "plot",
            legend.text = element_text(family = "inter", color = "#585860"), # roboto
            legend.title = element_blank(),
            legend.justification='left',
            legend.key.size = unit(1, "line"),
            legend.margin = margin(t=5,b=5, 0, subtitle_h_just)) +
      {
        if (fixed_aspect_ratio) theme(aspect.ratio = 0.35)
        }
  }
}

#######################################

# LAPOP Regression Coefficient Graphs #

#######################################

#' @include lapop_fonts.R
NULL
#'
#' LAPOP Regression Graphs
#'
#' This function creates plots of regression coefficients and predicted probabilities using LAPOP formatting.
#'
#' @param data Data Frame. Dataset to be used for analysis.  The data frame should have columns
#' titled coef (regression coefficients/predicted probabilities; numeric), proplabel (text of outcome variable; character),
#' varlabel (names of variables to be plotted; character), lb (lower bound of coefficient estimate; numeric),
#' ub (upper bound of estimate; numeric), and pvalue (p value of coefficient estimate; numeric).
#'  Default: None (must be supplied).
#' @param coef_var,label_var,varlabel_var,lb,ub,pval_var Numeric, character, character, numeric,
#' numeric, numeric. Each component of the data to be plotted can be manually specified in case
#' the default columns in the data frame should not be used (if, for example, the values for a given
#' variable were altered and stored in a new column).
#' @param ymin,ymax Numeric.  Minimum and maximum values for y-axis. Default: dynamic.
#' @param main_title Character.  Title of graph.  Default: None.
#' @param source_info Character.  Information on dataset used (country, years, version, etc.),
#' which is added to the end of "Source: " in the bottom-left corner of the graph.
#' Default: None (only "Source: " will be printed).
#' @param subtitle Character.  Describes the values/data shown in the graph, e.g., "Regression coefficients".
#' Default: None.
#' @param lang Character.  Changes default subtitle text and source info to either Spanish or English.
#' Will not translate input text, such as main title or variable labels.  Takes either "en" (English)
#' or "es" (Spanish).  Default: "en".
#' @param color_scheme Character.  Color of bars.  Takes hex number, beginning with "#".
#' Default: "#784885" (purple).
#' @param pred_prob Logical.  Is the graph showing predicted probabilities (instead of regression coefficients)?
#' Will only change text in the legend, not the data.  Default: FALSE.
#' @param subtitle_h_just Numeric.  Move the subtitle/legend text left (negative numbers) or right (positive numbers).
#' Ranges from -100 to 100.  Default: 0.
#' @return Returns an object of class \code{ggplot}, a ggplot figure showing
#' coefficients or predicted probabilities from a multivariate regression.
#'
#' @examples
#' \donttest{
#' require(lapop); lapop_fonts()
#' df <- data.frame(
#'   varlabel = c("Intimate\nPartner", "wealth", "Education", "Age", "Male"),
#'   coef = c(0.02, -0.07, -0.24, 0.01, 0.11),
#'   lb = c(-0.002, -0.110, -0.295, -0.060, 0.085),
#'   ub = c(0.049, -0.031, -0.187, 0.080, 0.135),
#'   pvalue = c(0.075, 0.000, 0.000, 0.784, 0.000),
#'   proplabel = c("0.02", "-0.07", "-0.24", "0.01", "0.11")
#' )
#'
#' lapop_coef(df,
#'            main_title = "Demographic and Socioeconomic Predictors of Normalizing IPV",
#'            pred_prob = TRUE,
#'            source_info = ", AmericasBarometer 2021",
#'            ymin = -0.3,
#'            ymax = 0.2)
#'}
#'@export
#'@import ggplot2
#'@import ggtext
#'@import showtext
#'
#'@author Luke Plutowski, \email{luke.plutowski@@vanderbilt.edu}
#'

lapop_coef <- function(data, coef_var = data$coef, label_var = data$proplabel,
                       varlabel_var = data$varlabel, lb = data$lb, ub = data$ub,
                       pval_var = data$pvalue,
                       lang = "en",
                       main_title = "",
                       subtitle = "",
                       source_info = "",
                       ymin = NULL,
                       ymax = NULL,
                       pred_prob = FALSE,
                       color_scheme = "#784885",
                       subtitle_h_just = 0){
  varlabel_var = factor(varlabel_var, levels = rev(unique(varlabel_var)))
  sig = ifelse(pval_var < 0.05, FALSE, TRUE)
  ci_text = ifelse(lang == "es",
                   paste0(" <span style='color:", color_scheme, "; font-size:18pt'> \u0131\u2014\u0131</span> ",
                          "<span style='color:#585860; font-size:13pt'>95% intervalo de confianza </span>"),
                   paste0(" <span style='color:", color_scheme, "; font-size:18pt'> \u0131\u2014\u0131</span> ",
                          "<span style='color:#585860; font-size:13pt'>95% confidence </span>",
                          "<span style='color:#585860'>interval</span>"))
  update_geom_defaults("text", list(family = "inter"))# roboto
  ggplot(data, aes(x = varlabel_var, y = coef_var)) +
    geom_hline(yintercept = 0, color = "#dddddf", lty = 2) +
    geom_errorbar(aes(x=varlabel_var, ymin = lb, ymax = ub), width = 0.3, lty = 1, color = color_scheme) +
    geom_point(aes(x = varlabel_var, y = coef_var, fill = sig), color = "black", size = 5.5, shape = 21) +
    geom_text(aes(label = label_var, vjust = -1.25), size = 5, color = color_scheme, fontface = "bold") +
    scale_fill_manual(values = color_scheme,
                      labels = paste0(" <span style='color:#585860; font-size:13pt'> ",
                                      ifelse(lang == "es", ifelse(pred_prob == TRUE, "Probabilidades pronosticadas", "Coeficientes de regresi\u00f3n"),
                                             ifelse(pred_prob == TRUE, "Predicted probabilities", "Regression coefficients")),
                                      "<span style='color:#FFFFFF00'>-----------</span>",
                                      ci_text),
                      limits = "FALSE",
                      na.value = "white") +
    coord_flip() +
    scale_y_continuous(limits = c(ymin, ymax)) +
    labs(title = main_title,
         y = " ",
         x = " ",
         caption = paste0(ifelse(lang == "es", "Fuente: LAPOP Lab", "Source: LAPOP Lab"),
                          source_info)) +
    theme(text = element_text(size = 14, family = "inter"), # roboto
          plot.title = element_text(size = 18, family = "inter", face = "bold"), # nunito
          plot.caption = element_text(size = 10.5, hjust = 0, vjust = 2, family = "inter", color="#585860"), # nunito
          plot.subtitle = element_text(size = 14, family = "inter-light", color="#585860"), # nunito-light
          axis.title.y = element_blank(),
          axis.ticks = element_blank(),
          axis.text = element_text(size = 14, family = "inter", color = "#585860"), # roboto
          panel.background = element_rect(fill = "white"),
          panel.grid = element_blank(),
          legend.position = "top",
          legend.title = element_blank(),
          plot.title.position = "plot",
          plot.caption.position = "plot",
          legend.justification='left',
          #legend.margin = margin(t=0, b=0),
          legend.text = element_markdown(family = "inter-light"), # nunito-light
          legend.key=element_blank(),
          legend.margin=margin(0, 0, 0, -30-subtitle_h_just))
  }

#######################################

# LAPOP Map World / Americas-Only

#######################################

#' @include lapop_fonts.R
NULL

#'
#' LAPOP World and Americas Map Graph
#'
#' The `lapop_map()` generates a stylized choropleth map using ISO2 country codes
#' from both continuous and factor variables. It is designed to map cross-country
#' results from `lpr_cc()` and supports either a full world map (`survey = "CSES"`)
#' # or an Americas-only map (`survey = "AmericasBarometer"`).
#'
#' @param data A data frame containing ISO2 country codes and a value to map.
#' @param pais_lab String. Column name containing ISO2 country codes (e.g., `"US"`, `"BR"`).
#' @param outcome String. Column name containing the numeric or categorical variable to visualize.
#' @param survey Either `"CSES"` (full world map) or `"AmericasBarometer"` (Americas only).
#' @param zoom Numeric (0–1). Controls how tightly the map zooms when `survey = "AmericasBarometer"`. Default is `1`.
#' @param palette Vector of up to 5 colors for continuous and factor variables.
#' @param main_title Character. Title of graph.  Default: None.
#' @param source_info Character. Information on dataset used (country, years, version, etc.),
#' which is added to the bottom-left corner of the graph. Default: LAPOP ("Source: LAPOP Lab" will be printed).
#' @param subtitle Character.  Describes the values/data shown in the graph, e.g., "percentage of Mexicans who say...)".
#' Default: None.
#' @param lang Character.  Changes default subtitle text and source info to either Spanish or English.
#' Will not translate input text, such as main title or variable labels.  Takes either "en" (English)
#' or "es" (Spanish).  Default: "en".
#' @param selected_countries Character or NULL. ISO2 code of the currently selected country (e.g. from
#' `input$pais` in Shiny). When not `NULL`, countries with no data are rendered with diagonal stripes
#' instead of solid gray. Default: `NULL` (solid `"#dddddf"`).
#'
#' @return A `ggplot2` choropleth map object.
#'
#' @examples
#' \donttest{
#' # Standalone — solid gray for no-data countries
#' lapop_fonts()
#'  data_cont <- data.frame(
#'   vallabel = c("US", "AR", "VE", "CH", "EC", "BO"),
#'   prop = c(37, 52, 80, 17, 69, 94)
#' )
#' lapop_map(data_cont, pais_lab = "vallabel", outcome = "prop", zoom = 0.9,
#'           survey = "AmericasBarometer", main_title = "Latin America and Caribbean Countries",
#'           subtitle = "% of respondents")
#'
#' }
#' if (interactive()) {
#'   lapop_map(data_cont, pais_lab = "vallabel", outcome = "prop", zoom = 0.9,
#'             survey = "AmericasBarometer", selected_countries = "BR")
#' }
#' @export
#' @import ggplot2
#' @import ggtext
#' @import grid
#' @importFrom sf st_bbox st_drop_geometry
#' @importFrom dplyr filter left_join rename
#'
#' @author Robert Vidigal, \email{robert.vidigal@@vanderbilt.edu}

lapop_map <- function(data,
                      outcome = "value",
                      pais_lab = "pais_lab",
                      survey = c("CSES", "AmericasBarometer"),
                      zoom = 1,
                      main_title = "",
                      subtitle = "",
                      palette = c("#F2A344", "#D97A1E", "#BF5A00", "#8A3900", "#4A1E00"),
                      source_info = "LAPOP",
                      lang = "en",
                      selected_countries = NULL) {

  survey <- match.arg(survey)
  zoom <- max(0, min(1, zoom))

  # ---------------------------------------------------------------
  # ISO2 vectors
  # ---------------------------------------------------------------
  americas_iso2 <- c(
    "US","CA","MX",
    "CR","SV","GT","HN","NI","PA","BZ",
    "DO","HT","TT","JM","CU","BB","BS","GD","LC","VC",
    "AR","BO","BR","CL","CO","EC","GY","PE","PY","SR","UY","VE"
  )

  if (is.null(world_sf)) {
    stop("world map data is not loaded. Restart R and load `lapop` again.")
  }

  world <- world_sf

  if (survey == "AmericasBarometer") {
    world <- world %>% dplyr::filter(pais_lab %in% americas_iso2)
  }

  # ---------------------------------------------------------------
  # Merge user data
  # ---------------------------------------------------------------
  if (!all(c(outcome, pais_lab) %in% names(data))) {
    stop("`outcome` and `pais_lab` must refer to columns in `data`.")
  }

  df <- data.frame(
    pais_lab = data[[pais_lab]],
    value = data[[outcome]],
    stringsAsFactors = FALSE
  )

  merged <- world %>% dplyr::left_join(df, by = "pais_lab") %>% sf::st_as_sf()
  outcome_is_factor <- is.factor(merged$value) || is.character(merged$value)

  # ---------------------------------------------------------------
  # Stripe flag: TRUE only when a country is selected (Shiny context)
  # Standalone use: selected_countries = NULL → always solid gray
  # ---------------------------------------------------------------
  use_stripes <- !is.null(selected_countries) && length(selected_countries) > 0 && any(nchar(selected_countries) > 0)

  na_data  <- merged %>% dplyr::filter(is.na(value))
  val_data <- merged %>% dplyr::filter(!is.na(value))

  # ---------------------------------------------------------------
  # BASE MAP
  # ---------------------------------------------------------------
  p <- ggplot2::ggplot()

  if (use_stripes && nrow(na_data) > 0) {
    if (!requireNamespace("ggpattern", quietly = TRUE)) {
      stop("Package 'ggpattern' is required for stripe rendering. Install with install.packages('ggpattern').")
    }
    p <- p +
      ggpattern::geom_sf_pattern(
        data            = na_data,
        pattern         = "stripe",
        pattern_colour  = "#aaaaaa",
        pattern_fill    = "#aaaaaa",
        pattern_density = 0.4,
        pattern_spacing = 0.03,
        pattern_angle   = 45,
        fill            = "#ffffff",  # white base under stripes
        color           = "black",
        size            = 0.25
      )
  } else if (nrow(na_data) > 0) {
    p <- p +
      ggplot2::geom_sf(
        data  = na_data,
        fill  = "#dddddf",            # solid gray — default / standalone
        color = "black",
        size  = 0.25
      )
  }

  p <- p +
    ggplot2::geom_sf(
      data  = val_data,
      ggplot2::aes(fill = value),
      color = "black",
      size  = 0.25
    )

  # ---------------------------------------------------------------
  # FILL SCALES
  # ---------------------------------------------------------------
  if (outcome_is_factor) {
    p <- p +
      ggplot2::scale_fill_manual(
        values = palette,
        drop   = FALSE,
        na.value = "#dddddf"
      )
  } else {
    p <- p +
      ggplot2::scale_fill_gradientn(
        labels = function(x) formatC(x, format = "g"),
        colors = palette,
        na.value = "#dddddf",
        guide = ggplot2::guide_colorbar(
          direction      = "horizontal",
          barwidth       = grid::unit(50, "pt"),
          barheight      = grid::unit(12, "pt"),
          label.position = "top"
        )
      )
  }

  # ---------------------------------------------------------------
  # Zoom logic for Americas
  # ---------------------------------------------------------------
  if (survey == "AmericasBarometer") {
    p <- p + ggplot2::coord_sf(
      xlim   = c(-170, -25),
      ylim   = c(-60, 80),
      expand = FALSE
    )
  }

  # ---------------------------------------------------------------
  # Caption logic
  # ---------------------------------------------------------------
  caption_text <- ifelse(
    lang == "es" & source_info == "LAPOP", "Fuente: LAPOP Lab",
    ifelse(lang == "en" & source_info == "LAPOP", "Source: LAPOP Lab",
           source_info)
  )

  # ---------------------------------------------------------------
  # Final theme
  # ---------------------------------------------------------------
  p + ggplot2::theme_void() +
    ggplot2::labs(
      title    = main_title,
      subtitle = subtitle,
      fill     = "",
      caption  = caption_text
    ) +
    ggplot2::theme(
      plot.margin        = ggplot2::margin(0, 175, 0, 0),
      legend.position    = "bottom",
      legend.box         = "vertical",
      legend.box.just    = "left",
      legend.justification = "left",
      legend.title       = ggplot2::element_blank(),
      legend.key.width   = grid::unit(16, "pt"),
      legend.key.height  = grid::unit(18, "pt"),
      legend.text        = ggtext::element_markdown(family = "inter-light"),
      plot.title         = ggplot2::element_text(size = 18, family = "inter", face = "bold"),
      plot.caption       = ggplot2::element_text(size = 10.5, vjust = 2, hjust = 0,
                                                 family = "inter", color = "#585860"),
      plot.subtitle      = ggplot2::element_text(size = 13, hjust = 0,
                                                 family = "inter", color = "#585860")
    )
}

#######################################

# LAPOP Bar Graphs #

#######################################

#' @include lapop_fonts.R
NULL

#' LAPOP Bar Graphs
#'
#' This function shows a bar graph for categorical variables using LAPOP formatting.
#' @param cc1,cc2 lapop_cc (ggplot) object.  Graphic for left and right panes, respectively.
#' @param subtitle1,subtitle2 Character.  Describes the values/data shown in the graph, e.g., "Percent who agree that...".
#' Default: None.
#' @param main_title Character.  Title of graph.  Default: None.
#' @param source_info Character.  Information on dataset used (country, years, version, etc.),
#' which is added to the end of "Source: LAPOP Lab" in the bottom-left corner of the graph.
#' Default: None (only "Source: LAPOP Lab" will be printed).
#' @param lang Character.  Changes default subtitle text and source info to either Spanish or English.
#' Will not translate input text, such as main title or variable labels.  Takes either "en" (English)
#' or "es" (Spanish).  Default: "en".
#' @param color_scheme Character.  Color of bars.
#' Takes hex numbers, beginning with "#". Default: "#008381".
#' @param file_name Character.  If desired, supply file path + name to save graph.
#' @param width_px,height_px Numeric.  Width and height of saved graph in pixels. Default: 895, 600.
#' @return Returns an object of class \code{ggplot}, a ggplot bar graph.
#'
#' @examples
#'\donttest{
#' require(lapop); lapop_fonts()
#' df1 <- data.frame(vallabel = c("Crime victim", "Non-victim"),
#' prop = c(36.1, 19.3),
#' proplabel = c("36%" ,"19%"),
#' lb = c(34.9, 18.1),
#' ub = c(37.3, 20.5))
#'
#' df2 <- data.frame(vallabel = c("Crime victim", "Non-victim"),
#' prop = c(45, 15),
#' proplabel = c("45%" ,"15%"),
#' lb = c(43, 13),
#' ub = c(47, 16))
#'
#' ccx <- lapop_cc(df1)
#' ccy <- lapop_cc(df2)
#'
#' lapop_cccomb(ccx, ccy,
#' subtitle1 = "% who support democracy",
#' subtitle2 = "% who are satisfied with democracy",
#' main_title = "Crime victims are more supportive of and satisfied with democracy",
#' source_info = ", AmericasBarometer 2023")
#'}
#'
#'@export
#'@import ggplot2
#'@import grid
#'@import gridtext
#'@importFrom gridExtra grid.arrange
#'
#'@author Luke Plutowski, \email{luke.plutowski@@vanderbilt.edu}


lapop_cccomb <- function(cc1, cc2,
                         subtitle1 = "",
                         subtitle2 = "",
                         main_title = "",
                         source_info = "",
                         lang = "en",
                         color_scheme = "#784885",
                         file_name = "",
                         width_px = 895,
                         height_px = 600
){
  cc1f = cc1 +
    theme(axis.title.x.bottom=element_text(size=13, family = "nunito"),
          plot.subtitle=element_text(size=14, family = "nunito", color = "#585860"),
          legend.position = "none") +
    labs(subtitle = subtitle1, caption = "")

  cc2f = cc2 +
    theme(axis.title.x.bottom=element_text(size=13, family = "nunito"),
          plot.subtitle=element_text(size=14, family = "nunito", color = "#585860"),
          legend.position = "none") +
    labs(subtitle = subtitle2, caption = "")


  ci_text = ifelse(lang == "es",
                   paste0(" <span style='color:", color_scheme, "; font-size:18pt'> \u0131\u2014\u0131</span> ",
                          "<span style='color:#585860; font-size:13pt'>95% intervalo de confianza </span>"),
                   ifelse(lang == "fr",
                          paste0(" <span style='color:", color_scheme, "; font-size:18pt'> \u0131\u2014\u0131</span> ",
                                 "<span style='color:#585860; font-size:13pt'>Intervalle de confiance de 95% </span>"),
                          paste0(" <span style='color:", color_scheme, "; font-size:18pt'> \u0131\u2014\u0131</span> ",
                                 "<span style='color:#585860; font-size:13pt'>95% confidence </span>",
                                 "<span style='color:#585860'>interval</span>")))
  # update_geom_defaults("text", list(family = "roboto"))

  top=textGrob(main_title,
               gp = gpar(fontfamily = "nunito", fontsize = 18, fontface = 'bold'),
                just = "left", x = 0)
  sub = gridtext::richtext_grob(ci_text, x = 0, hjust = -.05)
  bottom = textGrob(paste0("Source: LAPOP Lab", source_info),
                    gp = gpar(fontsize = 10.5, vjust = 2, hjust = 0.02, fontfamily = "nunito", color="#585860"),
                    just = "left", x = 0)
  g = c(list(top), list(sub), list(cc1f), list(cc2f), list(bottom))
  matlayout = rbind((1:1), (2:2), (3:4), (5:5))
  tab = gridExtra::grid.arrange(grobs=g,layout_matrix=matlayout,heights=c(1.4, 0.5, 10, 1.15))
  if(file_name != "") {
    ggsave(file_name, plot = tab, width = width_px/96, height = height_px/96)
  }
  return(invisible(tab))
}

#######################################

# LAPOP Save #

#######################################

#' LAPOP Save
#'
#' This function creates exports graphs created using the LAPOP templates.
#'
#' @param figure Ggplot object.
#' @param filename File path + name to be saved + .filetype.
#' @param format Character.  Options: "png", "svg". Default = "svg".
#' @param logo Logical.  Should logo be added to plot?  Default: FALSE.
#' @param width_px Numeric. Width in pixels.  Default: 750.
#' @param height_px Numeric.  Height in pixels.
#'
#' @return Saves a file (in either .svg or .png format) to provided directory.
#'
#' @examples
#' \donttest{df <- data.frame(
#'cat = c("Far Left", 1, 2, 3, 4, "Center", 6, 7, 8, 9, "Far Right"),
#'prop = c(4, 3, 5, 12, 17, 23, 15, 11, 5, 4, 1),
#'proplabel = c("4%", "3%", "5%", "12%", "17%", "23%", "15%", "11%", "5%", "4%", "1%")
#')
#' myfigure <- lapop_hist(df,
#'           main_title = "Centrists are a plurality among Peruvians",
#'           subtitle = "Distribution of ideological preferences",
#'           source_info = "Peru, 2019",
#'           ymax = 27
#')
#'
#' f <- file.path(tempdir(), "fig1.svg")
#' lapop_save(myfigure, f, format = "svg", width_px = 800)
#'}
#'@import grDevices
#'@import ggplot2
#'@importFrom svglite svglite
#'@export
#'

lapop_save <- function(figure, filename,
                       format = "svg",
                       logo = FALSE,
                       width_px = 895,
                       height_px = 500){
  format <- match.arg(format, choices = c("svg", "png"))

  if (format == "svg") {
    ggplot2::ggsave(
      filename = filename,
      plot = figure,
      width = width_px / 96,
      height = height_px / 96,
      units = "in",
      device = svglite::svglite
    )
  } else {
    ggplot2::ggsave(
      filename = filename,
      plot = figure,
      width = width_px / 96,
      height = height_px / 96,
      units = "in",
      dpi = 96,
      device = "png"
    )
  }

  invisible(filename)
}

#######################################

# LAPOP Cross-Country Bar Graph Pre-Processing #

#######################################

#' LAPOP Cross-Country Bar Graph Pre-Processing
#'
#' This function creates dataframes which can then be input in lapop_cc for
#' comparing values across countries with a bar graph using LAPOP formatting.
#'
#' @param data A survey object.  The data that should be analyzed.
#' @param outcome Outcome variable(s) of interest to be plotted across countries.
#' It can handle a single variable across countries, or multiple variables instead of multiple countries. See examples below.
#' @param xvar Grouping variable. Default: pais_lab. It can handle other variables grouping like year/wave.
#' @param rec Numeric. The minimum and maximum values of the outcome variable that
#' should be included in the numerator of the percentage.  For example, if the variable
#' is on a 1-7 scale and rec is c(5, 7), the function will show the percentage who chose
#' an answer of 5, 6, 7 out of all valid answers.  Default: c(1, 1).
#' @param rec2 Numeric. Same as rec(). Default: c(1, 1).
#' @param rec3 Numeric. Same as rec(). Default: c(1, 1).
#' @param rec4 Numeric. Same as rec(). Default: c(1, 1).
#' @param ci_level Numeric. Confidence interval level for estimates.  Default: 0.95
#' @param mean Logical.  If TRUE, will produce the mean of the variable rather than
#' rescaling to percentage.  Default: FALSE.
#' @param filesave Character.  Path and file name to save the dataframe as csv.
#' @param cfmt changes the format of the numbers displayed above the bars.
#' Uses sprintf string formatting syntax. Default is whole numbers for percentages
#' and tenths place for means.
#' @param sort Character. On what value the bars are sorted: the x or the y.
#' Options are "y" (default; for the value of the outcome variable), "xv" (for
#' the underlying values of the x variable), "xl" (for the labels of the x variable,
#' i.e., alphabetical).
#' @param order Character.  How the bars should be sorted.  Options are "hi-lo"
#' (default) or "lo-hi".
#' @param ttest Logical.  If TRUE, will conduct pairwise t-tests for difference
#' of means between all individual x levels and save them in attr(x,
#' "t_test_results"). Default: FALSE.
#' @param keep_nr Logical.  If TRUE, will convert "don't know" (missing code .a)
#' and "no response" (missing code .b) into valid data (value = 99) and use them
#' in the denominator when calculating percentages.  The default is to examine
#' valid responses only.  Default: FALSE.
#'
#' @return Returns a data frame, with data formatted for visualization by lapop_cc
#'
#' @examples
#'
#' require(lapop); data(ym23); data(bra23)
#'
#' # Set Survey Context on a small cross-country subset
#' ym23_small <- subset(ym23, pais %in% c(1, 15, 17))
#' ym23lpr <- lpr_data(ym23_small)
#'
#' # Single variable in Multiple Countries
#' lpr_cc(data = ym23lpr,
#'       outcome = "ing4",
#'       rec = c(5, 7),
#'       xvar = "pais")
#'
#' # Multiple variables in Single Country
#' \donttest{
#' bra23lpr <- lpr_data(bra23, wt = TRUE)
#' lpr_cc(data = bra23lpr,
#' outcome = c("b12", "b13"),
#' rec = c(5, 7))
#' }
#'
#'@export
#'@import dplyr
#'@import srvyr
#'
#'@author Luke Plutowski, \email{luke.plutowski@@vanderbilt.edu} & Robert Vidigal, \email{robert.vidigal@@vanderbilt.edu}
lpr_cc <- function(data,
                   outcome,
                   xvar = "pais_lab",
                   rec = list(c(1, 1)),
                   rec2 = list(c(1, 1)),
                   rec3 = list(c(1, 1)),
                   rec4 = list(c(1, 1)),
                   ci_level = 0.95,
                   mean = FALSE,
                   filesave = "",
                   cfmt = "",
                   sort = "y",
                   order = "hi-lo",
                   ttest = FALSE,
                   keep_nr = FALSE) {

  outcome_vars <- syms(outcome)  # Convert character vector to symbols for svryr

  if (length(outcome) > 1) {
    xvar <- NULL  # Disable grouping variable when multiple outcomes are used
  }

  if (length(rec) == 2 && all(sapply(rec, is.numeric))) {
    rec <- rep(list(rec), length(outcome_vars))  # Expand rec to a list
  }

  if (length(rec) < length(outcome_vars)) {
    stop("Length of rec must match number of outcome variables.")
  }

  results_list <- list()

  for (i in seq_along(outcome_vars)) {
    curr_outcome <- outcome_vars[[i]]
    curr_rec <- rec[[i]]

    if (keep_nr) {
      data_modified <- data %>%
        mutate(!!curr_outcome := case_when(
          na_tag(!!curr_outcome) == "a" | na_tag(!!curr_outcome) == "b" ~ 99,
          TRUE ~ as.numeric(!!curr_outcome)
        ))
    } else {
      data_modified <- data
    }

    cc <- data_modified %>%
      {
        if (!is.null(xvar)) {
          # Use filter and group_by directly
          filter(., !is.na(!!sym(xvar))) %>%
            group_by(vallabel = as_factor(!!sym(xvar)))
        } else {
          mutate(., vallabel = outcome[i])  # Assign outcome name when xvar is NULL
        }
      } %>%
      group_by(vallabel) %>%  # Ensure grouping happens regardless
      {
        if (mean) { # mean=TRUE calculation
          summarize(., prop = survey_mean(!!curr_outcome, na.rm = TRUE, vartype = "ci", level = ci_level)) %>%
            mutate(proplabel = case_when(cfmt != "" ~ sprintf("%.1f", prop),
                                         TRUE ~ sprintf("%.1f", prop)))
        } else { # percentages calculation
          summarize(., prop = survey_mean(between(!!curr_outcome, curr_rec[1], curr_rec[2]),
                                          na.rm = TRUE, vartype = "ci", level = ci_level) * 100) %>%
            mutate(proplabel = case_when(cfmt != "" ~ sprintf("%.0f%%", round(prop)),
                                         TRUE ~ sprintf("%.0f%%", round(prop))))
        }
      } %>%
      filter(prop != 0) %>%
      rename(lb = prop_low, ub = prop_upp) %>%
      ungroup()

    results_list[[i]] <- cc
  }

  final_results <- bind_rows(results_list) %>%
    select(vallabel, prop, proplabel, lb, ub)  # Reorder columns

  # Reordering results if needed
  if (sort == "y") {
    if (order == "hi-lo") {
      final_results <- final_results %>% arrange(desc(prop))
    } else if (order == "lo-hi") {
      final_results <- final_results %>% arrange(prop)
    }
  }

  # Perform t-tests if ttest = TRUE
  if (ttest) {
    final_results <- final_results %>%
      mutate(se = (ub - lb) / (2 * 1.96))   # Add standard error (se) column

    t_test_results <- data.frame(test = character(), diff = numeric(),
                                 ttest = numeric(), pval = numeric(), stringsAsFactors = FALSE)
    for (i in 1:(nrow(final_results) - 1)) {
      for (j in (i + 1):nrow(final_results)) {
        prop1 <- final_results$prop[i]
        se1 <- final_results$se[i]
        prop2 <- final_results$prop[j]
        se2 <- final_results$se[j]
        diff <- prop1 - prop2
        t_stat <- diff / sqrt(se1^2 + se2^2)
        df <- (se1^2 + se2^2)^2 / ((se1^2)^2 / (nrow(data) - 1) + (se2^2)^2 / (nrow(data) - 1))
        p_value <- 2 * pt(-abs(t_stat), df)
        t_test_results <- rbind(t_test_results,
                                data.frame(test = paste(final_results$vallabel[i], "vs", final_results$vallabel[j]),
                                           diff = diff,
                                           ttest = t_stat,
                                           pval = p_value))
      }
    }

    # Attach t-test results as an attribute to final_results
    attr(final_results, "t_test_results") <- t_test_results
  }

  # Filesaving
  if (filesave != "") {
    if (!dir.exists(dirname(filesave))) {
      dir.create(dirname(filesave), recursive = TRUE)
    }
    write.csv(final_results, filesave)
  }

  # Output
  return(final_results)
}

#######################################

# LAPOP Time-Series Line Graph Pre-Processing #

#######################################

#' LAPOP Time-Series Line Graph Pre-Processing
#'
#' This function creates dataframes which can then be input in lapop_ts for
#' comparing values across time with a line graph using LAPOP formatting.
#'
#' @param data A survey object.  The data that should be analyzed.
#' @param outcome Character.  Outcome variable of interest to be plotted
#' across time.
#' @param rec Numeric. The minimum and maximum values of the outcome variable that
#' should be included in the numerator of the percentage.  For example, if the variable
#' is on a 1-7 scale and rec is c(5, 7), the function will show the percentage who chose
#' an answer of 5, 6, 7 out of all valid answers.  Can also supply one value only,
#' to produce the percentage that chose that value out of all other values.
#' Default: c(1, 1).
#' @param use_wave Logical.  If TRUE, will use "wave" for the x-axis; otherwise,
#' will use "year".  Default: FALSE.
#' @param ci_level Numeric. Confidence interval level for estimates.  Default: 0.95
#' @param mean Logical.  If TRUE, will produce the mean of the variable rather than
#' rescaling to percentage.  Default: FALSE.
#' @param filesave Character.  Path and file name to save the dataframe as csv.
#' @param cfmt Character. changes the format of the numbers displayed above the bars.
#' Uses sprintf string formatting syntax. Default is whole numbers for percentages
#' and tenths place for means.
#' @param ttest Logical.  If TRUE, will conduct pairwise t-tests for difference
#' of means between all individual x levels and save them in attr(x,
#' "t_test_results"). Default: FALSE.
#' @param keep_nr Logical.  If TRUE, will convert "don't know" (missing code .a)
#' and "no response" (missing code .b) into valid data (value = 99) and use them
#' in the denominator when calculating percentages.  The default is to examine
#' valid responses only.  Default: FALSE.
#'
#' @return Returns a data frame, with data formatted for visualization by lapop_ts()
#'
#' @examples
#'
#' require(lapop); data(ym23)
#'
#' # Set Survey Context
#' ym23lpr<-lpr_data(ym23)
#'
#' # Run lpr_ts
#' lpr_ts(ym23lpr,
#' outcome = "ing4",
#' use_wave = TRUE,
#' mean = TRUE,
#' ttest = TRUE)
#'
#'@export
#'@import dplyr
#'@import srvyr
#'
#'@author Berta Diaz, \email{berta.diaz.martinez@@vanderbilt.edu} & Luke Plutowski, \email{luke.plutowski@@vanderbilt.edu}

lpr_ts <- function(data,
                   outcome,
                   rec = c(1, 1),
                   use_wave = FALSE,
                   ci_level = 0.95,
                   mean = FALSE,
                   filesave = "",
                   cfmt = "",
                   ttest = FALSE,
                   keep_nr = FALSE) {

  # If keep_nr is TRUE, convert don't knows (NA(a)) and no answers (NA(b)) to
  # non-NA data (a value of 99).
  if (keep_nr) {
    data <- data %>%
      mutate(!!sym(outcome) := case_when(
        na_tag(!!sym(outcome)) %in% c("a", "b") ~ 99,
        TRUE ~ as.numeric(!!sym(outcome))
      ))
  }

  if (length(rec) == 1) {
    rec = c(rec, rec)
  }


  ts_df <- data %>%
    group_by(wave = if (use_wave) as.character(as_factor(wave)) else as.character(year)) %>%
    {
      if (mean) {
        summarize(.,
                  prop = survey_mean(!!sym(outcome),
                                     na.rm = TRUE,
                                     vartype = "ci",
                                     level = ci_level)) %>%
          mutate(proplabel = if (cfmt != "") {
            sprintf(cfmt, prop)
          } else {
            sprintf("%.1f", prop)
          })
      } else {
        summarize(.,
                  prop = survey_mean(between(!!sym(outcome), rec[1], rec[2]),
                                     na.rm = TRUE,
                                     vartype = "ci",
                                     level = ci_level) * 100) %>%
          mutate(proplabel = if (cfmt != "") {
            sprintf(cfmt, round(prop))
          } else {
            sprintf("%.0f%%", round(prop))
          })}
    } %>%
    filter(prop != 0) %>%
    rename(lb = prop_low, ub = prop_upp)

  # Below chunks add missing values for wave to the time series
  all_waves <- data.frame(
    wave = if (use_wave) {
      c("2004", "2006", "2008", "2010", "2012",
        "2014", "2016/17", "2018/19", "2021", "2023")
    } else {
      c("2004", "2006", "2008", "2010", "2012",
        "2014", "2016", "2017", "2016/17", "2018",
        "2019", "2018/19", "2021", "2023")
    }
  )

  ts_df = merge(ts_df, all_waves,
                by = "wave",
                all.x = TRUE,
                all.y = TRUE)

  # If missing prop at either end of the series, delete
  na_rows = apply(ts_df, 1, function(row) any(is.na(row)))
  first_non_na = which(!na_rows)[1]
  last_non_na = which(!na_rows)[length(which(!na_rows))]
  ts_df = ts_df[first_non_na:last_non_na, ]

  #Ugly code to handle 2016/17 and 2018/19 when using "year"
  # if using year (use_wave = FALSE), and 2016/17 wave is missing,
  # the graph should display "2016/17" (not "2016" and "2017") with no data
  # same for 2018/19
  if (!use_wave) {
    if (is.na(ts_df$prop[ts_df$wave == "2016"]) &
        is.na(ts_df$prop[ts_df$wave == "2017"])) {
      ts_df = ts_df[-(which(ts_df$wave %in% c("2016", "2017"))),]
    } else if (!is.na(ts_df$prop[ts_df$wave == "2016"])) {
      ts_df = ts_df[-(which(ts_df$wave %in% c("2017", "2016/17"))),]
    } else if (!is.na(ts_df$prop[ts_df$wave == "2017"])) {
      ts_df = ts_df[-(which(ts_df$wave %in% c("2016", "2016/17"))),]
    }

    if (is.na(ts_df$prop[ts_df$wave == "2018"]) &
        is.na(ts_df$prop[ts_df$wave == "2019"])) {
      ts_df = ts_df[-(which(ts_df$wave %in% c("2018", "2019"))),]
    } else if (!is.na(ts_df$prop[ts_df$wave == "2018"])) {
      ts_df = ts_df[-(which(ts_df$wave %in% c("2019", "2018/19"))),]
    } else if (!is.na(ts_df$prop[ts_df$wave == "2019"])) {
      ts_df = ts_df[-(which(ts_df$wave %in% c("2018", "2018/19"))),]
    }
  }


  if (ttest) {

    ts_df <- ts_df %>%
      mutate(se = (ub - lb) / (2 * 1.96))

    t_test_results <- data.frame(
      test = character(),
      diff = numeric(),
      t_stat = numeric(),
      p_value = numeric(),
      stringsAsFactors = FALSE
    )

    for (i in 1:(nrow(ts_df) - 1)) {
      for (j in (i + 1):nrow(ts_df)) {
        prop1 <- ts_df$prop[i]
        se1 <- ts_df$se[i]
        prop2 <- ts_df$prop[j]
        se2 <- ts_df$se[j]

        diff <- prop1 - prop2
        t_stat <- diff / sqrt(se1^2 + se2^2)
        df <- (se1^2 + se2^2)^2 /
          ((se1^2)^2 / (nrow(ts_df) - 1) + (se2^2)^2 / (nrow(ts_df) - 1))
        p_value <- 2 * pt(-abs(t_stat), df)

        t_test_results <- rbind(
          t_test_results,
          data.frame(
            test = paste0("Wave ", ts_df$wave[i], " vs Wave ", ts_df$wave[j]),
            diff = round(diff, 3),
            t_stat = round(t_stat, 3),
            p_value = round(p_value, 3)
          )
        )
      }
    }

    attr(ts_df, "t_test_results") <- t_test_results
  }


  if (filesave != "") {
    write.csv(ts_df, filesave, row.names = FALSE)
  }

  return(ts_df)
}

#' Compute Design-Based Proportion and Confidence Interval
#'
#' Computes a weighted proportion (mean of a binary outcome) and its
#' confidence interval using complex survey design features. When
#' stratification and PSU variables are supplied, the function uses
#' Taylor linearized variance estimation via the \pkg{survey} package.
#'
#' If both `strata` and `psu` are provided, a full complex survey design
#' is declared. If they are `NULL`, the function computes a weighted
#' estimate assuming simple random sampling (SRS) with weights.
#'
#' Lonely PSUs are handled using `survey.lonely.psu = "adjust"`.
#'
#' @param data A data frame containing the outcome and survey design variables.
#' @param outcome Character string. Name of a binary variable coded 0/1.
#' @param weight Character string. Name of the sampling weight variable.
#'   Default is `"weight1500"`.
#' @param strata Character string. Name of the stratification variable.
#'   Default is `NULL`. If provided together with `psu`, a complex survey
#'   design is used.
#' @param psu Character string. Name of the primary sampling unit (cluster)
#'   variable. Default is `NULL`.
#' @param conf.level Numeric. Confidence level for the interval.
#'   Default is `0.95`.
#' @param na.rm Logical. If `TRUE`, rows with missing values in the
#'   required variables are removed prior to estimation.
#'
#' @return A data frame with:
#' \describe{
#'   \item{prop}{Estimated proportion (mean of binary outcome).}
#'   \item{lb}{Lower bound of the confidence interval.}
#'   \item{ub}{Upper bound of the confidence interval.}
#'   \item{se}{Standard error of the estimate.}
#' }
#'
#' @details
#' Variance estimation is performed using Taylor linearization as implemented
#' in \code{\link[survey]{svymean}}. When both `strata` and `psu` are supplied,
#' clustering and stratification are incorporated in the variance estimator.
#'
#' If `strata` and `psu` are not provided, the function assumes a weighted
#' simple random sample and estimates variance accordingly.
#'
#' @examples
#' \donttest{
#' # Design-based estimate
#' data(cm23)
#' lpr_ci(data = cm23,
#'          outcome = "b13",
#'          weight = "weight1500",
#'          strata = "strata",
#'          psu = "upm")
#'
#' # Weighted SRS estimate
#' data(bra23)
#' lpr_ci(data = bra23,
#'          outcome = "b13",
#'          weight = "wt")
#' }
#'
#' @import survey
#' @importFrom stats confint
#' @export

lpr_ci <- function(data,
                     outcome,      # binary variable (0/1)
                     weight = "weight1500",         # weight variable (e.g., "weight1500")
                     strata = NULL,  # strata variable (optional)
                     psu = NULL,     # PSU/cluster variable (optional)
                     conf.level = 0.95,
                     na.rm = TRUE) {

  # Remove missing values if requested
  vars_needed <- c(outcome, weight, strata, psu)
  vars_needed <- vars_needed[!is.null(vars_needed)]

  if (na.rm) {
    data <- data[complete.cases(data[, vars_needed]), ]
  }

  # Define survey design
  if (!is.null(strata) && !is.null(psu)) {

    options(survey.lonely.psu = "adjust") # FOR SINGLE PSU COUNTRIES

    design <- survey::svydesign(
      ids     = as.formula(paste0("~", psu)),
      strata  = as.formula(paste0("~", strata)),
      weights = as.formula(paste0("~", weight)),
      data    = data,
      nest    = TRUE
    )

  } else {

    # Weighted but no complex design
    design <- survey::svydesign(
      ids     = ~1,
      weights = as.formula(paste0("~", weight)),
      data    = data
    )
  }

  # Estimate proportion
  formula_outcome <- as.formula(paste0("~", outcome))

  est <- survey::svymean(formula_outcome, design)

  ci  <- stats::confint(est, level = conf.level)

  result <- data.frame(
    prop = coef(est)[1],
    lb   = ci[1],
    ub   = ci[2],
    se   = SE(est)[1]
  )

  return(result)
}

#######################################

# LAPOP Bar Graphs #

#######################################

#' @include lapop_fonts.R
NULL

#' LAPOP Bar Graphs
#'
#' This function shows a bar graph for categorical variables using LAPOP formatting.
#'
#' @param data Data Frame. Dataset to be used for analysis.  The data frame should have columns
#' titled cat (labels of each category in variable; character),
#' prop (outcome variable value; numeric), and proplabel (text of outcome variable value; character).
#' Default: None (must be provided).
#' @param cat_var,outcome_var,label_var Character, numeric, character.
#' Each component of the data to be plotted can be manually specified in case
#' the default columns in the data frame should not be used (if, for example, the values for a given
#' variable were altered and stored in a new column).
#' @param ymin,ymax Numeric.  Minimum and maximum values for y-axis. Defaults: 0, 100.
#' @param main_title Character.  Title of graph.  Default: None.
#' @param source_info Character.  Information on dataset used (country, years, version, etc.),
#' which is added to the bottom-left corner of the graph. Default: LAPOP ("Source: LAPOP Lab" will be printed).
#' @param subtitle Character.  Describes the values/data shown in the graph, e.g., "Percent who agree that...".
#' Default: None.
#' @param lang Character.  Changes default subtitle text and source info to either Spanish or English.
#' Will not translate input text, such as main title or variable labels.  Takes either "en" (English)
#' or "es" (Spanish).  Default: "en".
#' @param color_scheme Character.  Color of bars.
#' Takes hex numbers, beginning with "#". Default: "#008381".
#' @param order Logical.  Should bars be ordered from most frequent response to least?  Default: FALSE.
#' @return Returns an object of class \code{ggplot}, a ggplot bar graph.
#'
#' @examples
#' \donttest{
#' require(lapop); lapop_fonts()
#'
#' df <- data.frame(
#' cat = c("Far Left", 1, 2, 3, 4, "Center", 6, 7, 8, 9, "Far Right"),
#' prop = c(4, 3, 5, 12, 17, 23, 15, 11, 5, 4, 1),
#' proplabel = c("4%", "3%", "5%", "12%", "17%", "23%", "15%", "11%", "5%", "4%", "1%")
#' )
#' lapop_hist(df,
#'            main_title = "Centrists are a plurality among Peruvians",
#'            subtitle = "Distribution of ideological preferences",
#'            source_info = "Source: LAPOP Lab, AmericasBarometer Peru 2019",
#'            ymax = 27)
#'}
#'@export
#'@import ggplot2
#'@import ggtext
#'@importFrom stringr str_wrap
#'
#'@author Luke Plutowski, \email{luke.plutowski@@vanderbilt.edu} & Robert Vidigal, \email{robert.vidigal@@vanderbilt.edu}

lapop_hist <- function(data, outcome_var = data$prop, label_var = data$proplabel,
                       cat_var = data$cat,
                       ymin = 0,
                       ymax = 100,
                       lang = "en",
                       main_title = "",
                       subtitle = "",
                       source_info = "LAPOP",
                       order = FALSE,
                       color_scheme = "#008381"){
  if(order == TRUE){
    data = data[order(-data$prop), ]
    cat_var = cat_var[order(-outcome_var)]
    label_var = label_var[order(-outcome_var)]
    outcome_var = outcome_var[order(-outcome_var)]
  }
  update_geom_defaults("text", list(family = "inter")) # roboto
  ggplot(data, aes(x=factor(cat_var, levels = cat_var), y = outcome_var)) +
    geom_bar(stat = "identity", color = color_scheme, fill = paste0(color_scheme, "28"), width = 0.75) +
    geom_text(aes(label=label_var), vjust=-0.5, size = 5, fontface = "bold", color = color_scheme) +
    scale_x_discrete(labels = function(x) {
      stringr::str_wrap(x, width = 10)
    }) +
    scale_y_continuous(limits = c(ymin, ymax), expand = c(0, 0.3), labels = function(x) paste0(x, "%")) +
    labs(title=main_title,
         y = "",
         x = "",
         caption = paste0(ifelse(lang == "es" & source_info == "LAPOP", "Fuente: LAPOP Lab",
                                 ifelse(lang == "en" & source_info == "LAPOP", "Source: LAPOP Lab",
                                        source_info))),
         subtitle = subtitle) +
    theme(text = element_text(size = 14, family = "inter"), # roboto
          plot.title = element_text(size = 18, family = "inter", face = "bold"), # nunito
          plot.caption = element_text(size = 10.5, hjust = 0, vjust = 2, family = "inter", color="#585860"), # nunito
          plot.subtitle = element_text(size = 13, family = "inter-light", color="#585860", margin=margin(b = 10)), # nunito
          axis.title.y = element_blank(),
          axis.ticks = element_blank(),
          plot.title.position = "plot",
          plot.caption.position = "plot",
          axis.text = element_text(size = 14, family = "inter-light", color = "#585860"), # roboto
          panel.grid = element_line(color = "#dddddf"),
          panel.background = element_rect(fill = "white"),
          panel.grid.major.x = element_blank())
}

#######################################

# LAPOP Time-Series Graphs #

#######################################


#' @include lapop_fonts.R
NULL

#'
#' LAPOP Time-Series Graphs
#'
#' This function creates time series graphs using LAPOP formatting.  If there are waves missing at the
#' beginning or end of the series, the function will omit those waves from the graph (i.e.,
#' the x-axis will range from the earliest wave for which data is supplied to the latest).  If there are
#' waves missing in the middle of the series, those waves will be displayed on the x-axis, but no data will be
#' shown.
#'
#' The input data must have a specific format to produce a graph.  It must include columns for
#' the survey wave (wave), the outcome variable (prop), the lower bound of the estimate (lb),
#' the upper bound of the estimate (ub), and a string for the outcome variable label (proplabel).
#'
#' @param data Data Frame. Dataset to be used for analysis.  The data frame should have columns
#' titled wave (survey wave/year; character vector), prop (outcome variable; numeric),
#' proplabel (text of outcome variable; character); lb (lower bound of estimate; numeric),
#' and ub (upper bound of estimate; numeric). Default: None (must be supplied).
#' @param wave_var,outcome_var,label_var,lower_bound,upper_bound,point_var Character, numeric, character,
#' numeric, numeric, character. Each component of the data to be plotted can be manually specified in case
#' the default columns in the data frame should not be used (if, for example, the values for a given
#' variable were altered and stored in a new column).
#' @param ymin,ymax Numeric.  Minimum and maximum values for y-axis. Default: 0, 100.
#' @param main_title Character.  Title of graph.  Default: None.
#' @param source_info Character.  Information on dataset used (country, years, version, etc.),
#' which is added to the bottom-left corner of the graph. Default: LAPOP ("Source: LAPOP Lab" will be printed).
#' @param subtitle Character.  Describes the values/data shown in the graph, e.g., "Percent of Mexicans who agree...".
#' Default: None.
#' @param lang Character.  Changes default subtitle text and source info to either Spanish or English.
#' Will not translate input text, such as main title or variable labels.  #' Takes either "en" (English)
#' or "es" (Spanish).  Default: "en".
#' @param color_scheme Character.  Color of lines and dots.  Takes hex number, beginning with "#".
#' Default: "#A43D6A" (red).
#' @param percentages Logical.  Is the outcome variable a percentage?  Set to FALSE if you are using
#' means of the raw values, so that the y-axis adjusts accordingly. Default: TRUE.
#' @param label_vjust Numeric. Customize vertical space between points and their labels.
#' Default: -2.1.
#' @param max_years Numeric. Threshold for automatic x-axis label rotation. When the number of unique
#' country labels exceeds this value, labels will be smaller and if necessary rotated for better readability.
#' Default: 15 years.
#' @param label_angle Numeric. Angle (in degrees) to rotate x-axis labels when max_years is exceeded. Default: 0.
#' @param ci_type Character. Controls how confidence intervals are displayed in the
#' time-series plot. This parameter only affects how the confidence interval is visualized;
#' the point estimate and line plot remain unchanged. Options:
#' \itemize{
#'   \item \code{"linerange"} (default): Draws upper and lower bounds as dashed lines.
#'   \item \code{"errorbar"}: Displays confidence intervals using vertical error bars
#'         centered on the point estimate.
#'   \item \code{"ribbon"}: Shows a shaded confidence band between the lower and
#'         upper bounds.
#'   \item \code{"none"}: Suppresses confidence interval display.
#' }
#'
#' @return Returns an object of class \code{ggplot}, a ggplot line graph showing
#' values of a variable over time.
#'
#' @examples
#' \donttest{
#' require(lapop); lapop_fonts()
#'
#' df <- data.frame(wave = c("2008", "2010", "2016/17", "2018/19", "2021"),
#' prop = c(23.2, 14.4, 35.8, 36.6, 40),
#' proplabel = c("23.2%", "14.4%", "35.8%", "36.6%", "40.0%"),
#' lb = c(20.2, 11.9, 33.3, 33.1, 38),
#' ub = c(26.2, 16.9, 38.3, 40.1, 42)
#' )
#'
#' lapop_ts(df,
#' main_title = "Ecuadorians are becoming more interested in politics",
#' subtitle = "% politically interested",
#' source_info = "Source: LAPOP Lab, AmericasBarometer Ecuador 2006-2021",
#' ymin = 0,
#' ymax = 55)
#'}
#'@export
#'@import ggplot2
#'@importFrom ggtext element_markdown
#'@importFrom zoo na.approx
#'@import showtext
#'
#'@author Luke Plutowski, \email{luke.plutowski@@vanderbilt.edu} & Robert Vidigal, \email{robert.vidigal@@vanderbilt.edu}

lapop_ts <- function(data, outcome_var = data$prop, lower_bound = data$lb,
                     upper_bound = data$ub, wave_var = as.character(data$wave),
                     label_var = data$proplabel, point_var = data$prop,
                     ymin = 0, ymax = 100, main_title = "",
                     source_info = "LAPOP", subtitle = "",
                     lang = "en", color_scheme = "#A43D6A",
                     percentages = TRUE, label_vjust = -2.1,
                     max_years = 15, label_angle = 0,
                     ci_type = "linerange") {   # <<< NEW PARAM

  rotate_labels <- length(unique(data$wave)) > max_years
  label_size = 5
  current_label_size <- ifelse(rotate_labels, (label_size * 0.5), label_size)

  if(sum(is.na(outcome_var)) > 0) {
    outcome_var = zoo::na.approx(outcome_var)
    lower_bound = zoo::na.approx(lower_bound)
    upper_bound = zoo::na.approx(upper_bound)
  }

  ci_text = ifelse(lang == "es",
                   paste0(" <span style='color:", color_scheme,
                          "; font-size:18pt'> \u2013 \u2013 \u2013</span> ",
                          "<span style='color:#585860; font-size:13pt'>95% intervalo de confianza </span>"),
                   ifelse(lang == "fr",
                          paste0(" <span style='color:", color_scheme,
                                 "; font-size:18pt'> \u2013 \u2013 \u2013</span> ",
                                 "<span style='color:#585860; font-size:13pt'>Intervalle de confiance de 95% </span>"),
                          paste0(" <span style='color:", color_scheme,
                                 "; font-size:18pt'> \u2013 \u2013 \u2013</span> ",
                                 "<span style='color:#585860; font-size:13pt'>95% confidence </span>",
                                 "<span style='color:#585860'>interval</span>")))

  update_geom_defaults("text", list(family = "inter")) # roboto

  ts <- ggplot(data=data, aes(x=wave_var, y=outcome_var, group = 1)) +

    geom_line(color=color_scheme, linewidth = 1, alpha=0.48) +

  # CI TYPES
  # --------------------------------------------------------------
  {
    if (ci_type == "linerange") {
      list(
        geom_line(aes(y = lower_bound, group = 1),
                  color=color_scheme, linewidth = 1, alpha=0.48, linetype="dashed"),
        geom_line(aes(y = upper_bound, group = 1),
                  color=color_scheme, linewidth = 1, alpha=0.48, linetype="dashed")
      )
    } else if (ci_type == "errorbar") {
      geom_errorbar(aes(ymin = lower_bound, ymax = upper_bound),
                    width = 0.1, color = color_scheme, alpha = 0.75, linewidth = 1)
    } else if (ci_type == "ribbon") {
      geom_ribbon(aes(ymin = lower_bound, ymax = upper_bound),
                  fill = color_scheme, alpha = 0.48, color = NA)
    } else if (ci_type == "none") {
      NULL
    }
  } +

    geom_point(aes(y = point_var, color = " "), size = 3.5,
               alpha=0.48, key_glyph = "point") +

    scale_color_manual(values = color_scheme,
                       labels = paste0("<span style='color:#585860; font-size:13pt'> ",
                                       subtitle,
                                       "<span style='color:#FFFFFF00'>-----------</span>",
                                       ci_text)) +

    geom_text(aes(label = label_var), family = "inter", # roboto
              color=color_scheme, fontface="bold",
              size=5, vjust=label_vjust) +

    scale_x_discrete(limits = wave_var) +

    {
      if (percentages) {
        scale_y_continuous(limits=c(ymin, ymax),
                           breaks = seq(ymin, ymax,
                                        ifelse(ymax - ymin <= 50, 10, 20)),
                           labels = paste0(seq(ymin, ymax,
                                               ifelse(ymax - ymin <= 50, 10, 20)), "%"),
                           expand = c(0,0))
      } else {
        scale_y_continuous(limits=c(ymin, ymax), expand = c(0,0))
      }
    } +

    labs(title = main_title,
         caption = paste0(ifelse(lang == "es" & source_info == "LAPOP", "Fuente: LAPOP Lab",
                                 ifelse(lang == "en" & source_info == "LAPOP", "Source: LAPOP Lab",
                                        source_info))),
         x = " ", y = " ") +

    theme_minimal() +
    theme(text = element_text(size=14, family="inter"), # roboto
          plot.title = element_text(size=18, family="inter", face="bold"), # nunito
          plot.caption = element_text(size=10.5, vjust=2, hjust=0,
                                      family="inter", color="#585860"), # nunito
          axis.text = element_text(size = ifelse(rotate_labels, 10, 14),
                                   color="#585860"),
          panel.grid.major = element_line(color="#dddddf", linewidth=0.5),
          panel.grid.minor = element_line(color="#dddddf", linewidth=0.5),
          panel.border = element_rect(color="#dddddf", fill=NA, linewidth=1.0),
          legend.position="top",
          legend.title=element_blank(),
          legend.justification='left',
          legend.margin = margin(t=0, b=0, l=-40, r=0),
          legend.text = ggtext::element_markdown(family="inter-light")) # nunito-light

  if(rotate_labels) {
    ts <- ts +
      theme(axis.text.x = element_text(angle = label_angle,
                                       hjust = 0.5, vjust = 1)) +
      theme(plot.margin = margin(t=10, r=10,
                                 b=max(10, current_label_size*5), l=10))
  }

  return(ts)
}

utils::globalVariables(
  c("wave", "prop1", "prop2", "prop1_low", "prop1_upp", "pais", "ub1",
    "lb1", "ub2", "lb2", "pt", "write.csv", "complete.cases", "coef",
    "var", "as.formula", "qt", "combn", "xint",

    "Estimate", "Pr(>|t|)", "Std. Error", "conf.high", "conf.low",
    ".", "x_display", ":=", "label_table_names", "alpha_value",

    "geometry",    # sf object
    "value", # renamed user input variable to map

    "contrast", "hl_var", "lb", "note_id", "p.value", "prop",
    "prop2_low", "prop2_upp", "prop_low", "prop_upp", "proplabel",
    "proplabel1", "proplabel2", "pvalue", "strata", "ub", "upm",
    "vallabel", "varlabel", "varterm", "weight1500", "xvar_label",
    "year", "category")
)

#######################################

# LAPOP Dumbbell Graphs #

#######################################


#' @rdname lapop-deprecated
#' @section \code{lapop_db}:
#' For \code{lapop_db}, use \code{\link{lapop_dumb}}.
#'
#' @export

lapop_db <- function(data,
                     ymin = 0,
                     ymax = 100,
                     lang = "en",
                     main_title = "",
                     source_info = "",
                     subtitle = "",
                     sort = "wave2",
                     order = "hi-lo",
                     color_scheme = c("#482677", "#3CBC70"),
                     subtitle_h_just = 40,
                     subtitle_v_just = -18){
  .Deprecated("lapop_dumb")
  lapop_dumb(data = data,
             ymin = ymin,
             ymax = ymax,
             lang = lang,
             main_title = main_title,
             source_info = source_info,
             subtitle = subtitle,
             sort = sort,
             order = order,
             color_scheme = color_scheme,
             subtitle_h_just = subtitle_h_just,
             subtitle_v_just = subtitle_v_just)
}

#' @include lapop_fonts.R
NULL

#'
#' LAPOP Dummbell Graphs
#'
#' This function creates "dumbbell" graphs, which show averages for a variable across countries over two time periods, using LAPOP formatting.
#'
#' @param data Data Frame. Dataset to be used for analysis.  The data frame should have columns
#' titled pais (country name; character), wave1 (name of first wave/year (all rows are the same); character),
#' prop1 (outcome variable values for the first wave; numeric), proplabel1 (text of outcome variable for first wave; character),
#' wave2 (name of second wave/year (all rows are the same); character),
#' prop2 (outcome variable values for the second wave; numeric), proplabel2 (text of outcome variable for second wave; character).
#'  Default: None (must be supplied).
#' @param ymin,ymax Numeric.  Minimum and maximum values for y-axis. Defaults: 0 and 100.
#' @param main_title Character.  Title of graph.  Default: None.
#' @param source_info Character.  Information on dataset used (country, years, version, etc.),
#' which is added to the end of "Source: " in the bottom-left corner of the graph.
#' Default: LAPOP (only "Source: "LAPOP Lab will be printed).
#' @param subtitle Character.  Describes the values/data shown in the graph, e.g., "Percent who agree that...".
#' Default: None.
#' @param lang Character.  Changes default subtitle text and source info to either Spanish or English.
#' Will not translate input text, such as main title or variable labels.  Takes either "en" (English)
#' or "es" (Spanish).  Default: "en".
#' @param color_scheme Character.  Color of data points.  Must supply two values.  Takes hex numbers, beginning with "#".
#' Default: "#482677", "#3CBC70".
#' @param subtitle_h_just,subtitle_v_just Numeric.  Move the subtitle/legend text left/down (negative numbers) or right/up (positive numbers).
#' Ranges from -100 to 100.  Defaults: 40, -18.
#' @param sort Character.  The metric by which the data are sorted.  Options: "wave1" (outcome variable in first wave), "wave2" (outcome
#' variable in wave 2), "diff" (difference between the two waves), "alpha" (alphabetical by country name).
#' Default: "wave2".
#' @param order Whether data should be sorted from low to high or high to low on the sort metric.  Options: "hi-lo" and "lo-hi".
#' Default: "hi-lo".
#' @param text_nudge Numeric.  Move text of data further or closer to data point.  Default: 6.
#' @param drop_singles Logical.  Should rows with only one dot be removed?  Default: FALSE.
#' @return Returns an object of class \code{ggplot}, a ggplot figure showing
#' average values of some variable in two time periods across multiple countries
#'  (a dumbbell plot).
#'
#' @examples
#'\donttest{
#' require(lapop); lapop_fonts()
#' df <- data.frame(pais = c("Haiti", "Peru", "Honduras", "Colombia", "Ecuador",
#'                           "Panama", "Bolivia", "Argentina", "Paraguay",
#'                           "Dom. Rep.", "Brazil", "Jamaica", "Nicaragua",
#'                           "Guyana", "Costa Rica", "Mexico", "Guatemala",
#'                           "Chile", "Uruguay", "El Salvador"),
#'                  wave1 = rep("2018/19", 20),
#'                  prop1 = c(NA, 30, 58, 40, 49, 57, 33, 68, 38, 46, 30,
#'                            31, 70, NA, 43, 25, 38, 31, 34, 41),
#'                  proplabel1 = c(NA, "30%", "58%", "40%", "49%", "57%", "33%",
#'                                 "68%", "38%", "46%", "30%", "31%", "70%", NA,
#'                                 "43%", "25%", "38%", "31%", "34%", "41%"),
#'                  wave2 = rep("2021", 20),
#'                  prop2 = c(86, 73, 69, 67, 67, 65, 65, 65, 63, 62, 62,
#'                            57, 56, 56, 55, 55, 54, 51, 46, 42),
#'                  proplabel2 = c("86%", "73%", "69%", "67%", "67%", "65%", "65%",
#'                                 "65%", "63%", "62%", "62%", "57%", "56%", "56%",
#'                                 "55%", "55%", "54%", "51%", "46%", "42%"))
#'
#' lapop_dumb(df,
#' main_title = paste0("Personal economic conditions worsened across LAC"),
#'          subtitle = "% personal economic situation worsened",
#'          source_info = "Source: LAPOP Lab, AmericasBarometer 2018/19-2021")
#'}
#'
#'@export
#'@import ggplot2
#'@import ggtext
#'
#'@author Luke Plutowski, \email{luke.plutowski@@vanderbilt.edu} & Robert Vidigal, \email{robert.vidigal@@vanderbilt.edu}

lapop_dumb <- function(data,
                      ymin = 0,
                      ymax = 100,
                      lang = "en",
                      main_title = "",
                      source_info = "",
                      subtitle = "",
                      sort = "wave2",
                      order = "hi-lo",
                      color_scheme = c("#008381", "#A43D6A"),
                      subtitle_h_just = 40,
                      subtitle_v_just = -18,
                      text_nudge = 6,
                      drop_singles = FALSE){

  # ---- drop_singles fix ----
  if (drop_singles) {
    data$usable1 <- !is.na(data$prop1) & data$prop1 != 0
    data$usable2 <- !is.na(data$prop2) & data$prop2 != 0
    data$usable_both <- data$usable1 & data$usable2
    data <- data[data$usable_both, , drop = FALSE]
  }

  if(sort == "diff"){
    data$diff = data$prop2-data$prop1
    data = data[order(-data$diff),]
  } else if(sort == "wave2"){
    data = data[order(-data$prop2),]
  } else if(sort == "wave1"){
    data = data[order(-data$prop1),]
  } else if(sort == "alpha"){
    data = data[order(data$pais),]
  }

  if(order == "hi-lo"){
    data$pais = factor(data$pais, levels = rev(unique(data$pais)))
  } else if(order == "lo-hi") {
    data$pais = factor(data$pais, levels = unique(data$pais))
  }

  data$max1 = data$prop2 < data$prop1

  data$max1[is.na(data$max1)] <- FALSE

  names(color_scheme) = c(unique(data$wave1), unique(data$wave2))

  update_geom_defaults("text", list(family = "inter")) # roboto
    ggplot(data, aes(y=pais)) +
      # connecting segment ONLY when both values exist
      ggplot2::geom_segment(
        ggplot2::aes(x = prop1, xend = prop2, y = pais, yend = pais),
        color = "#bdbdc3",
        linewidth = 1.5,
        lineend = "round"
      ) +
      geom_point(aes(x = prop1, color = names(color_scheme)[1]),
                 size=4, na.rm=T) +
      geom_point(aes(x = prop2, color = names(color_scheme)[2]),
                 size=4, na.rm=T) +
      geom_text(data = data[data$max1 == FALSE, ],
                aes(x = prop1, label = proplabel1),
                nudge_x = -text_nudge, color = color_scheme[1],
                size = 5, fontface = "bold", na.rm=T) +
      geom_text(data = data[data$max1 == TRUE, ],
                aes(x = prop1, label = proplabel1),
                nudge_x = text_nudge, color = color_scheme[1],
                size = 5, fontface = "bold", na.rm=T) +
      geom_text(data = data[data$max1 == TRUE, ],
                aes(x = prop2, label = proplabel2),
                nudge_x = -text_nudge, color = color_scheme[2],
                size = 5, fontface = "bold", na.rm=T) +
      geom_text(data = data[data$max1 == FALSE, ],
                aes(x = prop2, label = proplabel2),
                nudge_x = text_nudge, color = color_scheme[2],
                size = 5, fontface = "bold", na.rm=T) +
      geom_vline(xintercept = 0, color = "#dddddf") +
      scale_x_continuous(limits=c(ymin, ymax), breaks=seq(ymin, ymax, 20),
                         labels = paste(seq(ymin,ymax, 20), "%", sep=""),
                         expand = c(0,0)) +
      scale_color_manual(values = color_scheme) +
      labs(title = main_title,
           y = "",
           x = " ",
           caption = paste0(ifelse(lang == "es" & source_info == "LAPOP", "Fuente: LAPOP Lab",
                                   ifelse(lang == "en" & source_info == "LAPOP", "Source: LAPOP Lab",
                            source_info))),
           subtitle = subtitle) +
      theme(text = element_text(size = 14, family = "inter"), # roboto
            plot.title = element_text(size = 17, family = "inter", face = "bold"), # nunito
            plot.caption = element_text(size = 10.5, hjust = 0, vjust = 2,
                                        family = "inter", color="#585860"), # nunito
            plot.subtitle = element_text(size = 14,
                                         family = "inter-light", color="#585860"), # nunito-light
            plot.margin = margin(5.5, 20, 5.5, 5.5, "points"),
            axis.title.y = element_blank(),
            axis.ticks = element_blank(),
            plot.title.position = "plot",
            plot.caption.position = "plot",
            axis.text = element_text(size = 12,
                                     family = "inter", # roboto
                                     color = "#585860",
                                     margin=margin(r=5)),
            panel.grid.major.y = element_line(color = "#dddddf"),
            panel.background = element_rect(fill = "white"),
            legend.position="top",
            legend.text = element_text(family = "inter", color = "#585860"), # roboto
            legend.title = element_blank(),
            legend.justification='right',
            legend.key.size = unit(1, "line"),
            legend.key = element_blank(),
            legend.margin = margin(t=subtitle_v_just,b=0, subtitle_h_just, 0))
}

#######################################

# LAPOP Multi-Line Time Series Graphs #

#######################################

#' @rdname lapop-deprecated
#' @section \code{lapop_tsmulti}:
#' For \code{lapop
#' _tsmulti}, use \code{\link{lapop_mline}}.
#'
#' @export

lapop_tsmulti <- function(data, varlabel = data$varlabel, wave_var = as.character(data$wave),
                          outcome_var = data$prop, label_var = data$proplabel,
                          point_var = data$prop,
                          ymin = 0,
                          ymax = 100,
                          main_title = "",
                          source_info = "",
                          subtitle = "",
                          lang = "en",
                          legend_h_just = 40,
                          legend_v_just = -20,
                          subtitle_h_just = 0,
                          color_scheme = c("#7030A0", "#3CBC70", "#1F968B", "#95D840", "")){
  .Deprecated("lapop_mline")
  lapop_mline(data = data, varlabel = varlabel, wave_var = wave_var,
              outcome_var = outcome_var, label_var = label_var,
              point_var = point_var,
              ymin = ymin,
              ymax = ymax,
              main_title = main_title,
              source_info = source_info,
              subtitle = subtitle,
              lang = lang,
              legend_h_just = legend_h_just,
              legend_v_just = legend_v_just,
              subtitle_h_just = subtitle_h_just,
              color_scheme = color_scheme
  )
}

#' @include lapop_fonts.R
NULL

#'
#' LAPOP Multi-line Time-Series Graphs
#'
#' This function creates a time series graph utilizing multiple lines representing values of
#' an outcome variable for different values of a secondary variable -- for example, support for
#' democracy over time by country.  This function is designed to be used for
#' AmericasBarometer data.  The maximum number of lines is four.  Unlike the lapop_ts()
#' single-line time series graph, this function will not print confidence lines nor will
#' it show text values for each year (just the final/most recent year).
#'
#'
#' @param data Data Frame. Dataset to be used for analysis.  The data frame should have columns
#' titled varlabel (values of secondary variable which will be used to make each line; character),
#' wave (survey wave/year; character), prop (outcome variable; numeric),
#' proplabel (text of outcome variable; character). Default: None (must be supplied).
#' @param varlabel,wave_var,outcome_var,label_var,point_var Character,
#' character, numeric, character, numeric. Each component of the data to be plotted
#' can be manually specified in case the default columns in the data frame should
#' not be used (if, for example, the values for a given variable were altered
#' and stored in a new column).
#' @param ymin,ymax Numeric.  Minimum and maximum values for y-axis. Default: 0, 100.
#' @param main_title Character.  Title of graph.  Default: None.
#' @param source_info Character.  Information on dataset used (country, years, version, etc.),
#' which is added to the end of "Source: " in the bottom-left corner of the graph.
#' Default: None (only "Source: " will be printed).
#' @param subtitle Character.  Describes the values/data shown in the graph, e.g., "Percent of Mexicans who agree...".
#' Default: None.
#' @param lang Character.  Changes default subtitle text and source info to either Spanish or English.
#' Will not translate input text, such as main title or variable labels.  #' Takes either "en" (English)
#' or "es" (Spanish).  Default: "en".
#' @param legend_h_just,legend_v_just Numeric.  Changes location of legend. From 0 to 100.
#'  (secondary variable labels).  Defaults: 40, -20.
#' @param subtitle_h_just Numeric.  Moves subtitle left to right.  From 0 to 1.
#'  (secondary variable labels).  Defaults: 0 (left justify).
#' @param color_scheme Character.  Color of lines and dots.  Takes hex number, beginning with "#".
#' Must specify four values, even if four are not used.
#' Default: c("#784885", "#008381", "#c74e49", "#2d708e", "#a43d6a", "#202020").
#' @param percentages Logical.  Is the outcome variable a percentage?  Set to FALSE if you are using
#' means of the raw values, so that the y-axis adjusts accordingly. Default: TRUE.
#' @param all_labels Logical.  If TRUE, show text above all points, instead of only those in the most recent wave. Default: FALSE.
#' @param ci Logical.  Add "tie fighter" confidence intervals.  Only recommended when each line represents a different variable.
#' @param legendnrow Numeric.  How many rows for legend labels. Default: 1.
#' @return Returns an object of class \code{ggplot}, a ggplot line graph showing values of a variable over time.
#'
#'@examples
#'\donttest{
#' df <- data.frame(varlabel = c(rep("Honduras", 9), rep("El Salvador", 9),
#'                               rep("Mexico", 9), rep("Guatemala", 9)),
#'                  wave = rep(c("2004", "2006", "2008", "2010", "2012",
#'                               "2014", "2016/17", "2018/19", "2021"), 4),
#'                  prop = c(19, 24, 21, 15, 11, 32, 41, 38, 54,
#'                           29, 29, 25, 24, 24, 28, 36, 26, 32,
#'                           14, 16, 14, 16, 9, 14, 18, 19, 26,
#'                           21, 15, 18, 20, 14, 18, 17, 25, 36),
#'                  proplabel = c("19%", "24%", "21%", "15%", "11%", "32%",
#'                                "41%", "38%", "54%",
#'                                "29%", "29%", "25%", "24%", "24%", "28%",
#'                                "36%", "26%", "32%",
#'                                "14%", "16%", "14%", "16%", "9%", "14%",
#'                                "18%", "19%", "26%",
#'                                "21%", "15%", "18%", "20%", "14%", "18%",
#'                                "17%", "25%", "36%"))
#' require(lapop); lapop_fonts()
#' lapop_mline(df,
#'       main_title = "Intentions to emigrate in Guatemala, Honduras and Mexico reached their highs",
#'       subtitle = "% who intend to migrate in:",
#'       source_info = ", AmericasBarometer 2004-2021")
#'}
#'@export
#'@import ggplot2
#'@importFrom ggtext element_markdown
#'@importFrom ggrepel geom_text_repel
#'@importFrom zoo na.approx
#'@importFrom stats na.omit
#'@import showtext
#'@import dplyr
#'
#'@author Luke Plutowski, \email{luke.plutowski@@vanderbilt.edu} & Robert Vidigal, \email{robert.vidigal@@vanderbilt.edu}

lapop_mline <- function(data, varlabel = data$varlabel, wave_var = as.character(data$wave),
                        outcome_var = data$prop, label_var = data$proplabel,
                        point_var = data$prop,
                        ymin = 0,
                        ymax = 100,
                        main_title = "",
                        source_info = "",
                        subtitle = "",
                        lang = "en",
                        legend_h_just = 40,
                        legend_v_just = -20,
                        subtitle_h_just = 0,
                        color_scheme = c("#784885", "#008381", "#c74e49", "#2d708e", "#a43d6a", "#202020"),
                        percentages = TRUE,
                        all_labels = FALSE,
                        ci = FALSE,
                        legendnrow = 1){
  if(!inherits(varlabel, "character") & !inherits(varlabel, "factor")){
    varlabel = as.character(varlabel)
    data$varlabels = as.character(data$varlabel)
  }
  #interpolate data for missing waves are still plotted on the x-axis (without data)
  if(sum(is.na(outcome_var)) > 0) {
    outcome_var = data %>%
      group_by(varlabel) %>%
      mutate(first = which(prop == dplyr::first(na.omit(prop))),
             prop = ifelse(is.na(prop) & wave != max(wave) & wave != min(wave) & seq_along(wave) >= first,
                           zoo::na.approx(prop, na.rm = FALSE), prop)) %>%
      ungroup() %>%
      pull(prop)
  }
  varlabel = factor(varlabel, levels = unique(varlabel))
  mycolors = color_scheme[seq_along(unique(varlabel))]
  textcolors = rep(mycolors, each = length(unique(wave_var)))
  end_labels = data %>%
    group_by(varlabel) %>%
    mutate(last = max(which(prop == dplyr::last(na.omit(prop)))),
           end_labels = ifelse(seq_along(wave) == last, proplabel, NA)) %>%
    ungroup() %>%
    pull(end_labels)
  update_geom_defaults("text", list(family = "inter")) # roboto
  ggplot(data, aes(x = wave_var, y = outcome_var, group = varlabel, color = varlabel)) +
    geom_line(aes(color = varlabel), linewidth = 1, alpha=0.48, show.legend = FALSE) +
    {if(ci == TRUE){
      geom_errorbar(aes(ymin=lb, ymax=ub), width = 0.2, show.legend = FALSE)
      }
    } +
    geom_point(aes(y = point_var, color = varlabel), size = 3.5, alpha=0.48,
               key_glyph = draw_key_blank) +
    scale_color_manual(breaks = levels(varlabel),
                       labels = paste("<span style='color:",
                                      mycolors,
                                      "'>",
                                      levels(varlabel),
                                      "</span>"),
                       values = mycolors,
                       guide=guide_legend(nrow = legendnrow)) +

    scale_x_discrete(limits = unique(wave_var),
                     expand = expansion(mult = c(0.02, 0.06))) +
    coord_cartesian(clip = "off") +

    {if(all_labels){
      geom_text(aes(label=label_var, color = varlabel),
                family = "inter", fontface = "bold", size = 5, vjust = -2,
                show.legend = FALSE) # roboto
    }
      else{
        ggrepel::geom_text_repel(
          aes(label = end_labels), # fontface can be set outside aes
          fontface = "bold",
          color = textcolors,
          family = "inter", # roboto
          size = 4.5,
          nudge_x = 0.35, # small nudge to the right
          direction = "y",
          segment.color = NA # hides the “continuation” line
        )
      }
    } +
    {
      if (percentages) {
        scale_y_continuous(limits=c(ymin, ymax),
                           breaks=seq(ymin, ymax, 10),
                           labels = paste(seq(ymin, ymax, 10), "%", sep=""),
                           expand = c(0,0))
      }
      else {
        scale_y_continuous(limits=c(ymin, ymax),
                           expand = c(0,0))
      }
    } +
    labs(title = main_title,
         caption = paste0(ifelse(lang == "es", "Fuente: LAPOP Lab", "Source: LAPOP Lab"),
                          source_info),
         x = " ",
         y = " ",
         subtitle = subtitle) +
    theme_minimal() +
    theme(text = element_text(size = 14, family = "inter"), # roboto
          plot.title = element_text(size = 18, family = "inter", face = "bold"), # nunito
          plot.caption = element_text(size = 10.5, vjust = 2, hjust = 0, family = "inter", color="#585860"), # nunito
          plot.subtitle = element_text(size = 14, family = "inter-light", color="#585860", hjust = subtitle_h_just), # nunito-light
          axis.title.y = element_blank(),
          axis.text = element_text(size = 14, color = "#585860"),
          panel.grid = element_line(color = "#dddddf"),
          panel.border = element_rect(linetype = "solid", color = "#dddddf", fill = NA),
          legend.position = "top",
          plot.title.position = "plot",
          plot.caption.position = "plot",
          legend.title = element_blank(),
          legend.justification='right',
          legend.margin = margin(t=legend_v_just,b=0, legend_h_just, 0),
          legend.spacing.x = unit(0.2, 'cm'),
          legend.text=element_markdown(family = "inter", face = "bold")) # nunito
}

#######################################

# LAPOP Rescale #

#######################################

#' LAPOP Rescale
#'
#' This function allows users to rescale and reorder variables.  It is designed
#' for variables of class "labelled" but the rescaling will work for numeric
#' and factor variables too.
#'
#' @param var Vector (class "labelled" or "haven_labelled").  The original variable
#' to rescale.
#' @param min Integer. Minimum value for the new rescaled variables; default is 0.
#' @param max Integer. Maximum value for the new rescaled variables; default is 1.
#' @param reverse Logical.  Reverse code the variable before rescaling. Default: FALSE.
#' @param only_reverse Logical.  Reverse code the variable, but do not rescale. Default: FALSE.
#' @param only_flip Logical. Flip the variable coding.  Unlike "only_reverse", this will
#' exactly preserve the values of the old variable.  For example, for a variable
#' with codes 1, 2, 3, 5, 10, only_flip will code the values 10, 5, 3, 2, 1 (instead
#' of 10, 9, 8, 6, 1).  Generally, reverse should be preferred to preserve the
#' underlying scale.  Not compatible with rescale. Default: FALSE.
#' @param map Logical. If TRUE, will print a cross-tab showing the old variable
#' and the new, recoded variable.  Used to verify the new variable is coded correctly.
#' Default: FALSE.
#' @param new_varlabel Character.  Variable label for the new variable.
#' Default: old variable's label.
#' @param new_vallabels Character vector. Supply custom names for value labels. Default:
#' value labels of old variable.
#' @return The input variable rescaled
#'
#' @examples
#'
#' require(lapop); data(ym23)
#'
#' # Regular data.frame
#' ym23$pn4r <- lpr_resc(ym23$pn4,
#' reverse = TRUE,
#' map = TRUE)
#'
#' # LPR data.frame
#' ym23lpr<-lpr_data(ym23)
#' ym23lpr$variables$pn4r <- lpr_resc(ym23lpr$variables$pn4,
#' reverse = TRUE,
#' map = TRUE)
#'
#'@export
#'@import haven
#'
#'@author Luke Plutowski, \email{luke.plutowski@@vanderbilt.edu} & Robert Vidigal, \email{robert.vidigal@@vanderbilt.edu}

lpr_resc <- function(var,
                     min = 0L,
                     max = 1L,
                     reverse = FALSE,
                     only_reverse = FALSE,
                     only_flip = FALSE,
                     map = FALSE,
                     new_varlabel = NULL,
                     new_vallabels = NULL) {

  og_var <- var

  if (is.character(var)) {
    stop("Cannot rescale character variable.")
  }

  # Extract labels if haven_labelled
  if (inherits(var, c("haven_labelled", "labelled"))) {
    original_labels <- attr(haven::zap_missing(var), "labels")
  }

  # Convert factor to numeric
  if (is.factor(var)) {
    original_levels <- levels(var)
    var <- as.numeric(var)
  }

  # Internal reverse helper
  reverse_values <- function(x) {
    if (all(is.na(x))) return(x)
    rng <- range(x, na.rm = TRUE)
    reversed <- (rng[2] + rng[1]) - x
    return(reversed)
  }

  # Reverse logic
  if (reverse || only_reverse) {
    var <- reverse_values(var)
  } else if (only_flip) {
    unique_vals <- sort(unique(var))
    flipped_vals <- rev(unique_vals)
    var <- flipped_vals[match(var, unique_vals)]

    # Flip labels if available
    if (exists("original_labels")) {
      flipped_labels <- rev(original_labels)
      names(flipped_labels) <- names(original_labels)
      attr(var, "labels") <- flipped_labels
    }
  } else {
    # Rescale to [min, max]
    var <- (var - min(var, na.rm = TRUE)) /
      (max(var, na.rm = TRUE) - min(var, na.rm = TRUE)) *
      (max - min) + min
  }

  # Optional mapping table
  if (map) {
    print(table(haven::as_factor(haven::zap_missing(og_var)),
                haven::as_factor(haven::zap_missing(var))))
  }

  # Update variable label
  if (!is.null(new_varlabel)) {
    attr(var, "label") <- as.character(new_varlabel)
  }

  # Update value labels
  if (!is.null(new_vallabels) && exists("original_labels")) {
    names(original_labels) <- new_vallabels
    attr(var, "labels") <- original_labels
  }

  return(var)
}

#######################################

# LAPOP Visualization Templates #

#######################################

.lapop_font_paths <- function(pkg = "lapop") {
  regular_path <- system.file("fonts", "Inter-Regular.ttf", package = pkg)
  light_path <- system.file("fonts", "Inter-Light.ttf", package = pkg)

  if (identical(regular_path, "") || identical(light_path, "")) {
    local_fonts_dir <- file.path(getwd(), "inst", "fonts")
    regular_fallback <- file.path(local_fonts_dir, "Inter-Regular.ttf")
    light_fallback <- file.path(local_fonts_dir, "Inter-Light.ttf")

    if (file.exists(regular_fallback) && file.exists(light_fallback)) {
      regular_path <- regular_fallback
      light_path <- light_fallback
    }
  }

  list(
    regular = regular_path,
    light = light_path
  )
}

.lapop_register_fonts <- function(pkg = "lapop") {
  font_paths <- .lapop_font_paths(pkg = pkg)

  if (!file.exists(font_paths$regular) || !file.exists(font_paths$light)) {
    stop("Bundled Inter font files were not found.")
  }

  existing_families <- sysfonts::font_families()

  if (!("inter" %in% existing_families)) {
    sysfonts::font_add(family = "inter", regular = font_paths$regular)
  }

  if (!("inter-light" %in% existing_families)) {
    sysfonts::font_add(family = "inter-light", regular = font_paths$light)
  }

  if ("register_font" %in% getNamespaceExports("systemfonts")) {
    try(
      systemfonts::register_font(
        name = "inter",
        plain = font_paths$regular,
        bold = font_paths$regular,
        italic = font_paths$regular,
        bolditalic = font_paths$regular
      ),
      silent = TRUE
    )
    try(
      systemfonts::register_font(
        name = "inter-light",
        plain = font_paths$light,
        bold = font_paths$light,
        italic = font_paths$light,
        bolditalic = font_paths$light
      ),
      silent = TRUE
    )
  }

  invisible(TRUE)
}

#' LAPOP Fonts
#'
#' This function loads fonts needed for LAPOP graph formatting.
#' No arguments needed; just run lapop_fonts() at the beginning of your session.
#'
#'@return No return value, called for side effects
#'@author Luke Plutowski, \email{luke.plutowski@@vanderbilt.edu} & Robert Vidigal, \email{robert.vidigal@@vanderbilt.edu}
#'
#'@examples
#'\donttest{require(lapop); lapop_fonts()}
#'
#'@import showtext sysfonts
#'@export
lapop_fonts <- function() {
  .lapop_register_fonts()
  showtext::showtext_auto(enable = TRUE)
  message("LAPOP fonts loaded successfully: Inter and Inter Light.")
  invisible(TRUE)
}

###############################################################

# LAPOP Extract Variables Labels from AmericasBarometer Notes #

###############################################################

#' Extract Notes from AmericasBarometer Attributes
#'
#' Extracts notes stored in a dataset's attributes and organizes them into a tidy data frame.
#' This function is particularly useful for processing Stata datasets imported into R that
#' contain variable notes in their attributes.
#'
#' @param data A dataset (data frame) containing "expansion.fields" in its attributes.
#'
#' @return A data frame with three columns:
#' \describe{
#'   \item{variable_name}{Name of the variable the note belongs to}
#'   \item{note_id}{Identifier for the note}
#'   \item{note_value}{The actual note text}
#' }
#'
#' @details
#' This function processes the attributes of a dataset to extract notes that are typically
#' stored in a specific format. It skips any notes associated with "_dta" (dataset-level notes)
#' and only returns variable-specific notes. The function expects the notes to be organized
#' as a list where each element contains exactly three components: variable name, note ID,
#' and note value.
#'
#' @examples
#'\donttest{
#' require(lapop); data(bra23)
#'
#' # Extract the notes
#' notesBRA23 <- lpr_extract_notes(bra23)
#' tail(notesBRA23[notesBRA23$variable_name=="ing4",]) # for ing4 variable
#'}
#' @export

lpr_extract_notes <- function(data) { # Extract notes from expansion.fields
  data = attr(data, "expansion.fields")

  # DF for output
  notes_df <- data.frame(variable_name = character(),
                         noteid = character(),
                         note_value = character(),
                         stringsAsFactors = FALSE)

  for (i in seq_along(data)) {

    sublist <- data[[i]] # object is a nested list

    if (length(sublist) == 3) {

      variable_name <- sublist[[1]]

      noteid <- sublist[[2]]

      note_value <- sublist[[3]]

       if (variable_name!="_dta") { # Add the row to notes_df
        notes_df <- rbind(notes_df, data.frame(variable_name = variable_name,
                                               note_id = noteid,
                                               note_value = note_value,
                                               stringsAsFactors = FALSE))
      }
    }
  }

  notes_df<-subset(notes_df, variable_name!="_dta")

  return(notes_df)

}

#######################################

# LAPOP Visualization Templates #

#######################################
#' LAPOP Fonts (design)
#'
#' This function loads fonts needed for LAPOP graph formatting.  In contrast to lapop_fonts(),
#' this renders text as text instead of polygons, which allows post-hoc editing.
#'
#'@return No return value, called for side effects
#'@author Luke Plutowski, \email{luke.plutowski@@vanderbilt.edu}
#'
#'@import sysfonts
#'@import systemfonts
#'@import showtext
#'@export

 lapop_fonts_design <- function() {
   .lapop_register_fonts()
   showtext::showtext_auto(enable = FALSE)
   invisible(TRUE)
}

#' cm23: Single-country Multi-year Dataset
#'
#' A dataset containing the AmericasBarometer Brazil  Country Merge up to 2023.
#'
#' @format A data frame
#' \describe{
#'  \item{ing4}{Support for Democracy}
#'  \item{b13}{Trust in the National Legislature}
#'  \item{b21}{Trust in Political Parties}
#'  \item{b31}{Trust in the Supreme Court of Justice}
#'  \item{wave}{Survey round year for regional or multi-country data}
#'  \item{pais}{Country of survey}
#'  \item{year}{Survey round year for single-country data}
#'  \item{upm}{Primary Sampling Unit}
#'  \item{strata}{Stratification}
#'  \item{weight1500}{Cross-country and cross-time weight}
#' }
#' @source LAPOP AmericasBarometer (https://www.vanderbilt.edu/lapop/)
#' @export
cm23 <- load("data/cm23.rda")

## lapop-deprecated.r
#' @title Deprecated functions in package \pkg{lapop}.
#' @description The functions listed below are deprecated and will be defunct in
#'   the near future. When possible, alternative functions with similar
#'   functionality are also mentioned. Help pages for deprecated functions are
#'   available at \code{help("<function>-deprecated")}.
#' @name lapop-deprecated
#' @keywords internal
#' @return No return value, called for side effects
NULL

#' ym23: Multi-country Single-year Dataset
#'
#' A dataset containing the AmericasBarometer Year Merge of 2023.
#'
#' @format A data frame
#' \describe{
#' \item{b12}{Trust in Armed Forces}
#' \item{b18}{Trust in National Police}
#' \item{ing4}{Support for Democracy}
#' \item{pn4}{Satisfaction with Democracy}
#' \item{vb21n}{Influence Political Change}
#' \item{q14f}{Migration Intentions}
#' \item{edre}{Education}
#' \item{wealth}{Wealth}
#' \item{q1tc_r}{Gender}
#' \item{upm}{Primary Sampling Unit}
#' \item{strata}{Stratification}
#' \item{wave}{Survey round year for regional or multi-country data}
#' \item{pais}{Country of survey}
#' \item{year}{Survey round year for single-country data}
#' \item{weight1500}{Cross-country and cross-time weight}
#' }
#' @source LAPOP AmericasBarometer (https://www.vanderbilt.edu/lapop/)  # Ensure this line has a valid source.
#' @export
#'
ym23 <- load("data/ym23.rda")

---
title: "LAPOP Visualization Guide"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{LAPOP Visualization Guide}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
description: >
  This vignette provides examples of how to use the LAPOP R package to
  create weighted visualizations with properly labeled facets.  
lang: "en-US" 
---

```{r fig setup, include=FALSE}

knitr::opts_chunk$set(echo = TRUE, message = FALSE, warning = FALSE)
knitr::opts_chunk$set(fig.width = 9, fig.height = 5)

```

## Introduction

This vignette demonstrates how to use the `lapop` package to reproduce figures style from the 2023 *Pulse of Democracy* report. 

Attention: we are only using a subset from the data used in the actual report.

[Read the full report here](https://www.vanderbilt.edu/lapop/ab2023/AB2023-Pulse-of-Democracy-final-20240604.pdf)

## Load Packages

```{r, lapop package}
library(lapop)
```

The lapop package brings two datasets already with design effects (weighted).

```{r, load data}
# Single-country Single-year AmericasBarometer 2023 Brazil
data(bra23)

# AmericasBarometer 2006-2023 Brazil Merge
data(cm23)

# AmericasBarometer 2023 Year Merge
data(ym23)
```

### Survey Design for the Data

The AmericasBarometer use `weight1500` for comparative datasets by time or country, `wt` weights are only for country specific datasets (single-country single-year). In the AmericasBarometer the primary sampling units (ids = upm) and stratification variable (strata = strata) to account for the survey’s complex design. Finally, PSUs are nested within strata, properly reflecting the survey’s hierarchical sampling structure. The `lpr_data()` function apply survey design effects. 

```{r, weight datasets, eval = TRUE}
# Single-Country Single-Year
bra23w <- lpr_data(bra23, wt = TRUE)

print(bra23w)

# Single-Country Multi-Year
cm23w <- lpr_data(cm23)

print(cm23w)

# Multi-Country Single-Year
ym23w <- lpr_data(ym23)

print(ym23w)
```

## Fonts for 2023 *Pulse of Democracy* report. 
When you load `lapop` with `library(lapop)`, the package now auto-registers the bundled LAPOP fonts and enables the standard plotting setup. You can still call `lapop::lapop_fonts()` manually if you want to re-run font registration in the current session.

```{r, lapop fonts, eval=F}
lapop::lapop_fonts() # Re-run font registration explicitly if needed
```

## Figure 1.1: Time Series

```{r, time series, eval = TRUE, fig.alt = "Support for democracy decline a decade ago and remains comparatively low in Brazil"}
fig1.1_data <- lpr_ts(cm23w, outcome = "ing4", rec = c(5, 7), use_wave = TRUE)

lapop_ts(fig1.1_data,
         ymin = 50,
         ymax = 80,
         main_title = "Support for democracy decline a decade ago and remains comparatively low in Brazil",
         subtitle = "% who support democracy")
```

## Figure 1.2: Cross-Country Comparison

```{r, cross country, eval = TRUE, fig.alt = "In many countries, only about one in two adults support democracy"}
fig1.2_data <- lpr_cc(ym23w, outcome = "ing4", rec = c(5, 7))

lapop_cc(fig1.2_data,
         main_title = "In many countries, only about one in two adults support democracy",
         subtitle = "% who support democracy")
```

## Figure 2.1: Multi-line Over Time

```{r, mline, eval = TRUE, warnings=F, fig.alt = "Trust in executives has declined to a level similar to other political institutions"}
fig2.1_data <- lpr_mline(cm23w,
                         outcome = c("b13", "b21", "b31"),
                         rec = c(5, 7), rec2 = c(5, 7), rec3 = c(5, 7))

# Changing legend for readibility
fig2.1_data$varlabel <- ifelse(fig2.1_data$varlabel == "b13", "National Legislature", 
                          ifelse(fig2.1_data$varlabel=="b21", "Political Parties",
                                 "Supreme Court"))

lapop_mline(fig2.1_data,
            main_title = "Trust in executives has declined to a level similar to other \npolitical institutions",
            subtitle = "% who trust...")
```

## Figure 2.3: CC Multiple Variables
```{r, ccm, warnings = FALSE, eval = TRUE, fig.alt = "The public typically reports more trust in the armed forces than the police, but levels vary"}

fig2.3_data <- lpr_ccm(ym23w,
                       outcome_vars = c("b12", "b18"),
                       rec = c(5, 7),
                       rec2 = c(5, 7))

# Changing legend for readibility
fig2.3_data$var <- ifelse(fig2.3_data$var == "b12", "Armed Forces", "National Police")

lapop_ccm(fig2.3_data,
          main_title = "The public typically reports more trust in the armed forces than the police,\nbut levels vary",
          subtitle = "% who trust...")

```

## Figure Mover Example: Satisfaction with Democracy

```{r, mover, eval = TRUE, fig.alt = "Satisfaction with democracy is significantly lower among women, those with higher educational attainment, and the middle class"}
library(dplyr)

ym23w$variables <- ym23w$variables %>%
    mutate(
    edrer = as.numeric(edre),
    gender = as.numeric(q1tc_r)
  ) %>%
  mutate(edrer = case_when(
    edrer <= 2 ~ "None/Primary",
    edrer %in% c(3, 4) ~ "Secondary",
    edrer > 4 ~ "Superior",
    TRUE ~ NA_character_
  ), 
  edrer = factor(edrer, levels = c("None/Primary", "Secondary", "Superior")),
  gender = ifelse(q1tc_r == 3, NA, q1tc_r)) %>%
  mutate(gender = case_when(
    gender == 1 ~ "Men",
    gender == 2 ~ "Women",
     TRUE ~ NA_character_
  ))

# Changing legend for readibility 
attributes(ym23w$variables$gender)$label <- "Gender"
attributes(ym23w$variables$edrer)$label <- "Education"
attributes(ym23w$variables$wealth)$label <- "Wealth"

fig_mover <- lpr_mover(ym23w,
                       outcome = "pn4",
                       grouping_vars = c("gender", "edrer", "wealth"),
                       rec = c(1, 2)) 

lapop_mover(fig_mover,
            main_title = "Satisfaction with democracy is significantly lower among women,\nthose with higher educational attainment, and the middle class",
            subtitle = "% who are satisfied with democracy")
```

## Figure Histogram: Citizens’ Views

```{r, histogram, eval = TRUE, fig.alt = "On average in the LAC region, one in three say voting is the best way to influence change"}
spot32_data <- lpr_hist(ym23w, outcome = "vb21n")

# Translating to English
spot32_data$cat <- c("Vote", "Run for\noffice", 
                     "Protest", 
                     "Participate\nin local orgs.", 
                     "Other", 
                     "Change is\nimpossible")

lapop_hist(spot32_data,
           main_title = "On average in the LAC region, one in three say voting is the best way \nto influence change",
           subtitle = "How do you believe you can best influence change?")
```

## Figure 3.5: Dumbell Plot

```{r, dumbell, eval = TRUE, fig.alt = "Among those with emigration intentions, the percentage who say they are very likely to emigrate increased in Nicaragua and Guatemala"}
fig3.5_data <- lpr_dumb(ym23w,
                        outcome = "q14f",
                        rec = c(1, 1),
                        over = c(2018, 2023),
                        xvar = "pais",
                        ttest = TRUE)

# Select Countries
fig3.5_data <- fig3.5_data[c(1:3,5:6), ]

lapop_dumb(fig3.5_data,
           main_title = "Among those with emigration intentions, the percentage who say they are \nvery likely to emigrate increased in Nicaragua and Guatemala",
           subtitle = "% who say it is very likely they will emigrate",
           source = "LAPOP Lab, AmericasBarometer 2018/19 and 2023")
```

## Figure 3.8: Stack Plot by Country

```{r, stack, eval= TRUE, fig.alt = "Nicaragua has the highest percentage of individuals with migration intentions, while Haiti has the lowest"}
attributes(ym23w$variables$q14f)$label <- "Migration intentions"

fig3.8_data <- lpr_stack(ym23w,
                         outcome = "q14f",
                         xvar = "pais",
                         order = "hi-lo")

lapop_stack(fig3.8_data,
            xvar = "xvar_label",
            source = "LAPOP Lab, AmericasBarometer 2023",
            main_title = "Nicaragua has the highest percentage of individuals with migration intentions,\nwhile Haiti has the lowest")
```

## Bonus: Reverse Variables
According to the LAPOP Editorial Guidelines, when variables are measured on a scale, reversing them can enhance clarity and coherence in data representation. For instance, if a survey question aims to measure positive sentiment but higher numeric values indicate negative responses, reversing the variable aligns the coding with the expected direction of responses. This makes it easier for readers to understand the findings.

```{r, reverse, eval = TRUE, warnings=F}
# For data.frames
cm23$ing4r <- lpr_resc(cm23$ing4, reverse = TRUE, map=TRUE)

# For LPR_DATA() objects
cm23w$variables$ing4r <- lpr_resc(cm23w$variables$ing4, reverse = TRUE, map=TRUE)
```

## End

This vignette is intended as a **template and starting point** for your own 
reports and exploratory visualizations using LAPOP data. 

By adapting these examples, you can produce **consistent, high-quality charts** 
that follow the LAPOP Lab’s editorial guidelines and communicate survey results effectively.

For further guidance:

- **Function documentation:** See the package  help pages for detailed arguments 
and examples (e.g., `?lpr_cc`, `?lapop_cc`).
- **Editorial guidelines:** Follow LAPOP Lab’s recommendations for plot labeling 
to ensure clarity and comparability.
- **More articles:** Check other vignettes and the [pkgdown site](https://lapop-central.github.io/lapop/) for additional 
tutorials and updates.

If you encounter any issues or have suggestions for improvement, 
please let us know via our [GitHub issues page](https://github.com/lapop-central/lapop/issues).

Thank you for using the **LAPOP R package** to help make high-quality, 
reproducible social science research easier and more accessible!

---
title: "AmericasBarometer Labels Guide"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{AmericasBarometer Labels Guide}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
description: >
  This vignette provides examples of how to use lapop helper functions to
  access AmericasBarometer question wording and response option labels in R.  
lang: "en-US" 
---

```{r fig setup, include=FALSE}

knitr::opts_chunk$set(echo = TRUE, message = FALSE, warning = FALSE)
knitr::opts_chunk$set(fig.width = 12, fig.height = 6)
```

## Introduction
This concise guide demonstrates how R users can effectively work with AmericasBarometer datasets to access variable metadata directly eliminating the need for external codebooks. While these features are more readily visible in Stata, they often require specific approaches in R. The guide outlines:

1. The multilingual structure and storage of AmericasBarometer data (including variable labels and value labels).

2. How to implement our custom LAPOP Lab functions for seamless dataset analysis.

## Installing LAPOP Lab package

The `lapop` package contains custom functions to analyze data and plot figures following the LAPOP Lab's guidelines.

If you do not have the LAPOP Lab package already installed in your machine you can install it using the `devtools` package below:

```{r, INSTALL LPR, eval = FALSE}
# Install the LAPOP Lab package:
devtools::install_github("https://github.com/lapop-central/lapop", 
                         force=TRUE, 
                         build_vignettes = TRUE)
```

After installation is successful, you will need to load the package into your library:

```{r, LOAD LPR}
library(lapop)
# Note: no need for lapop_fonts() 
# because we are not plotting figures
```

## How LAPOP Lab stores variables metadata
LAPOP Lab's AmericasBarometer stores variable labels and value labels in parallel languages (i.e., Spanish/English/Portuguese) directly within the dataset structure. This allows researchers to access metadata without external codebooks by:

Embedding language-specific labels as attributes in Stata-formatted files and to use standardized variable naming conventions that maintain consistency across survey waves since 2004. Therefore, Metadata is structured to facilitate longitudinal analysis. The project employs Stata-formatted datasets (.dta) with embedded metadata, preserving:

* Variable labels (i.e., question wording).

* Value labels (i.e., responses options).

* Missing value codes (i.e., Not Applicable, Don't Know, Refused).

As a result, a key advantage is LAPOP Lab's active metadata preservation even when distributing data through third-party platforms like ICPSR or NORC, the Stata-formatted metadata remains intact, allowing R/python users to still access labels via the `readstata13` package.

## How to load AmericasBarometer Data in R

The preferred method for loading LAPOP Lab datasets is to use the `read.dta13()` function from the `readstata13` package. Below, I show a key difference between this recommended approach and two alternative methods commonly used by researchers: `haven::read_dta()` and `rio::import()`.

In the R programming language, attributes are additional metadata that can be attached to R objects. Attributes provide a way to store extra information about an object beyond its basic data structure. To access the attributes of an object, you can use the `attr()` or `attributes()` base functions. 

```{r, LOAD DATA CHUNK, eval = FALSE}
# READSTATA13 PACKAGE (RECOMMENDED FOR LABELS)
data1 <- read.dta13("./BRA 2023 LAPOP AmericasBarometer (v1.0s).dta")
names(attributes(data1))

# HAVEN PACKAGE (RECOMMENDED FOR PLOTTING)
data2 <- read_dta("./BRA 2023 LAPOP AmericasBarometer (v1.0s).dta")
names(attributes(data2))

# RIO PACKAGE (NOT RECOMMENDED)
data3 <- import("./BRA 2023 LAPOP AmericasBarometer (v1.0s).dta")
names(attributes(data3))
```

The methods `haven` and `rio` packages  do not recover all the metadata from `dta` files. Thereby, we proceed with the dataset loaded with the `readstata13` package.

## AmericasBarometer Variables Labels (Question Wording)
AmericasBarometer variable labels are stored as `notes` in Stata .dta files. We 
can find them inside the "expansion.fields" attribute.

The `varlabels` object created is a nested list (much like a JSON object) that will contain multilingual notes with the question wording of each variable. For instance, for the variable `ing4` in the `Brazil AB 2023` dataset will contain notes with the question wording and lead-in in English, Spanish, and Portuguese.

```{r, QWORDING, eval = TRUE, warnings = F}
data(bra23)

# Extracting attributes from expasion.fields
names(attributes(bra23))
varlabels = attr(bra23, "expansion.fields")
```

## How do I find the correct language labels?

It is important to keep in mind that note numbers may change across countries datasets and years depending on the language the survey was fielded. For Brazil, it is Brazilian-Portuguese, whereas in most other countries it is in Spanish but also in English like in Jamaica.

A streamlined approach for users who want to quickly find the values of the notes on variable labels without assigning to the dataset would be assessing those notes directly with a combination of `lapply()` and `Filter()` functions from base R, or also `compact()` and `map()` functions from the `purrr` package.

Both approaches extract the third element (x[3]) from each list in the "expansion.fields" attribute of the dataset, but only if the first element (x[1]) equals "ing4" (i.e., the variable of interest). Both produce a semi-structured list of matching third nested elements.

```{r, STREAMLINED APPROACH, eval = TRUE, warnings = F}
# Base R approach:
# The base R version uses lapply to iterate over the list and returns NULL when 
# the condition isn't met, then Filter(Negate(is.null), ...) removes those NULLs.
head(
  Filter(Negate(is.null), lapply(attr(bra23, "expansion.fields"), 
  function(x) {
    if (x[1] == "ing4") list(note_id = x[2], note_value = x[3])})), n = 3)

# purrr package approach
#  The purrr version uses map() to do the same, and also remove NULL results. 
tail(
  purrr::map_dfr(attr(bra23, "expansion.fields"), ~ {
  if (.x[1] == "ing4") 
    data.frame(
      note_id = .x[2],
      note_value = .x[3],
      stringsAsFactors = FALSE
    )
})
)

```

To ease the researchers and analysts job, LAPOP Lab developed custom functions to deal with those labels in R. First, we will use `lpr_extract_notes` to transform the nested lists from `expansion.fields` attribute into a long dataframe format that will contain 3 columns: `variable_name`, `note_id`, and `note_value`

```{r, NOTES EXTRACT, eval = TRUE, warnings = F}
# Extract AB notes with lpr function
BRA23notes <- lpr_extract_notes(bra23)
head(BRA23notes, n = 10) # columns names
table(BRA23notes$note_id) # note information available!
```

One can assign variable labels to their preferred language as attributes to the dataset using another LAPOP Lab function called `lpr_set_attr()` and than access it during data analysis without the need of an accompanying codebook.

```{r, SET ATTRIBUTES, eval = TRUE, warnings = F, warning=F}
# English
bra23<-lpr_set_attr(bra23, BRA23notes, verbose = F,
                    noteid = "note2", 
                    attribute_name = "qwording_en") 
# Spanish
bra23<-lpr_set_attr(bra23, BRA23notes, verbose = F,
                    noteid = "note1", 
                    attribute_name = "qwording_es")

# Portuguese
bra23<-lpr_set_attr(bra23, BRA23notes, verbose = F,
                    noteid = "note3", 
                    attribute_name = "qwording_pt") 

# Printing languages
attr(bra23$ing4, "qwording_en") # English
attr(bra23$ing4, "qwording_es") # Spanish
attr(bra23$ing4, "qwording_pt") # Portuguese
```

## AmericasBarometer Values Labels (Response Options)

AmericasBarometer values labels (i.e., response options) differently from the variable labels are stored inside the "label.table" attribute. To extract them we will use another LAPOP Lab custom function called `lpr_format_labels` that will set up the response options
as an attribute across languages. Default language is English and default attribute name is `roslabel` but the function allows custom attribute names as shown below:

```{r, ROs, eval = TRUE, warnings = F}
bra23 <- lpr_set_ros(bra23) # Default English

bra23 <- lpr_set_ros(bra23, lang_id = "es", 
                     attribute_name = "respuestas") # Spanish

bra23 <- lpr_set_ros(bra23, lang_id = "pt", 
                     attribute_name = "ROsLabels_pt") # Portuguese

# Printing ROs
attr(bra23$ing4, "roslabel") # English
attr(bra23$ing4, "respuestas") # Spanish
attr(bra23$ing4, "ROsLabels_pt") # Portuguese
```

<br>

## Takeaways (TLDR)

<br>

### Data Structure
AmericasBarometer datasets are distributed in Stata .dta format with multilingual metadata (question wording and response options) embedded as attributes. These support cross-national and longitudinal comparability.

<br>

###  Preferred Loading Method
Use the readstata13::read.dta13() function to preserve full metadata structure. Other methods such as `haven::read_dta()` or `rio::import()` often fail to import these critical attributes.

<br>

### Variable Labels (Question Wording):
Stored in the "expansion.fields" attribute. Use the custom `lpr_extract_notes()` function to convert this into a tidy data frame.

Assign preferred language labels to each variable with `lpr_set_attr()` using the appropriate noteid.

<br>

### Value Labels (Response Options):
Stored in the "label.table" attribute. Use `lpr_set_ros()` to assign these response labels as attributes in English, Spanish, or Portuguese.

<br>

### Quick Access Options
For advanced users, the guide also demonstrates streamlined methods using `lapply()` and `purrr::map()` to quickly view variable labels without modifying the dataset.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lapop_fonts_design.R
\name{lapop_fonts_design}
\alias{lapop_fonts_design}
\title{LAPOP Fonts (design)}
\usage{
lapop_fonts_design()
}
\value{
No return value, called for side effects
}
\description{
This function loads fonts needed for LAPOP graph formatting.  In contrast to lapop_fonts(),
this renders text as text instead of polygons, which allows post-hoc editing.
}
\author{
Luke Plutowski, \email{luke.plutowski@vanderbilt.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lapop_fonts.R
\name{lapop_fonts}
\alias{lapop_fonts}
\title{LAPOP Fonts}
\usage{
lapop_fonts()
}
\value{
No return value, called for side effects
}
\description{
This function loads fonts needed for LAPOP graph formatting.
No arguments needed; just run lapop_fonts() at the beginning of your session.
}
\examples{
\donttest{require(lapop); lapop_fonts()}

}
\author{
Luke Plutowski, \email{luke.plutowski@vanderbilt.edu} & Robert Vidigal, \email{robert.vidigal@vanderbilt.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lpr_mover.R
\name{lpr_mover}
\alias{lpr_mover}
\title{LAPOP "Multiple-Over" Breakdown Graphs}
\usage{
lpr_mover(
  data,
  outcome,
  grouping_vars,
  rec = list(c(1, 1)),
  rec2 = c(1, 1),
  rec3 = c(1, 1),
  rec4 = c(1, 1),
  ci_level = 0.95,
  mean = FALSE,
  filesave = "",
  cfmt = "",
  ttest = FALSE,
  keep_nr = FALSE
)
}
\arguments{
\item{data}{A survey object. The data that should be analyzed.}

\item{outcome}{Character. Outcome variable(s) of interest to be plotted across secondary
variable(s).}

\item{grouping_vars}{A character vector specifying one or more grouping variables.
For each variable, the function calculates the average of the outcome variable,
broken down by the distinct values within the grouping variable(s).}

\item{rec}{Numeric. The minimum and maximum values of the frst outcome variable that
should be included in the numerator of the percentage.  For example, if the variable
is on a 1-7 scale and rec is c(5, 7), the function will show the percentage who chose
an answer of 5, 6, 7 out of all valid answers.  Can also supply one value only,
to produce the percentage that chose that value out of all other values.
Default: c(1, 1).}

\item{rec2}{Numeric. Similar to 'rec' for the second outcome. Default: c(1, 1).}

\item{rec3}{Numeric.  Similar to 'rec' for the third outcome. Default: c(1, 1).}

\item{rec4}{Numeric.  Similar to 'rec' for the fourth outcome. Default: c(1, 1).}

\item{ci_level}{Numeric. Confidence interval level for estimates.  Default: 0.95}

\item{mean}{Logical.  If TRUE, will produce the mean of the variable rather than
recoding to percentage.  Default: FALSE.}

\item{filesave}{Character.  Path and file name to save the dataframe as csv.}

\item{cfmt}{Changes the format of the numbers displayed above the bars.
Uses sprintf string formatting syntax. Default is whole numbers for percentages
and tenths place for means.}

\item{ttest}{Logical.  If TRUE, will conduct pairwise t-tests for difference
of means between all individual year-xvar levels and save them in attr(x,
"t_test_results"). Default: FALSE.}

\item{keep_nr}{Logical.  If TRUE, will convert "don't know" (missing code .a)
and "no response" (missing code .b) into valid data (value = 99) and use them
in the denominator when calculating percentages.  The default is to examine
valid responses only.  Default: FALSE.}
}
\value{
Returns a data frame, with data formatted for visualization by lapop_mover
}
\description{
This function creates a dataframe which can then be input in lapop_mover() for
comparing means across values of secondary variable(s) using LAPOP formatting.
}
\examples{
\donttest{
require(lapop); data(ym23)

# Set SUrvey Context
ym23lpr<-lpr_data(ym23)

# Single DV
lpr_mover(data = ym23lpr,
 outcome = "ing4",
 grouping_vars = c("q1tc_r", "edre"),
 rec = c(5, 7), ttest = FALSE)

# Multiple DV
lpr_mover(data = ym23lpr,
outcome = c("ing4", "pn4"),
grouping_vars = c("q1tc_r", "edre"),
rec = c(5, 7), rec2 = c(1, 2),
ttest = FALSE)

# Single DV X Single IV
lpr_mover(data = ym23lpr,
outcome="ing4",
grouping_vars="pn4",
rec=c(5,7),
ttest = FALSE)

# Multiple DV X Single IV
lpr_mover(data = ym23lpr,
outcome=c("ing4", "pn4"),
grouping_vars="edre",
rec=c(5,7), rec2=c(1,2),
ttest = FALSE)

# Multiple DV X Multiple IV
lpr_mover(data = ym23lpr,
outcome=c("ing4", "pn4"),
grouping_vars=c("edre", "q1tc_r"),
rec=c(5,7), rec2=c(1,2),
ttest = FALSE)
}

}
\author{
Luke Plutowski, \email{luke.plutowski@vanderbilt.edu} && Robert Vidigal, \email{robert.vidigal@vanderbilt.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lapop_ts.R
\name{lapop_ts}
\alias{lapop_ts}
\title{LAPOP Time-Series Graphs}
\usage{
lapop_ts(
  data,
  outcome_var = data$prop,
  lower_bound = data$lb,
  upper_bound = data$ub,
  wave_var = as.character(data$wave),
  label_var = data$proplabel,
  point_var = data$prop,
  ymin = 0,
  ymax = 100,
  main_title = "",
  source_info = "LAPOP",
  subtitle = "",
  lang = "en",
  color_scheme = "#A43D6A",
  percentages = TRUE,
  label_vjust = -2.1,
  max_years = 15,
  label_angle = 0,
  ci_type = "linerange"
)
}
\arguments{
\item{data}{Data Frame. Dataset to be used for analysis.  The data frame should have columns
titled wave (survey wave/year; character vector), prop (outcome variable; numeric),
proplabel (text of outcome variable; character); lb (lower bound of estimate; numeric),
and ub (upper bound of estimate; numeric). Default: None (must be supplied).}

\item{wave_var, outcome_var, label_var, lower_bound, upper_bound, point_var}{Character, numeric, character,
numeric, numeric, character. Each component of the data to be plotted can be manually specified in case
the default columns in the data frame should not be used (if, for example, the values for a given
variable were altered and stored in a new column).}

\item{ymin, ymax}{Numeric.  Minimum and maximum values for y-axis. Default: 0, 100.}

\item{main_title}{Character.  Title of graph.  Default: None.}

\item{source_info}{Character.  Information on dataset used (country, years, version, etc.),
which is added to the bottom-left corner of the graph. Default: LAPOP ("Source: LAPOP Lab" will be printed).}

\item{subtitle}{Character.  Describes the values/data shown in the graph, e.g., "Percent of Mexicans who agree...".
Default: None.}

\item{lang}{Character.  Changes default subtitle text and source info to either Spanish or English.
Will not translate input text, such as main title or variable labels.  #' Takes either "en" (English)
or "es" (Spanish).  Default: "en".}

\item{color_scheme}{Character.  Color of lines and dots.  Takes hex number, beginning with "#".
Default: "#A43D6A" (red).}

\item{percentages}{Logical.  Is the outcome variable a percentage?  Set to FALSE if you are using
means of the raw values, so that the y-axis adjusts accordingly. Default: TRUE.}

\item{label_vjust}{Numeric. Customize vertical space between points and their labels.
Default: -2.1.}

\item{max_years}{Numeric. Threshold for automatic x-axis label rotation. When the number of unique
country labels exceeds this value, labels will be smaller and if necessary rotated for better readability.
Default: 15 years.}

\item{label_angle}{Numeric. Angle (in degrees) to rotate x-axis labels when max_years is exceeded. Default: 0.}

\item{ci_type}{Character. Controls how confidence intervals are displayed in the
time-series plot. This parameter only affects how the confidence interval is visualized;
the point estimate and line plot remain unchanged. Options:
\itemize{
  \item \code{"linerange"} (default): Draws upper and lower bounds as dashed lines.
  \item \code{"errorbar"}: Displays confidence intervals using vertical error bars
        centered on the point estimate.
  \item \code{"ribbon"}: Shows a shaded confidence band between the lower and
        upper bounds.
  \item \code{"none"}: Suppresses confidence interval display.
}}
}
\value{
Returns an object of class \code{ggplot}, a ggplot line graph showing
values of a variable over time.
}
\description{
This function creates time series graphs using LAPOP formatting.  If there are waves missing at the
beginning or end of the series, the function will omit those waves from the graph (i.e.,
the x-axis will range from the earliest wave for which data is supplied to the latest).  If there are
waves missing in the middle of the series, those waves will be displayed on the x-axis, but no data will be
shown.
}
\details{
The input data must have a specific format to produce a graph.  It must include columns for
the survey wave (wave), the outcome variable (prop), the lower bound of the estimate (lb),
the upper bound of the estimate (ub), and a string for the outcome variable label (proplabel).
}
\examples{
\donttest{
require(lapop); lapop_fonts()

df <- data.frame(wave = c("2008", "2010", "2016/17", "2018/19", "2021"),
prop = c(23.2, 14.4, 35.8, 36.6, 40),
proplabel = c("23.2\%", "14.4\%", "35.8\%", "36.6\%", "40.0\%"),
lb = c(20.2, 11.9, 33.3, 33.1, 38),
ub = c(26.2, 16.9, 38.3, 40.1, 42)
)

lapop_ts(df,
main_title = "Ecuadorians are becoming more interested in politics",
subtitle = "\% politically interested",
source_info = "Source: LAPOP Lab, AmericasBarometer Ecuador 2006-2021",
ymin = 0,
ymax = 55)
}
}
\author{
Luke Plutowski, \email{luke.plutowski@vanderbilt.edu} & Robert Vidigal, \email{robert.vidigal@vanderbilt.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lpr_extract_ros.R
\name{lpr_extract_ros}
\alias{lpr_extract_ros}
\title{Extract Response Option (RO) values and texts for all variables into a tidy table.}
\usage{
lpr_extract_ros(
  data,
  lang_id = "en",
  include_special = FALSE,
  restrict_to_present = TRUE,
  one_row_per_var = FALSE,
  pair_sep = " | ",
  attr_name = "label.table"
)
}
\arguments{
\item{data}{A data.frame read with readstata13/haven/etc.}

\item{lang_id}{Language code used in label table names ("en", "es", "pt").
If `NULL` or `""`, auto-detect per variable (dataset-level only). Ignored for per-variable `levels`.}

\item{include_special}{Logical; if FALSE, drop codes >= 1000 when codes are numeric. Default FALSE.}

\item{restrict_to_present}{Logical; if TRUE, keep only codes that appear in the data. Default TRUE.}

\item{one_row_per_var}{Logical; if TRUE, return one row per variable_name with concatenated ROs. Default FALSE.}

\item{pair_sep}{String used to separate each "(value) answer_text" pair when collapsing. Default " | ".}

\item{attr_name}{Name of the attribute that stores RO info. Default "label.table".}
}
\value{
If `one_row_per_var = FALSE`: tibble with columns `variable_name`, `value`, `answer_text`.
If `one_row_per_var = TRUE`: tibble with columns `variable_name`, `answer_text` (collapsed pairs).
}
\description{
Works with:
 (a) dataset-level dictionaries (e.g., attr(data, "label.table") is a list keyed by "<VAR>_<lang>"), or
 (b) per-variable attributes (e.g., attr(data[[VAR]], "levels") or factor levels).
}
\examples{
toy <- data.frame(
  ing4 = c(1L, 2L, 1L),
  b12 = c(1L, 2L, NA_integer_)
)
attr(toy, "label.table") <- list(
  ing4_pt = c("Apoia muito" = 1L, "Apoia" = 2L, "NS/NR" = 1000L),
  b12_pt = c("Muito" = 1L, "Algo" = 2L, "NS/NR" = 1000L)
)

lpr_extract_ros(toy, lang_id = "pt")
lpr_extract_ros(toy, lang_id = "pt", one_row_per_var = TRUE)

}
\author{
Robert Vidigal, \email{robert.vidigal@vanderbilt.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lapop_mline.R
\name{lapop_mline}
\alias{lapop_mline}
\title{LAPOP Multi-line Time-Series Graphs}
\usage{
lapop_mline(
  data,
  varlabel = data$varlabel,
  wave_var = as.character(data$wave),
  outcome_var = data$prop,
  label_var = data$proplabel,
  point_var = data$prop,
  ymin = 0,
  ymax = 100,
  main_title = "",
  source_info = "",
  subtitle = "",
  lang = "en",
  legend_h_just = 40,
  legend_v_just = -20,
  subtitle_h_just = 0,
  color_scheme = c("#784885", "#008381", "#c74e49", "#2d708e", "#a43d6a", "#202020"),
  percentages = TRUE,
  all_labels = FALSE,
  ci = FALSE,
  legendnrow = 1
)
}
\arguments{
\item{data}{Data Frame. Dataset to be used for analysis.  The data frame should have columns
titled varlabel (values of secondary variable which will be used to make each line; character),
wave (survey wave/year; character), prop (outcome variable; numeric),
proplabel (text of outcome variable; character). Default: None (must be supplied).}

\item{varlabel, wave_var, outcome_var, label_var, point_var}{Character,
character, numeric, character, numeric. Each component of the data to be plotted
can be manually specified in case the default columns in the data frame should
not be used (if, for example, the values for a given variable were altered
and stored in a new column).}

\item{ymin, ymax}{Numeric.  Minimum and maximum values for y-axis. Default: 0, 100.}

\item{main_title}{Character.  Title of graph.  Default: None.}

\item{source_info}{Character.  Information on dataset used (country, years, version, etc.),
which is added to the end of "Source: " in the bottom-left corner of the graph.
Default: None (only "Source: " will be printed).}

\item{subtitle}{Character.  Describes the values/data shown in the graph, e.g., "Percent of Mexicans who agree...".
Default: None.}

\item{lang}{Character.  Changes default subtitle text and source info to either Spanish or English.
Will not translate input text, such as main title or variable labels.  #' Takes either "en" (English)
or "es" (Spanish).  Default: "en".}

\item{legend_h_just, legend_v_just}{Numeric.  Changes location of legend. From 0 to 100.
(secondary variable labels).  Defaults: 40, -20.}

\item{subtitle_h_just}{Numeric.  Moves subtitle left to right.  From 0 to 1.
(secondary variable labels).  Defaults: 0 (left justify).}

\item{color_scheme}{Character.  Color of lines and dots.  Takes hex number, beginning with "#".
Must specify four values, even if four are not used.
Default: c("#784885", "#008381", "#c74e49", "#2d708e", "#a43d6a", "#202020").}

\item{percentages}{Logical.  Is the outcome variable a percentage?  Set to FALSE if you are using
means of the raw values, so that the y-axis adjusts accordingly. Default: TRUE.}

\item{all_labels}{Logical.  If TRUE, show text above all points, instead of only those in the most recent wave. Default: FALSE.}

\item{ci}{Logical.  Add "tie fighter" confidence intervals.  Only recommended when each line represents a different variable.}

\item{legendnrow}{Numeric.  How many rows for legend labels. Default: 1.}
}
\value{
Returns an object of class \code{ggplot}, a ggplot line graph showing values of a variable over time.
}
\description{
This function creates a time series graph utilizing multiple lines representing values of
an outcome variable for different values of a secondary variable -- for example, support for
democracy over time by country.  This function is designed to be used for
AmericasBarometer data.  The maximum number of lines is four.  Unlike the lapop_ts()
single-line time series graph, this function will not print confidence lines nor will
it show text values for each year (just the final/most recent year).
}
\examples{
\donttest{
df <- data.frame(varlabel = c(rep("Honduras", 9), rep("El Salvador", 9),
                              rep("Mexico", 9), rep("Guatemala", 9)),
                 wave = rep(c("2004", "2006", "2008", "2010", "2012",
                              "2014", "2016/17", "2018/19", "2021"), 4),
                 prop = c(19, 24, 21, 15, 11, 32, 41, 38, 54,
                          29, 29, 25, 24, 24, 28, 36, 26, 32,
                          14, 16, 14, 16, 9, 14, 18, 19, 26,
                          21, 15, 18, 20, 14, 18, 17, 25, 36),
                 proplabel = c("19\%", "24\%", "21\%", "15\%", "11\%", "32\%",
                               "41\%", "38\%", "54\%",
                               "29\%", "29\%", "25\%", "24\%", "24\%", "28\%",
                               "36\%", "26\%", "32\%",
                               "14\%", "16\%", "14\%", "16\%", "9\%", "14\%",
                               "18\%", "19\%", "26\%",
                               "21\%", "15\%", "18\%", "20\%", "14\%", "18\%",
                               "17\%", "25\%", "36\%"))
require(lapop); lapop_fonts()
lapop_mline(df,
      main_title = "Intentions to emigrate in Guatemala, Honduras and Mexico reached their highs",
      subtitle = "\% who intend to migrate in:",
      source_info = ", AmericasBarometer 2004-2021")
}
}
\author{
Luke Plutowski, \email{luke.plutowski@vanderbilt.edu} & Robert Vidigal, \email{robert.vidigal@vanderbilt.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lapop-deprecated.R, R/lapop_dumb.R,
%   R/lapop_mline.R, R/lapop_mover.R, R/lapop_stack.R
\name{lapop-deprecated}
\alias{lapop-deprecated}
\alias{lapop_db}
\alias{lapop_tsmulti}
\alias{lapop_demog}
\alias{lapop_sb}
\title{Deprecated functions in package \pkg{lapop}.}
\usage{
lapop_db(
  data,
  ymin = 0,
  ymax = 100,
  lang = "en",
  main_title = "",
  source_info = "",
  subtitle = "",
  sort = "wave2",
  order = "hi-lo",
  color_scheme = c("#482677", "#3CBC70"),
  subtitle_h_just = 40,
  subtitle_v_just = -18
)

lapop_tsmulti(
  data,
  varlabel = data$varlabel,
  wave_var = as.character(data$wave),
  outcome_var = data$prop,
  label_var = data$proplabel,
  point_var = data$prop,
  ymin = 0,
  ymax = 100,
  main_title = "",
  source_info = "",
  subtitle = "",
  lang = "en",
  legend_h_just = 40,
  legend_v_just = -20,
  subtitle_h_just = 0,
  color_scheme = c("#7030A0", "#3CBC70", "#1F968B", "#95D840", "")
)

lapop_demog(
  data,
  lang = "en",
  main_title = "",
  subtitle = "",
  source_info = "",
  rev_values = FALSE,
  rev_variables = FALSE,
  subtitle_h_just = 0,
  ymin = 0,
  ymax = 100,
  x_lab_angle = 90,
  color_scheme = c("#7030A0", "#00ADA9", "#3CBC70", "#7EA03E", "#568424", "#ACB014")
)

lapop_sb(
  data,
  outcome_var = data$prop,
  prop_labels = data$proplabel,
  var_labels = data$varlabel,
  value_labels = data$vallabel,
  lang = "en",
  main_title = "",
  subtitle = "",
  source_info = "",
  rev_values = FALSE,
  rev_variables = FALSE,
  hide_small_values = TRUE,
  order_bars = FALSE,
  subtitle_h_just = 0,
  color_scheme = c("#2D708E", "#1F9689", "#00ADA9", "#21A356", "#568424", "#ACB014")
)
}
\value{
No return value, called for side effects
}
\description{
The functions listed below are deprecated and will be defunct in
  the near future. When possible, alternative functions with similar
  functionality are also mentioned. Help pages for deprecated functions are
  available at \code{help("<function>-deprecated")}.
}
\section{\code{lapop_db}}{

For \code{lapop_db}, use \code{\link{lapop_dumb}}.
}

\section{\code{lapop_tsmulti}}{

For \code{lapop
_tsmulti}, use \code{\link{lapop_mline}}.
}

\section{\code{lapop_demog}}{

For \code{lapop_demog}, use \code{\link{lapop_mover}}.
}

\section{\code{lapop_sb}}{

For \code{lapop_sb}, use \code{\link{lapop_stack}}.
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lpr_ccm.R
\name{lpr_ccm}
\alias{lpr_ccm}
\title{LAPOP Grouped Bar Graph Pre-Processing}
\usage{
lpr_ccm(
  data,
  outcome_vars,
  xvar = "pais_lab",
  rec1 = c(1, 1),
  rec2 = c(1, 1),
  rec3 = c(1, 1),
  ci_level = 0.95,
  mean = FALSE,
  filesave = "",
  cfmt = "",
  sort = "y",
  order = "hi-lo",
  ttest = FALSE,
  keep_nr = FALSE
)
}
\arguments{
\item{data}{A survey object.  The data that should be analyzed.}

\item{outcome_vars}{Character vector.  Outcome variable(s) of interest to be plotted
across country (or other x variable). Max of 3 (three) variables.}

\item{xvar}{Character string. Outcome variables are broken down by this variable. You can set
xvar to "wave" or "year" for cross-time comparisons. Default: pais_lab.}

\item{rec1, rec2, rec3}{Numeric. The minimum and maximum values of the outcome variable that
should be included in the numerator of the percentage.  For example, if the variable
is on a 1-7 scale and rec1 is c(5, 7), the function will show the percentage who chose
an answer of 5, 6, 7 out of all valid answers.  Can also supply one value only,
to produce the percentage that chose that value out of all other values.
Default: c(1, 1).}

\item{ci_level}{Numeric. Confidence interval level for estimates.  Default: 0.95}

\item{mean}{Logical.  If TRUE, will produce the mean of the variable rather than
rescaling to percentage.  Default: FALSE.}

\item{filesave}{Character.  Path and file name to save the dataframe as csv.}

\item{cfmt}{Character. Changes the format of the numbers displayed above the bars.
Uses sprintf string formatting syntax. Default is whole numbers for percentages
and tenths place for means.}

\item{sort}{Character. On what value the bars are sorted.
Options are "y" (default; for the value of the first outcome variable), "xv" (for
the underlying values of the x variable), "xl" (for the labels of the x variable,
i.e., alphabetical).}

\item{order}{Character.  How the bars should be sorted.  Options are "hi-lo"
(default) or "lo-hi".}

\item{ttest}{Logical.  If TRUE, will conduct pairwise t-tests for difference
of means between all outcomes vs. all x-vars and save them in attr(x,
"t_test_results"). Default: FALSE.}

\item{keep_nr}{Logical.  If TRUE, will convert "don't know" (missing code .a)
and "no response" (missing code .b) into valid data (value = 99) and use them
in the denominator when calculating percentages.  The default is to examine
valid responses only.  Default: FALSE.}
}
\value{
Returns a data frame, with data formatted for visualization by lapop_ccm()
}
\description{
This function creates dataframes which can then be input in lapop_ccm for
comparing values for multiple variables across countries with a bar graph
using LAPOP formatting.
}
\examples{

require(lapop); data(ym23)

# Set Survey Context on a small cross-country subset
ym23_small <- subset(ym23, pais \%in\% c(1, 15, 17))
ym23lpr <- lpr_data(ym23_small)

# Multiple outcomes over countries
lpr_ccm(ym23lpr,
outcome_vars = c("b12", "b18"),
rec1 = c(1, 3),
rec2 = c(5, 7))

# Multiple outcomes over years
\donttest{
lpr_ccm(ym23lpr,
outcome_vars = c("b12", "b18"),
xvar = "wave",
rec1 = c(1, 3),
rec2 = c(5, 7),
ttest = TRUE)
}

}
\author{
Luke Plutowski, \email{luke.plutowski@vanderbilt.edu} & Robert Vidigal, \email{robert.vidigal@vanderbilt.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lpr_hist.R
\name{lpr_hist}
\alias{lpr_hist}
\title{LAPOP Bar/Histogram Graphs}
\usage{
lpr_hist(
  data,
  outcome,
  filesave = "",
  cfmt = "",
  sort = "xv",
  order = "lo-hi",
  keep_nr = FALSE
)
}
\arguments{
\item{data}{A survey object.  The data that should be analyzed.}

\item{outcome}{Character. Outcome variable of interest.}

\item{filesave}{Character.  Path and file name to save the dataframe as csv.}

\item{cfmt}{Character. Changes the format of the numbers displayed above the bars.
Uses sprintf string formatting syntax. Default is whole numbers.}

\item{sort}{Character. On what value the bars are sorted.
Options are "y" (for the value of the outcome variable), "xv" (default; for
the underlying values of the x variable), "xl" (for the labels of the x variable,
i.e., alphabetical).}

\item{order}{Character.  How the bars should be sorted.  Options are "hi-lo"
or "lo-hi" (default).}

\item{keep_nr}{Logical.  If TRUE, will convert "don't know" (missing code .a)
and "no response" (missing code .b) into valid data (value = 99).
The default is to examine valid responses only.  Default: FALSE.}
}
\value{
Returns a data frame, with data formatted for visualization by lapop_hist
}
\description{
This function creates dataframes which can then be input in lapop_hist for
showing a bar graph using LAPOP formatting.
}
\examples{

require(lapop); data(bra23)

# Set Survey Context: single country and year (requires wt = T)
bra23lpr <- lpr_data(bra23, wt = TRUE)

lpr_hist(bra23lpr,
outcome = "ing4",
sort = "xv",
order = "hi-lo")

}
\author{
Shashwat Dhar \email{shashwat.dhar@vanderbilt.edu} & Luke Plutowski, \email{luke.plutowski@vanderbilt.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lapop_stack.R
\name{lapop_stack}
\alias{lapop_stack}
\title{LAPOP Stacked Bar Graphs}
\usage{
lapop_stack(
  data,
  outcome_var = data$prop,
  prop_labels = data$proplabel,
  var_labels = data$varlabel,
  value_labels = data$vallabel,
  xvar = NULL,
  lang = "en",
  main_title = "",
  subtitle = "",
  source_info = "LAPOP",
  rev_values = FALSE,
  rev_variables = FALSE,
  hide_small_values = TRUE,
  order_bars = FALSE,
  subtitle_h_just = 0,
  fixed_aspect_ratio = TRUE,
  legendnrow = 1,
  color_scheme = c("#2D708E", "#008381", "#C74E49", "#784885", "#a43d6a", "#202020")
)
}
\arguments{
\item{data}{Data Frame. Dataset to be used for analysis.  The data frame should have columns
titled varlabel (name(s)/label(s) of variable(s) of interest; character), vallabel (names/labels of values for each variable; character),
prop (outcome variable value; numeric), and proplabel (text of outcome variable value; character).
Default: None (must be provided).}

\item{outcome_var, prop_labels, var_labels, value_labels}{Numeric, character, character, character.
Each component of the data to be plotted can be manually specified in case
the default columns in the data frame should not be used (if, for example, the values for a given
variable were altered and stored in a new column).}

\item{xvar}{Character. Column name to group the plots by. This should match a column name in the dataset.
Default: NULL (no grouping).}

\item{lang}{Character.  Changes default subtitle text and source info to either Spanish or English.
Will not translate input text, such as main title or variable labels.  Takes either "en" (English)
or "es" (Spanish).  Default: "en".}

\item{main_title}{Character.  Title of graph.  Default: None.}

\item{subtitle}{Character.  Describes the values/data shown in the graph, e.g., "Percent who support...".
Default: None.}

\item{source_info}{Character.  Information on dataset used (country, years, version, etc.),
which is added to the end of "Source: " in the bottom-left corner of the graph.
Default: LAPOP ("Source: LAPOP Lab" will be printed).}

\item{rev_values}{Logical.  Should the order of the values for each variable be reversed?  Default: FALSE.}

\item{rev_variables}{Logical.  Should the order of the variables be reversed?  Default: FALSE.}

\item{hide_small_values}{Logical.  Should labels for categories with less than 5 percent be hidden?  Default: TRUE.}

\item{order_bars}{Logical.  Should categories be placed in descending order for each bar?  Default: FALSE.
showing the distributions of multiple categorical variables.}

\item{subtitle_h_just}{Numeric.  Move the subtitle/legend text left (negative numbers) or right (positive numbers).
Ranges from -100 to 100.  Default: 0.}

\item{fixed_aspect_ratio}{Logical.  Should the aspect ratio be set to a specific value (0.35)?
This prevents bars from stretching vertically to fit the plot area.  Set to false when you have
a large number of bars (> 10).  Default: TRUE.}

\item{legendnrow}{Numeric.  How many rows for legend labels. Default: 1.}

\item{color_scheme}{Character.  Color of data bars for each value.  Allows up to 6 values.
Takes hex numbers, beginning with "#".
Default: c("#2D708E", "#008381", "#C74E49", "#784885", "#a43d6a","#202020")
(navy blue, turquoise, teal, green, sap green, pea soup).}
}
\value{
Returns an object of class \code{ggplot}, a ggplot stacked bar graph
}
\description{
This function shows a stacked bar graph using LAPOP formatting.
}
\examples{
\donttest{
df <- data.frame(varlabel = c(rep("Politicians can\nidentify voters", 5),
                             rep("Wealthy can\nbuy results", 5),
                             rep("Votes are\ncounted correctly", 5)),
                vallabel = rep(c("Always", "Often", "Sometimes",
                                 "Never", "Other"), 3),
                prop = c(36, 10, 19, 25, 10, 46, 10, 23, 11, 10, 35,
                         10, 32, 13, 10),
                proplabel = c("36\%", "10\%", "19\%", "25\%", "10\%", "46\%",
                              "10\%", "23\%", "11\%", "10\%", "35\%", "10\%",
                              "32\%", "13\%", "10\%"))
require(lapop); lapop_fonts()
lapop_stack(df,
        main_title = "Trust in key features of the electoral process is low in Latin America",
        subtitle = "\% believing it happens:",
        source_info = "Source: LAPOP Lab, AmericasBarometer 2019")
}
}
\author{
Luke Plutowski, \email{luke.plutowski@vanderbilt.edu} & Robert Vidigal, \email{robert.vidigal@vanderbilt.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lapop_cccomb.R
\name{lapop_cccomb}
\alias{lapop_cccomb}
\title{LAPOP Bar Graphs}
\usage{
lapop_cccomb(
  cc1,
  cc2,
  subtitle1 = "",
  subtitle2 = "",
  main_title = "",
  source_info = "",
  lang = "en",
  color_scheme = "#784885",
  file_name = "",
  width_px = 895,
  height_px = 600
)
}
\arguments{
\item{cc1, cc2}{lapop_cc (ggplot) object.  Graphic for left and right panes, respectively.}

\item{subtitle1, subtitle2}{Character.  Describes the values/data shown in the graph, e.g., "Percent who agree that...".
Default: None.}

\item{main_title}{Character.  Title of graph.  Default: None.}

\item{source_info}{Character.  Information on dataset used (country, years, version, etc.),
which is added to the end of "Source: LAPOP Lab" in the bottom-left corner of the graph.
Default: None (only "Source: LAPOP Lab" will be printed).}

\item{lang}{Character.  Changes default subtitle text and source info to either Spanish or English.
Will not translate input text, such as main title or variable labels.  Takes either "en" (English)
or "es" (Spanish).  Default: "en".}

\item{color_scheme}{Character.  Color of bars.
Takes hex numbers, beginning with "#". Default: "#008381".}

\item{file_name}{Character.  If desired, supply file path + name to save graph.}

\item{width_px, height_px}{Numeric.  Width and height of saved graph in pixels. Default: 895, 600.}
}
\value{
Returns an object of class \code{ggplot}, a ggplot bar graph.
}
\description{
This function shows a bar graph for categorical variables using LAPOP formatting.
}
\examples{
\donttest{
require(lapop); lapop_fonts()
df1 <- data.frame(vallabel = c("Crime victim", "Non-victim"),
prop = c(36.1, 19.3),
proplabel = c("36\%" ,"19\%"),
lb = c(34.9, 18.1),
ub = c(37.3, 20.5))

df2 <- data.frame(vallabel = c("Crime victim", "Non-victim"),
prop = c(45, 15),
proplabel = c("45\%" ,"15\%"),
lb = c(43, 13),
ub = c(47, 16))

ccx <- lapop_cc(df1)
ccy <- lapop_cc(df2)

lapop_cccomb(ccx, ccy,
subtitle1 = "\% who support democracy",
subtitle2 = "\% who are satisfied with democracy",
main_title = "Crime victims are more supportive of and satisfied with democracy",
source_info = ", AmericasBarometer 2023")
}

}
\author{
Luke Plutowski, \email{luke.plutowski@vanderbilt.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lpr_set_attr.R
\name{lpr_set_attr}
\alias{lpr_set_attr}
\title{Set Variable Attributes from AmericasBarometer Notes (with propagation)}
\usage{
lpr_set_attr(
  data,
  notes,
  noteid = character(),
  attribute_name = character(),
  verbose = FALSE,
  propagate = TRUE,
  overwrite = TRUE
)
}
\arguments{
\item{data}{data.frame with variables to annotate}

\item{notes}{data.frame with columns variable_name, note_id, note_value}

\item{noteid}{character scalar; which note_id to use (e.g., "qtext_en")}

\item{attribute_name}{character scalar; attribute name to set (e.g., "qwording_en")}

\item{verbose}{logical; if TRUE it prints all variables notes available but not found in data}

\item{propagate}{logical; if TRUE, also set on <varname>_* children.
Useful for nominal variables or multiple response options variables. Default TRUE.}

\item{overwrite}{logical; if FALSE, do not overwrite existing attribute on a variable. Default TRUE.}
}
\value{
The input data frame with attributes applied (i.e., question wording)
}
\description{
Applies notes stored in a data frame as attributes to variables in `data`.
If a variable has expanded children (e.g., vb20_1, vb20_2), the attribute
is propagated to all of them by default.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lapop_mover.R
\name{lapop_mover}
\alias{lapop_mover}
\title{LAPOP Multiple-Over/Breakdown Graphs}
\usage{
lapop_mover(
  data,
  lang = "en",
  main_title = "",
  subtitle = "",
  qword = NULL,
  source_info = "LAPOP",
  rev_values = FALSE,
  rev_variables = FALSE,
  subtitle_h_just = 0,
  ymin = 0,
  ymax = 100,
  x_lab_angle = 90,
  color_scheme = c("#784885", "#008381", "#c74e49", "#2d708e", "#a43d6a")
)
}
\arguments{
\item{data}{Data Frame. Dataset to be used for analysis.  The data frame should have columns
titled varlabel (name(s)/label(s) of secondary variable(s); character), vallabel (names/labels of values for secondary variable; character),
prop (outcome variable value; numeric), proplabel (text of outcome variable value; character),
lb (lower bound of estimate; numeric), and ub (upper bound of estimate; numeric).
Default: None (must be provided).}

\item{lang}{Character.  Changes default subtitle text and source info to either Spanish or English.
Will not translate input text, such as main title or variable labels.  Takes either "en" (English)
or "es" (Spanish).  Default: "en".}

\item{main_title}{Character.  Title of graph.  Default: None.}

\item{subtitle}{Character.  Describes the values/data shown in the graph, e.g., "Percent who agree that...".
Default: None.}

\item{qword}{Character.  Describes the question wording shown in the graph, e.g., "Do you agree that...".
Default: NULL.}

\item{source_info}{Character.  Information on dataset used (country, years, version, etc.),
which is added to the bottom-left corner of the graph. Default: LAPOP ("Source: LAPOP Lab" will be printed).}

\item{rev_values}{Logical.  Should the order of the values for each variable be reversed?  Default: FALSE.}

\item{rev_variables}{Logical.  Should the order of the variables be reversed?  Default: FALSE.}

\item{subtitle_h_just}{Numeric.  Move the subtitle/legend text left (negative numbers) or right (positive numbers).
Ranges from -100 to 100.  Default: 0.}

\item{ymin, ymax}{Numeric.  Minimum and maximum values for y-axis. Defaults: 0 and 100.}

\item{x_lab_angle}{Numeric.  Angle/orientation of the value labels.  Default: 90.}

\item{color_scheme}{Character.  Color of data points and text for each secondary variable.  Allows up to 6 values.
Takes hex numbers, beginning with "#".
Default: c("#784885", "#008381", "#c74e49", "#2d708e", "#a43d6a")
(purple, teal, green, olive, sap green, pea soup).}
}
\value{
Returns an object of class \code{ggplot}, a ggplot figure showing
average values of some variable broken down by one or more secondary variables
(commonly, demographic variables).
}
\description{
This function shows the values of an outcome variable for subgroups of a secondary variable, using LAPOP formatting.
}
\examples{
\donttest{
df <- data.frame(varlabel = c(rep("Gender", 2), rep("Age", 6),
                              rep("Education", 4), rep("Wealth", 5)),
                 vallabel = c("Women", "Men", "18-25", "26-35", "36-45",
                              "46-55", "56-65", "66+", "  None", "Primary",
                              "Secondary", "Post-Sec.", "Low", "2",
                              "3", "4", "High"),
                 prop = c(20, 22, 21, 24, 22, 21, 17, 15, 20, 18, 21, 25, 21,
                          21, 21, 21, 22),
                 proplabel = c("20\%", "22\%", "21\%", "24\%", "22\%", "21\%",
                               "17\%", "15\%", "20\%", "18\%", "21\%", "25\%",
                               "21\%", "21\%", "21\%", "21\%", "22\%"),
                 lb = c(19, 21, 20, 23, 21, 20, 15, 13, 16, 17, 20, 24, 20,
                        20, 20, 20, 21),
                 ub = c(21, 23, 22, 25, 23, 22, 19, 17, 24, 19, 22, 26, 22,
                        22, 22, 22, 23))
require(lapop); lapop_fonts
lapop_mover(df,
            main_title = paste0("More educated, men, and younger individuals",
                                " in the LAC region are the\nmost likely",
                                  " to be crime victims"),
            subtitle = "\% victim of a crime", qword = "",
            source_info = "Source: LAPOP Lab, AmericasBarometer",
            ymin = 0,
            ymax = 40)
}
}
\author{
Luke Plutowski, \email{luke.plutowski@vanderbilt.edu} & Robert Vidigal, \email{robert.vidigal@vanderbilt.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lapop_ccm.R
\name{lapop_ccm}
\alias{lapop_ccm}
\title{LAPOP Cross-Country Bar Graphs}
\usage{
lapop_ccm(
  data,
  pais = data$pais,
  outcome_var = data$prop,
  lower_bound = data$lb,
  upper_bound = data$ub,
  label_var = data$proplabel,
  var = data$var,
  ymin = 0,
  ymax = 100,
  lang = "en",
  main_title = "",
  source_info = "",
  subtitle = "",
  sort = "",
  y_label = "",
  x_label = "",
  highlight = "",
  color_scheme = c("#784885", "#008381", "#C74E49"),
  label_size = 4,
  text_position = 0.7
)
}
\arguments{
\item{data}{Data Frame. Dataset to be used for analysis.  The data frame should have columns
titled pais (values of x-axis variable (usually pais); character vector), prop (outcome variable; numeric),
proplabel (text of outcome variable; character), lb (lower bound of estimate; numeric),
 ub (upper bound of estimate; numeric), and var (labels of secondary variables; character).
 Default: None (must be supplied).}

\item{pais, outcome_var, label_var, lower_bound, upper_bound, var}{Character, numeric, character,
numeric, numeric, character. Each component of the plot data can be manually specified in case
the default columns in the data frame should not be used (if, for example, the values for a given
variable were altered and stored in a new column).}

\item{ymin, ymax}{Numeric.  Minimum and maximum values for y-axis. Default: 0 to 100.}

\item{lang}{Character.  Changes default subtitle text and source info to either Spanish or English.
Will not translate input text, such as main title or variable labels.  Takes either "en" (English)
or "es" (Spanish).  Default: "en".}

\item{main_title}{Character.  Title of graph.  Default: None.}

\item{source_info}{Character.  Information on dataset used (country, years, version, etc.),
which is added to the end of "Source: " in the bottom-left corner of the graph.
Default: None (only "Source: " will be printed).}

\item{subtitle}{Character.  Describes the values/data shown in the graph, e.g., "percentage of Mexicans who say...)".
Default: None.}

\item{sort}{Character. Method of sorting bars.  Options: "var1" (highest to lowest on variable 1),
"var2" (highest to lowest on variable 2), "var3" (highest to lowest on variable 3),
"alpha" (alphabetical along x-axis/pais). Default: Order of data frame.}

\item{y_label}{Character.  Y-axis label.}

\item{x_label}{Character.  X-axis label.}

\item{highlight}{Character.  Country of interest.  Will highlight (make darker) that country's bar.
Input must match entry in "vallabel" exactly. Default: None.}

\item{color_scheme}{Character.  Color of bars.  Takes hex number, beginning with "#".
Default: "#784885", "#008381", "#C74E49".}

\item{label_size}{Numeric.  Size of text for data labels (percentages above bars).  Default: 4.}

\item{text_position}{Numeric.  Amount that text above error bars should be offset (to avoid overlap).  Default: 0.7}
}
\value{
Returns an object of class \code{ggplot}, a ggplot figure showing
average values of some variables across multiple countries.
}
\description{
This function creates bar graphs for comparing values across countries using LAPOP formatting.
}
\examples{
\donttest{
require(lapop); lapop_fonts()

df <- data.frame(pais = c(rep("HT", 2), rep("PE", 2), rep("HN", 2), rep("CO", 2),
             rep("UY", 2), rep("CR", 2), rep("EC", 2), rep("CL", 2),
              rep("BR", 2), rep("BO", 2), rep("JA", 2), rep("PN", 2)),
              var = rep(c("countfair1", "countfair3"), 3),
              prop = c(30, 38, 40, 49, 57, 33, 80, 54, 30, 43, 61, 42,
                       38, 54, 74, 61, 50, 34, 48, 34, 72, 41, 58, 57),
              proplabel = c("30\%", "38\%", "40\%", "49\%", "57\%", "33\%",
                            "80\%", "54\%", "30\%", "43\%", "61\%", "42\%",
                            "38\%", "54\%", "74\%", "61\%", "50\%", "34\%",
                            "48\%", "34\%", "72\%", "41\%", "58\%", "57\%"),
              lb = c(27, 35, 37, 46, 54, 30, 77, 51, 27, 40, 58, 39,
                     35, 51, 71, 58, 47, 31, 45, 31, 69, 38, 55, 54),
              ub = c(33, 41, 43, 52, 60, 36, 83, 57, 33, 46, 64, 45,
                     41, 57, 77, 64, 53, 37, 51, 37, 75, 44, 61, 60))

lapop_ccm(df, sort = "var", source_info = ", AmericasBarometer")
}
}
\author{
Luke Plutowski, \email{luke.plutowski@vanderbilt.edu} & Robert Vidigal, \email{robert.vidigal@vanderbilt.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lpr_data.R
\name{lpr_data}
\alias{lpr_data}
\title{LAPOP Data Processing}
\usage{
lpr_data(data_path, wt = FALSE)
}
\arguments{
\item{data_path}{The path for a AmericasBarometer data or a an existing dataframe.}

\item{wt}{Logical.  If TRUE, use `wt` (weights only for single-country single-year data)
instead of `weight1500` (the default weights for multiple-country and multiple-year data).
Default: FALSE.}
}
\value{
Returns a svy_tbl object
}
\description{
This function takes LAPOP datasets and adds survey features such as sampling
design effects, outputting a svy_tbl object that can then be analyzed using
lpr_ wrangling commands.
}
\examples{
\donttest{
data(bra23)
data(cm23)

bra23w <- lpr_data(bra23, wt = TRUE)
cm23w <- lpr_data(cm23)
}

}
\author{
Luke Plutowski, \email{luke.plutowski@vanderbilt.edu} & Robert Vidigal, \email{robert.vidigal@vanderbilt.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lapop_dumb.R
\name{lapop_dumb}
\alias{lapop_dumb}
\title{LAPOP Dummbell Graphs}
\usage{
lapop_dumb(
  data,
  ymin = 0,
  ymax = 100,
  lang = "en",
  main_title = "",
  source_info = "",
  subtitle = "",
  sort = "wave2",
  order = "hi-lo",
  color_scheme = c("#008381", "#A43D6A"),
  subtitle_h_just = 40,
  subtitle_v_just = -18,
  text_nudge = 6,
  drop_singles = FALSE
)
}
\arguments{
\item{data}{Data Frame. Dataset to be used for analysis.  The data frame should have columns
titled pais (country name; character), wave1 (name of first wave/year (all rows are the same); character),
prop1 (outcome variable values for the first wave; numeric), proplabel1 (text of outcome variable for first wave; character),
wave2 (name of second wave/year (all rows are the same); character),
prop2 (outcome variable values for the second wave; numeric), proplabel2 (text of outcome variable for second wave; character).
 Default: None (must be supplied).}

\item{ymin, ymax}{Numeric.  Minimum and maximum values for y-axis. Defaults: 0 and 100.}

\item{lang}{Character.  Changes default subtitle text and source info to either Spanish or English.
Will not translate input text, such as main title or variable labels.  Takes either "en" (English)
or "es" (Spanish).  Default: "en".}

\item{main_title}{Character.  Title of graph.  Default: None.}

\item{source_info}{Character.  Information on dataset used (country, years, version, etc.),
which is added to the end of "Source: " in the bottom-left corner of the graph.
Default: LAPOP (only "Source: "LAPOP Lab will be printed).}

\item{subtitle}{Character.  Describes the values/data shown in the graph, e.g., "Percent who agree that...".
Default: None.}

\item{sort}{Character.  The metric by which the data are sorted.  Options: "wave1" (outcome variable in first wave), "wave2" (outcome
variable in wave 2), "diff" (difference between the two waves), "alpha" (alphabetical by country name).
Default: "wave2".}

\item{order}{Whether data should be sorted from low to high or high to low on the sort metric.  Options: "hi-lo" and "lo-hi".
Default: "hi-lo".}

\item{color_scheme}{Character.  Color of data points.  Must supply two values.  Takes hex numbers, beginning with "#".
Default: "#482677", "#3CBC70".}

\item{subtitle_h_just, subtitle_v_just}{Numeric.  Move the subtitle/legend text left/down (negative numbers) or right/up (positive numbers).
Ranges from -100 to 100.  Defaults: 40, -18.}

\item{text_nudge}{Numeric.  Move text of data further or closer to data point.  Default: 6.}

\item{drop_singles}{Logical.  Should rows with only one dot be removed?  Default: FALSE.}
}
\value{
Returns an object of class \code{ggplot}, a ggplot figure showing
average values of some variable in two time periods across multiple countries
 (a dumbbell plot).
}
\description{
This function creates "dumbbell" graphs, which show averages for a variable across countries over two time periods, using LAPOP formatting.
}
\examples{
\donttest{
require(lapop); lapop_fonts()
df <- data.frame(pais = c("Haiti", "Peru", "Honduras", "Colombia", "Ecuador",
                          "Panama", "Bolivia", "Argentina", "Paraguay",
                          "Dom. Rep.", "Brazil", "Jamaica", "Nicaragua",
                          "Guyana", "Costa Rica", "Mexico", "Guatemala",
                          "Chile", "Uruguay", "El Salvador"),
                 wave1 = rep("2018/19", 20),
                 prop1 = c(NA, 30, 58, 40, 49, 57, 33, 68, 38, 46, 30,
                           31, 70, NA, 43, 25, 38, 31, 34, 41),
                 proplabel1 = c(NA, "30\%", "58\%", "40\%", "49\%", "57\%", "33\%",
                                "68\%", "38\%", "46\%", "30\%", "31\%", "70\%", NA,
                                "43\%", "25\%", "38\%", "31\%", "34\%", "41\%"),
                 wave2 = rep("2021", 20),
                 prop2 = c(86, 73, 69, 67, 67, 65, 65, 65, 63, 62, 62,
                           57, 56, 56, 55, 55, 54, 51, 46, 42),
                 proplabel2 = c("86\%", "73\%", "69\%", "67\%", "67\%", "65\%", "65\%",
                                "65\%", "63\%", "62\%", "62\%", "57\%", "56\%", "56\%",
                                "55\%", "55\%", "54\%", "51\%", "46\%", "42\%"))

lapop_dumb(df,
main_title = paste0("Personal economic conditions worsened across LAC"),
         subtitle = "\% personal economic situation worsened",
         source_info = "Source: LAPOP Lab, AmericasBarometer 2018/19-2021")
}

}
\author{
Luke Plutowski, \email{luke.plutowski@vanderbilt.edu} & Robert Vidigal, \email{robert.vidigal@vanderbilt.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lapop_coef.R
\name{lapop_coef}
\alias{lapop_coef}
\title{LAPOP Regression Graphs}
\usage{
lapop_coef(
  data,
  coef_var = data$coef,
  label_var = data$proplabel,
  varlabel_var = data$varlabel,
  lb = data$lb,
  ub = data$ub,
  pval_var = data$pvalue,
  lang = "en",
  main_title = "",
  subtitle = "",
  source_info = "",
  ymin = NULL,
  ymax = NULL,
  pred_prob = FALSE,
  color_scheme = "#784885",
  subtitle_h_just = 0
)
}
\arguments{
\item{data}{Data Frame. Dataset to be used for analysis.  The data frame should have columns
titled coef (regression coefficients/predicted probabilities; numeric), proplabel (text of outcome variable; character),
varlabel (names of variables to be plotted; character), lb (lower bound of coefficient estimate; numeric),
ub (upper bound of estimate; numeric), and pvalue (p value of coefficient estimate; numeric).
 Default: None (must be supplied).}

\item{coef_var, label_var, varlabel_var, lb, ub, pval_var}{Numeric, character, character, numeric,
numeric, numeric. Each component of the data to be plotted can be manually specified in case
the default columns in the data frame should not be used (if, for example, the values for a given
variable were altered and stored in a new column).}

\item{lang}{Character.  Changes default subtitle text and source info to either Spanish or English.
Will not translate input text, such as main title or variable labels.  Takes either "en" (English)
or "es" (Spanish).  Default: "en".}

\item{main_title}{Character.  Title of graph.  Default: None.}

\item{subtitle}{Character.  Describes the values/data shown in the graph, e.g., "Regression coefficients".
Default: None.}

\item{source_info}{Character.  Information on dataset used (country, years, version, etc.),
which is added to the end of "Source: " in the bottom-left corner of the graph.
Default: None (only "Source: " will be printed).}

\item{ymin, ymax}{Numeric.  Minimum and maximum values for y-axis. Default: dynamic.}

\item{pred_prob}{Logical.  Is the graph showing predicted probabilities (instead of regression coefficients)?
Will only change text in the legend, not the data.  Default: FALSE.}

\item{color_scheme}{Character.  Color of bars.  Takes hex number, beginning with "#".
Default: "#784885" (purple).}

\item{subtitle_h_just}{Numeric.  Move the subtitle/legend text left (negative numbers) or right (positive numbers).
Ranges from -100 to 100.  Default: 0.}
}
\value{
Returns an object of class \code{ggplot}, a ggplot figure showing
coefficients or predicted probabilities from a multivariate regression.
}
\description{
This function creates plots of regression coefficients and predicted probabilities using LAPOP formatting.
}
\examples{
\donttest{
require(lapop); lapop_fonts()
df <- data.frame(
  varlabel = c("Intimate\nPartner", "wealth", "Education", "Age", "Male"),
  coef = c(0.02, -0.07, -0.24, 0.01, 0.11),
  lb = c(-0.002, -0.110, -0.295, -0.060, 0.085),
  ub = c(0.049, -0.031, -0.187, 0.080, 0.135),
  pvalue = c(0.075, 0.000, 0.000, 0.784, 0.000),
  proplabel = c("0.02", "-0.07", "-0.24", "0.01", "0.11")
)

lapop_coef(df,
           main_title = "Demographic and Socioeconomic Predictors of Normalizing IPV",
           pred_prob = TRUE,
           source_info = ", AmericasBarometer 2021",
           ymin = -0.3,
           ymax = 0.2)
}
}
\author{
Luke Plutowski, \email{luke.plutowski@vanderbilt.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lpr_extract_notes.R
\name{lpr_extract_notes}
\alias{lpr_extract_notes}
\title{Extract Notes from AmericasBarometer Attributes}
\usage{
lpr_extract_notes(data)
}
\arguments{
\item{data}{A dataset (data frame) containing "expansion.fields" in its attributes.}
}
\value{
A data frame with three columns:
\describe{
  \item{variable_name}{Name of the variable the note belongs to}
  \item{note_id}{Identifier for the note}
  \item{note_value}{The actual note text}
}
}
\description{
Extracts notes stored in a dataset's attributes and organizes them into a tidy data frame.
This function is particularly useful for processing Stata datasets imported into R that
contain variable notes in their attributes.
}
\details{
This function processes the attributes of a dataset to extract notes that are typically
stored in a specific format. It skips any notes associated with "_dta" (dataset-level notes)
and only returns variable-specific notes. The function expects the notes to be organized
as a list where each element contains exactly three components: variable name, note ID,
and note value.
}
\examples{
\donttest{
require(lapop); data(bra23)

# Extract the notes
notesBRA23 <- lpr_extract_notes(bra23)
tail(notesBRA23[notesBRA23$variable_name=="ing4",]) # for ing4 variable
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lpr_dumb.R
\name{lpr_dumb}
\alias{lpr_dumb}
\title{LAPOP Dumbbell Graphs}
\usage{
lpr_dumb(
  data,
  outcome,
  xvar = "pais",
  over,
  rec = c(1, 1),
  ci_level = 0.95,
  mean = FALSE,
  filesave = "",
  cfmt = "",
  sort = "prop2",
  order = "hi-lo",
  ttest = FALSE,
  keep_nr = FALSE
)
}
\arguments{
\item{data}{A survey object.  The data that should be analyzed.}

\item{outcome}{Outcome variable(s) of interest to be plotted across countries
and waves, supplied as a character string or vector of strings.}

\item{xvar}{Character. The grouping variable to be plotted
along the x-axis (technically, the vertical axis for lapop_dumb). Usually
country (pais). Default: "pais".}

\item{over}{Numeric. A vector of values for "wave" that specify which two
waves should be included in the plot.}

\item{rec}{Numeric. The minimum and maximum values of the outcome variable that
should be included in the numerator of the percentage.  For example, if the variable
is on a 1-7 scale and rec is c(5, 7), the function will show the percentage who chose
an answer of 5, 6, 7 out of all valid answers.  Can also supply one value only,
to produce the percentage that chose that value out of all other values.
Default: c(1, 1).}

\item{ci_level}{Numeric. Confidence interval level for estimates.  Default: 0.95}

\item{mean}{Logical.  If TRUE, will produce the mean of the variable rather than
recoding to percentage.  Default: FALSE.}

\item{filesave}{Character.  Path and file name to save the dataframe as csv.}

\item{cfmt}{Character. Changes the format of the numbers displayed above the bars.
Uses sprintf string formatting syntax. Default is whole numbers for percentages
and tenths place for means.}

\item{sort}{Character. On what value the bars are sorted.
Options are "prop1" (for the value of the outcome variable in wave 1), "prop2"
(default; for the value of the outcome variable in wave 2), "xv" (for
the underlying values of the x variable), "xl" (for the labels of the x variable,
i.e., alphabetical), and "diff" (for the difference between the outcome between
the two waves).}

\item{order}{Character.  How the bars should be sorted.  Options are "hi-lo"
(default) or "lo-hi".}

\item{ttest}{Logical.  If TRUE, will conduct pairwise t-tests for difference
of means between all pais-wave combinations and save them in attr(x,
"t_test_results"). Default: FALSE.}

\item{keep_nr}{Logical.  If TRUE, will convert "don't know" (missing code .a)
and "no response" (missing code .b) into valid data (value = 99) and use them
in the denominator when calculating percentages.  The default is to examine
valid responses only.  Default: FALSE.}
}
\value{
Returns a data frame, with data formatted for visualization by lapop_dumb()
}
\description{
This function creates dataframes which can then be input in lapop_dumb for
comparing means of a variable across countries and two waves using LAPOP formatting.
}
\examples{

require(lapop); data(cm23)

# Set Survey Context
cm23lpr <- lpr_data(cm23)

# Single outcome over years
lpr_dumb(cm23lpr,
outcome = "ing4",
rec = c(5, 7),
over = c("2018/19", "2023"),
sort = "diff")

# Multiple outcomes over years
lpr_dumb(cm23lpr,
outcome=c("b13","b21", "b31"),
rec=c(5,7),
over=c("2018/19", "2023"))

}
\author{
Luke Plutowski, \email{luke.plutowski@vanderbilt.edu} & Robert Vidigal, \email{robert.vidigal@vanderbilt.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lapop_hist.R
\name{lapop_hist}
\alias{lapop_hist}
\title{LAPOP Bar Graphs}
\usage{
lapop_hist(
  data,
  outcome_var = data$prop,
  label_var = data$proplabel,
  cat_var = data$cat,
  ymin = 0,
  ymax = 100,
  lang = "en",
  main_title = "",
  subtitle = "",
  source_info = "LAPOP",
  order = FALSE,
  color_scheme = "#008381"
)
}
\arguments{
\item{data}{Data Frame. Dataset to be used for analysis.  The data frame should have columns
titled cat (labels of each category in variable; character),
prop (outcome variable value; numeric), and proplabel (text of outcome variable value; character).
Default: None (must be provided).}

\item{cat_var, outcome_var, label_var}{Character, numeric, character.
Each component of the data to be plotted can be manually specified in case
the default columns in the data frame should not be used (if, for example, the values for a given
variable were altered and stored in a new column).}

\item{ymin, ymax}{Numeric.  Minimum and maximum values for y-axis. Defaults: 0, 100.}

\item{lang}{Character.  Changes default subtitle text and source info to either Spanish or English.
Will not translate input text, such as main title or variable labels.  Takes either "en" (English)
or "es" (Spanish).  Default: "en".}

\item{main_title}{Character.  Title of graph.  Default: None.}

\item{subtitle}{Character.  Describes the values/data shown in the graph, e.g., "Percent who agree that...".
Default: None.}

\item{source_info}{Character.  Information on dataset used (country, years, version, etc.),
which is added to the bottom-left corner of the graph. Default: LAPOP ("Source: LAPOP Lab" will be printed).}

\item{order}{Logical.  Should bars be ordered from most frequent response to least?  Default: FALSE.}

\item{color_scheme}{Character.  Color of bars.
Takes hex numbers, beginning with "#". Default: "#008381".}
}
\value{
Returns an object of class \code{ggplot}, a ggplot bar graph.
}
\description{
This function shows a bar graph for categorical variables using LAPOP formatting.
}
\examples{
\donttest{
require(lapop); lapop_fonts()

df <- data.frame(
cat = c("Far Left", 1, 2, 3, 4, "Center", 6, 7, 8, 9, "Far Right"),
prop = c(4, 3, 5, 12, 17, 23, 15, 11, 5, 4, 1),
proplabel = c("4\%", "3\%", "5\%", "12\%", "17\%", "23\%", "15\%", "11\%", "5\%", "4\%", "1\%")
)
lapop_hist(df,
           main_title = "Centrists are a plurality among Peruvians",
           subtitle = "Distribution of ideological preferences",
           source_info = "Source: LAPOP Lab, AmericasBarometer Peru 2019",
           ymax = 27)
}
}
\author{
Luke Plutowski, \email{luke.plutowski@vanderbilt.edu} & Robert Vidigal, \email{robert.vidigal@vanderbilt.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lapop_save.R
\name{lapop_save}
\alias{lapop_save}
\title{LAPOP Save}
\usage{
lapop_save(
  figure,
  filename,
  format = "svg",
  logo = FALSE,
  width_px = 895,
  height_px = 500
)
}
\arguments{
\item{figure}{Ggplot object.}

\item{filename}{File path + name to be saved + .filetype.}

\item{format}{Character.  Options: "png", "svg". Default = "svg".}

\item{logo}{Logical.  Should logo be added to plot?  Default: FALSE.}

\item{width_px}{Numeric. Width in pixels.  Default: 750.}

\item{height_px}{Numeric.  Height in pixels.}
}
\value{
Saves a file (in either .svg or .png format) to provided directory.
}
\description{
This function creates exports graphs created using the LAPOP templates.
}
\examples{
\donttest{df <- data.frame(
cat = c("Far Left", 1, 2, 3, 4, "Center", 6, 7, 8, 9, "Far Right"),
prop = c(4, 3, 5, 12, 17, 23, 15, 11, 5, 4, 1),
proplabel = c("4\%", "3\%", "5\%", "12\%", "17\%", "23\%", "15\%", "11\%", "5\%", "4\%", "1\%")
)
myfigure <- lapop_hist(df,
          main_title = "Centrists are a plurality among Peruvians",
          subtitle = "Distribution of ideological preferences",
          source_info = "Peru, 2019",
          ymax = 27
)

f <- file.path(tempdir(), "fig1.svg")
lapop_save(myfigure, f, format = "svg", width_px = 800)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/world.R
\docType{data}
\name{world}
\alias{world}
\title{bra23: Single-country Single-year Dataset}
\format{
A data frame
\describe{
\item{pais}{Country name}
\item{pais_lab}{Country ISO2 code}
\item{geometry}{Polygon with geometry}
}
}
\source{
LAPOP AmericasBarometer (https://www.vanderbilt.edu/lapop/)
}
\usage{
world
}
\description{
A dataset containing the World Map geometry for lapop_map() function.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lpr_stack.R
\name{lpr_stack}
\alias{lpr_stack}
\title{LAPOP Stacked Bar Graph Pre-Processing}
\usage{
lpr_stack(
  data,
  outcome,
  xvar = NULL,
  sort = "y",
  order = "hi-lo",
  filesave = "",
  keep_nr = FALSE
)
}
\arguments{
\item{data}{The data that should be analyzed. It requires a survey object from lpr_data() function.}

\item{outcome}{Vector of variables be plotted.}

\item{xvar}{Character. Outcome variable will be broken down by this variable. Default is NULL}

\item{sort}{Character. On what value the bars are sorted: the x or the y.
Options are "y" (default; for the value of the outcome variable), "xv" (for
the underlying values of the x variable), "xl" (for the labels of the x variable,
i.e., alphabetical).}

\item{order}{Character. How the bars should be sorted. Options are "hi-lo"
(default) or "lo-hi".}

\item{filesave}{Character. Path and file name to save the dataframe as csv.}

\item{keep_nr}{Logical. If TRUE, will convert "don't know" (missing code .a)
and "no response" (missing code .b) into valid data (value = 99) and use them
in the denominator when calculating percentages.  The default is to examine
valid responses only.  Default: FALSE.}
}
\value{
Returns a data frame, with data formatted for visualization by lapop_stack
}
\description{
This function creates dataframes which can then be input in lapop_stack() for
plotting variables categories with a stacked bar graph using LAPOP formatting.
}
\examples{
\donttest{
require(lapop); data(ym23)

# Set Survey Context
ym23lpr<-lpr_data(ym23)

# Multiple outcomes stacked
lpr_stack(data = ym23lpr,
outcome = c("b12", "b18"))

# Single outcome over years
lpr_stack(data = ym23lpr,
outcome = "q14f",
xvar="year")
}

}
\author{
Robert Vidigal, \email{robert.vidigal@vanderbilt.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bra23.R
\docType{data}
\name{bra23}
\alias{bra23}
\title{bra23: Single-country Single-year Dataset}
\format{
A data frame
\describe{
\item{ing4}{Support for Democracy}
\item{b12}{Trust in Armed Forces}
\item{b13}{Trust in the National Legislature}
\item{b21}{Trust in Political Parties}
\item{b31}{Trust in the Supreme Court of Justice}
\item{fs2}{Food security}
\item{idio2}{Personal economic situation}
\item{wealth}{Wealth}
\item{wave}{Survey round year for regional or multi-country data}
\item{pais}{Country of survey}
\item{year}{Survey round year for single-country data}
\item{upm}{Primary Sampling Unit}
\item{strata}{Stratification}
\item{wt}{Country-specific post stratification weight}
}
}
\source{
LAPOP AmericasBarometer (https://www.vanderbilt.edu/lapop/)
}
\usage{
bra23
}
\description{
A dataset containing the AmericasBarometer Brazil 2023 survey round.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lpr_coef.R
\name{lpr_coef}
\alias{lpr_coef}
\title{LAPOP Regression Coefficients Graph Pre-Processing}
\usage{
lpr_coef(
  outcome = NULL,
  xvar = NULL,
  interact = NULL,
  model = "linear",
  data = NULL,
  estimate = c("coef"),
  vlabs = NULL,
  omit = NULL,
  filesave = NULL,
  replace = FALSE,
  level = 95
)
}
\arguments{
\item{outcome}{Dependent variable for the svyglm regression model. (e.g., "outcome_name"). Only one variable allowed.}

\item{xvar}{Vector of independent variables for the svyglm regression model (e.g., "xvar1+xvar2+xvar3" and so on). Multiple variables are allowed.}

\item{interact}{Interaction terms (e.g., "xvar1`*`xvar2 + xvar3`:`xvar4"). Supports `:` and `*` operators for interacting variables. Optional, default is NULL.}

\item{model}{Model family object for glm. Default is gaussian regression (i.e., "linear"). For a logit model, use model="binomial"}

\item{data}{Survey design data from lpr_data() output.}

\item{estimate}{Character. Graph either the coefficients (i.e., `coef`) or the change in probabilities (i.e., `contrast`). Default is "coef."}

\item{vlabs}{Character. Rename variable labels to be displayed in the graph produced by lapop_coef(). For instance, vlabs=c("old_varname" = "new_varname").}

\item{omit}{Character. Do not display coefficients for these independent variables. Default is to display all variables included in the model. To omit any variables you need to include the raw "varname" in the omit argument.}

\item{filesave}{Character. Path and file name with csv extension to save the dataframe output.}

\item{replace}{Logical. Replace the dataset output if it already exists. Default is FALSE.}

\item{level}{Numeric. Set confidence level in numeric values; default is 95 percent.}
}
\value{
Returns a data frame, with data formatted for visualization by lapop_coef
}
\description{
This function creates a data frame which can then be input in lapop_coef() for
plotting regression coefficients graph using LAPOP formatting.
}
\examples{

require(lapop); data(bra23)

# Set Survey Context
bra23lpr <- lpr_data(bra23, wt = TRUE)

# Example 1: Linear model
lpr_coef(data = bra23lpr,
 outcome = "ing4",
 xvar = "wealth+idio2",
 model = "linear",
 estimate = "coef")

# Example 2: Logit model with contrasts
lpr_coef(data = bra23lpr,
 outcome = "fs2",
 xvar = "wealth+idio2",
 model = "binomial",
 estimate = "contrast")

# Example 3: Interactive linear model
lpr_coef(data = bra23lpr,
 outcome = "ing4",
 xvar = "wealth+idio2",
 interact = "wealth*idio2",
 model = "linear",
 estimate = "coef")

# Example 4: Interactive logit model
lpr_coef(data = bra23lpr,
         outcome = "fs2",
         xvar = "wealth+idio2",
         interact = "wealth*idio2",
         model = "binomial",
         estimate = "contrast")

}
\author{
Robert Vidigal, \email{robert.vidigal@vanderbilt.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lpr_resc.R
\name{lpr_resc}
\alias{lpr_resc}
\title{LAPOP Rescale}
\usage{
lpr_resc(
  var,
  min = 0L,
  max = 1L,
  reverse = FALSE,
  only_reverse = FALSE,
  only_flip = FALSE,
  map = FALSE,
  new_varlabel = NULL,
  new_vallabels = NULL
)
}
\arguments{
\item{var}{Vector (class "labelled" or "haven_labelled").  The original variable
to rescale.}

\item{min}{Integer. Minimum value for the new rescaled variables; default is 0.}

\item{max}{Integer. Maximum value for the new rescaled variables; default is 1.}

\item{reverse}{Logical.  Reverse code the variable before rescaling. Default: FALSE.}

\item{only_reverse}{Logical.  Reverse code the variable, but do not rescale. Default: FALSE.}

\item{only_flip}{Logical. Flip the variable coding.  Unlike "only_reverse", this will
exactly preserve the values of the old variable.  For example, for a variable
with codes 1, 2, 3, 5, 10, only_flip will code the values 10, 5, 3, 2, 1 (instead
of 10, 9, 8, 6, 1).  Generally, reverse should be preferred to preserve the
underlying scale.  Not compatible with rescale. Default: FALSE.}

\item{map}{Logical. If TRUE, will print a cross-tab showing the old variable
and the new, recoded variable.  Used to verify the new variable is coded correctly.
Default: FALSE.}

\item{new_varlabel}{Character.  Variable label for the new variable.
Default: old variable's label.}

\item{new_vallabels}{Character vector. Supply custom names for value labels. Default:
value labels of old variable.}
}
\value{
The input variable rescaled
}
\description{
This function allows users to rescale and reorder variables.  It is designed
for variables of class "labelled" but the rescaling will work for numeric
and factor variables too.
}
\examples{

require(lapop); data(ym23)

# Regular data.frame
ym23$pn4r <- lpr_resc(ym23$pn4,
reverse = TRUE,
map = TRUE)

# LPR data.frame
ym23lpr<-lpr_data(ym23)
ym23lpr$variables$pn4r <- lpr_resc(ym23lpr$variables$pn4,
reverse = TRUE,
map = TRUE)

}
\author{
Luke Plutowski, \email{luke.plutowski@vanderbilt.edu} & Robert Vidigal, \email{robert.vidigal@vanderbilt.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lpr_ci.R
\name{lpr_ci}
\alias{lpr_ci}
\title{Compute Design-Based Proportion and Confidence Interval}
\usage{
lpr_ci(
  data,
  outcome,
  weight = "weight1500",
  strata = NULL,
  psu = NULL,
  conf.level = 0.95,
  na.rm = TRUE
)
}
\arguments{
\item{data}{A data frame containing the outcome and survey design variables.}

\item{outcome}{Character string. Name of a binary variable coded 0/1.}

\item{weight}{Character string. Name of the sampling weight variable.
Default is `"weight1500"`.}

\item{strata}{Character string. Name of the stratification variable.
Default is `NULL`. If provided together with `psu`, a complex survey
design is used.}

\item{psu}{Character string. Name of the primary sampling unit (cluster)
variable. Default is `NULL`.}

\item{conf.level}{Numeric. Confidence level for the interval.
Default is `0.95`.}

\item{na.rm}{Logical. If `TRUE`, rows with missing values in the
required variables are removed prior to estimation.}
}
\value{
A data frame with:
\describe{
  \item{prop}{Estimated proportion (mean of binary outcome).}
  \item{lb}{Lower bound of the confidence interval.}
  \item{ub}{Upper bound of the confidence interval.}
  \item{se}{Standard error of the estimate.}
}
}
\description{
Computes a weighted proportion (mean of a binary outcome) and its
confidence interval using complex survey design features. When
stratification and PSU variables are supplied, the function uses
Taylor linearized variance estimation via the \pkg{survey} package.
}
\details{
If both `strata` and `psu` are provided, a full complex survey design
is declared. If they are `NULL`, the function computes a weighted
estimate assuming simple random sampling (SRS) with weights.

Lonely PSUs are handled using `survey.lonely.psu = "adjust"`.


Variance estimation is performed using Taylor linearization as implemented
in \code{\link[survey]{svymean}}. When both `strata` and `psu` are supplied,
clustering and stratification are incorporated in the variance estimator.

If `strata` and `psu` are not provided, the function assumes a weighted
simple random sample and estimates variance accordingly.
}
\examples{
\donttest{
# Design-based estimate
data(cm23)
lpr_ci(data = cm23,
         outcome = "b13",
         weight = "weight1500",
         strata = "strata",
         psu = "upm")

# Weighted SRS estimate
data(bra23)
lpr_ci(data = bra23,
         outcome = "b13",
         weight = "wt")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lpr_cc.R
\name{lpr_cc}
\alias{lpr_cc}
\title{LAPOP Cross-Country Bar Graph Pre-Processing}
\usage{
lpr_cc(
  data,
  outcome,
  xvar = "pais_lab",
  rec = list(c(1, 1)),
  rec2 = list(c(1, 1)),
  rec3 = list(c(1, 1)),
  rec4 = list(c(1, 1)),
  ci_level = 0.95,
  mean = FALSE,
  filesave = "",
  cfmt = "",
  sort = "y",
  order = "hi-lo",
  ttest = FALSE,
  keep_nr = FALSE
)
}
\arguments{
\item{data}{A survey object.  The data that should be analyzed.}

\item{outcome}{Outcome variable(s) of interest to be plotted across countries.
It can handle a single variable across countries, or multiple variables instead of multiple countries. See examples below.}

\item{xvar}{Grouping variable. Default: pais_lab. It can handle other variables grouping like year/wave.}

\item{rec}{Numeric. The minimum and maximum values of the outcome variable that
should be included in the numerator of the percentage.  For example, if the variable
is on a 1-7 scale and rec is c(5, 7), the function will show the percentage who chose
an answer of 5, 6, 7 out of all valid answers.  Default: c(1, 1).}

\item{rec2}{Numeric. Same as rec(). Default: c(1, 1).}

\item{rec3}{Numeric. Same as rec(). Default: c(1, 1).}

\item{rec4}{Numeric. Same as rec(). Default: c(1, 1).}

\item{ci_level}{Numeric. Confidence interval level for estimates.  Default: 0.95}

\item{mean}{Logical.  If TRUE, will produce the mean of the variable rather than
rescaling to percentage.  Default: FALSE.}

\item{filesave}{Character.  Path and file name to save the dataframe as csv.}

\item{cfmt}{changes the format of the numbers displayed above the bars.
Uses sprintf string formatting syntax. Default is whole numbers for percentages
and tenths place for means.}

\item{sort}{Character. On what value the bars are sorted: the x or the y.
Options are "y" (default; for the value of the outcome variable), "xv" (for
the underlying values of the x variable), "xl" (for the labels of the x variable,
i.e., alphabetical).}

\item{order}{Character.  How the bars should be sorted.  Options are "hi-lo"
(default) or "lo-hi".}

\item{ttest}{Logical.  If TRUE, will conduct pairwise t-tests for difference
of means between all individual x levels and save them in attr(x,
"t_test_results"). Default: FALSE.}

\item{keep_nr}{Logical.  If TRUE, will convert "don't know" (missing code .a)
and "no response" (missing code .b) into valid data (value = 99) and use them
in the denominator when calculating percentages.  The default is to examine
valid responses only.  Default: FALSE.}
}
\value{
Returns a data frame, with data formatted for visualization by lapop_cc
}
\description{
This function creates dataframes which can then be input in lapop_cc for
comparing values across countries with a bar graph using LAPOP formatting.
}
\examples{

require(lapop); data(ym23); data(bra23)

# Set Survey Context on a small cross-country subset
ym23_small <- subset(ym23, pais \%in\% c(1, 15, 17))
ym23lpr <- lpr_data(ym23_small)

# Single variable in Multiple Countries
lpr_cc(data = ym23lpr,
      outcome = "ing4",
      rec = c(5, 7),
      xvar = "pais")

# Multiple variables in Single Country
\donttest{
bra23lpr <- lpr_data(bra23, wt = TRUE)
lpr_cc(data = bra23lpr,
outcome = c("b12", "b13"),
rec = c(5, 7))
}

}
\author{
Luke Plutowski, \email{luke.plutowski@vanderbilt.edu} & Robert Vidigal, \email{robert.vidigal@vanderbilt.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lpr_set_ros.R
\name{lpr_set_ros}
\alias{lpr_set_ros}
\title{Set Response Option (ROS) labels for variables in AmericasBarometer datasets}
\usage{
lpr_set_ros(data, lang_id = "en", attribute_name = "roslabel")
}
\arguments{
\item{data}{A data frame loaded using readstata13 containing label.table attributes.}

\item{lang_id}{Language identifier for the labels ("en" for English,
"es" for Spanish, "pt" for Portuguese). Default is "en" (English).}

\item{attribute_name}{The name of the attribute where the formatted response
options string will be stored. Default is "roslabel".}
}
\value{
The input data frame with response option labels added to variables as attributes
}
\description{
This function extracts formatted response option labels for AmericasBarometer
variables, using label tables stored as attributes. The labels are formatted
with their corresponding numeric codes and can be pulled in multiple languages.
}
\details{
The function looks for label tables stored as attributes of the data frame,
with names following the pattern "VARNAME_lang_id" (e.g., "ing4_en" for English
labels of variable ing4). Each label table should be a named numeric vector where
names are the response labels and values are the corresponding codes.

Special codes (values >= 1000) are excluded from the response options string.
}
\examples{

require(lapop); data(bra23)

# Apply the function
bra23 <- lpr_set_ros(bra23) # Default English
bra23 <- lpr_set_ros(bra23, lang_id = "es", attribute_name = "respuestas") # Spanish
bra23 <- lpr_set_ros(bra23, lang_id = "pt", attribute_name = "ROsLabels_pt") # Portuguese

# View the resulting attribute
attr(bra23$ing4, "roslabel")
attr(bra23$ing4, "respuestas")
attr(bra23$ing4, "ROsLabels_pt")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lapop_cc.R
\name{lapop_cc}
\alias{lapop_cc}
\title{LAPOP Cross-Country Bar Graphs}
\usage{
lapop_cc(
  data,
  outcome_var = data$prop,
  lower_bound = data$lb,
  vallabel = data$vallabel,
  upper_bound = data$ub,
  label_var = data$proplabel,
  ymin = 0,
  ymax = 100,
  lang = "en",
  highlight = "",
  main_title = "",
  source_info = "LAPOP",
  subtitle = "",
  sort = "",
  color_scheme = "#784885",
  label_size = 5,
  max_countries = 30,
  label_angle = 0
)
}
\arguments{
\item{data}{Data Frame. Dataset to be used for analysis.  The data frame should have columns
titled vallabel (values of x-axis variable (e.g. pais); character vector), prop (outcome variable; numeric),
proplabel (text of outcome variable; character), lb (lower bound of estimate; numeric),
and ub (upper bound of estimate; numeric). Default: None (must be supplied).}

\item{vallabel, outcome_var, label_var, lower_bound, upper_bound}{Character, numeric, character,
numeric, numeric. Each component of the plot data can be manually specified in case
the default columns in the data frame should not be used (if, for example, the values for a given
variable were altered and stored in a new column). x}

\item{ymin, ymax}{Numeric.  Minimum and maximum values for y-axis. Default: 0 to 100.}

\item{lang}{Character.  Changes default subtitle text and source info to either Spanish or English.
Will not translate input text, such as main title or variable labels.  Takes either "en" (English)
or "es" (Spanish).  Default: "en".}

\item{highlight}{Character.  Country of interest.  Will highlight (make darker) that country's bar.
Input must match entry in "vallabel" exactly. Default: None.}

\item{main_title}{Character.  Title of graph.  Default: None.}

\item{source_info}{Character.  Information on dataset used (country, years, version, etc.),
which is added to the bottom-left corner of the graph. Default: LAPOP ("Source: LAPOP Lab" will be printed).}

\item{subtitle}{Character.  Describes the values/data shown in the graph, e.g., "percentage of Mexicans who say...)".
Default: None.}

\item{sort}{Character. Method of sorting bars.  Options: "hi-lo" (highest to lowest y value), "lo-hi" (lowest to highest),
"alpha" (alphabetical by vallabel/x-axis label). Default: Order of data frame.}

\item{color_scheme}{Character.  Color of bars.  Takes hex number, beginning with "#".
Default: #784885.}

\item{label_size}{Numeric.  Size of text for data labels (percentages above bars).  Default: 5.}

\item{max_countries}{Numeric. Threshold for automatic x-axis label rotation. When the number of unique
country labels exceeds this value, labels will be rotated for better readability. Default: 20.}

\item{label_angle}{Numeric. Angle (in degrees) to rotate x-axis labels when max_countries is exceeded. Default: 0.}
}
\value{
Returns an object of class \code{ggplot}, a ggplot figure showing
average values of some variables across multiple countries.
}
\description{
This function creates bar graphs for comparing values across countries using LAPOP formatting.
}
\examples{
\donttest{
require(lapop); lapop_fonts()

df <- data.frame(
vallabel = c("PE", "CO", "BR", "PN", "GT", "DO", "MX", "BO", "EC"),
prop      = c(36.1, 19.3, 16.6, 13.3, 13.0, 11.1,  9.5,  9.0,  8.1),
proplabel = c("36\%" ,"19\%" ,"17\%" ,"13\%" ,"13\%" ,"11\%" ,"10\%", "9\%", "8\%"),
lb        = c(34.9, 18.1, 15.4, 12.1, 11.8,  9.9,  8.3,  7.8,  6.9),
ub        = c(37.3, 20.5, 17.8, 14.5, 14.2, 12.3, 10.7, 10.2,  9.3)
)
lapop_cc(df,
         main_title = "Normalization of Intimate Partner Violence in LAC Countries",
         subtitle = "\% who say domestic violence is private matter",
         source_info = "LAPOP Lab, AmericasBarometer 2021",
         highlight = "PE",
         ymax = 50)
}
}
\author{
Luke Plutowski, \email{luke.plutowski@vanderbilt.edu} & Robert Vidigal, \email{robert.vidigal@vanderbilt.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lpr_mline.R
\name{lpr_mline}
\alias{lpr_mline}
\title{LAPOP Multi-Line Time Series Graph Pre-Processing}
\usage{
lpr_mline(
  data,
  outcome,
  rec = c(1, 1),
  rec2 = c(1, 1),
  rec3 = c(1, 1),
  rec4 = c(1, 1),
  xvar,
  use_wave = FALSE,
  use_cat = FALSE,
  ci_level = 0.95,
  mean = FALSE,
  filesave = "",
  cfmt = "",
  ttest = FALSE,
  keep_nr = FALSE
)
}
\arguments{
\item{data}{A survey object.  The data that should be analyzed.}

\item{outcome}{Character vector.  Outcome variable(s) of interest to be plotted
across time.  If only one value is provided, the graph will show the outcome
variable, over time, broken down by a secondary variable (x-var).
If more than one value is supplied, the graph will show each outcome variable
across time (no secondary variable).}

\item{rec, rec2, rec3, rec4}{Numeric. The minimum and maximum values of the outcome
variable that should be included in the numerator of the percentage.
For example, if the variable is on a 1-7 scale and rec is c(5, 7), the
function will show the percentage who chose an answer of 5, 6, 7 out of
all valid answers.  Can also supply one value only, to produce the percentage
that chose that value out of all other values. Default: c(1, 1).}

\item{xvar}{Character. Variable on which to break down the outcome variable.
In other words, the line graph will produce multiple lines for each value of
xvar (technically, it is the z-variable, not the x variable, which is year/wave).
Ignored if multiple outcome variables are supplied.}

\item{use_wave}{Logical.  If TRUE, will use "wave" for the x-axis; otherwise,
will use "year".  Default: FALSE.}

\item{use_cat}{Logical. If TRUE, will show the percentages of category values
of a single variable; otherwise will show percentages of the range of values
from rec(). Default FALSE.}

\item{ci_level}{Numeric. Confidence interval level for estimates.  Default: 0.95}

\item{mean}{Logical.  If TRUE, will produce the mean of the variable rather than
rescaling to percentage.  Default: FALSE.}

\item{filesave}{Character.  Path and file name to save the dataframe as csv.}

\item{cfmt}{Character. changes the format of the numbers displayed above the bars.
Uses sprintf string formatting syntax. Default is whole numbers for percentages
and tenths place for means.}

\item{ttest}{Logical.  If TRUE, will conduct pairwise t-tests for difference
of means between all individual x levels and save them in attr(x,
"t_test_results"). Default: FALSE.}

\item{keep_nr}{Logical.  If TRUE, will convert "don't know" (missing code .a)
and "no response" (missing code .b) into valid data (value = 99) and use them
in the denominator when calculating percentages.  The default is to examine
valid responses only.  Default: FALSE.}
}
\value{
Returns a data frame, with data formatted for visualization by lapop_mline
}
\description{
This function creates a dataframe which can then be input in lapop_mline for
to show a time series plot with multiple lines.  If one "outcome" variable and an
`xvar` variable is supplied, the function produces the values of a single outcome
variable, broken down by a secondary variable, across time.  If multiple outcome
variables (up to four) are supplied, it will show means/percentages of those
variables across time (essentially, it allows you to do lpr_ts for multiple variables).
}
\examples{
\donttest{
require(lapop); data(ym23)

# Set Survey Context
ym23lpr <- lpr_data(ym23)

# Single Variable by Country and Year
lpr_mline(ym23lpr,
outcome = "ing4",
rec = c(5, 7),
xvar = "pais",
use_wave = TRUE)

# Multiple Variables
lpr_mline(ym23lpr,
outcome = c("b12", "b18"),
rec = c(5, 7),
rec2 = c(1, 2),
rec3 = c(5, 7),
use_wave = TRUE)

# Binary Single Variable by Category
lpr_mline(ym23lpr,
outcome = "pn4",
use_cat = TRUE,
use_wave = TRUE)

# Recode Categorical Variable (max 4-categories)
lpr_mline(ym23lpr,
outcome = "ing4",
rec = c(5, 7),
use_cat = TRUE,
use_wave = TRUE)
}
}
\author{
Luke Plutowski, \email{luke.plutowski@vanderbilt.edu} & Robert Vidigal, \email{robert.vidigal@vanderbilt.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cm23.R
\docType{data}
\name{cm23}
\alias{cm23}
\title{cm23: Single-country Multi-year Dataset}
\format{
A data frame
\describe{
 \item{ing4}{Support for Democracy}
 \item{b13}{Trust in the National Legislature}
 \item{b21}{Trust in Political Parties}
 \item{b31}{Trust in the Supreme Court of Justice}
 \item{wave}{Survey round year for regional or multi-country data}
 \item{pais}{Country of survey}
 \item{year}{Survey round year for single-country data}
 \item{upm}{Primary Sampling Unit}
 \item{strata}{Stratification}
 \item{weight1500}{Cross-country and cross-time weight}
}
}
\source{
LAPOP AmericasBarometer (https://www.vanderbilt.edu/lapop/)
}
\usage{
cm23
}
\description{
A dataset containing the AmericasBarometer Brazil  Country Merge up to 2023.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ym23.R
\docType{data}
\name{ym23}
\alias{ym23}
\title{ym23: Multi-country Single-year Dataset}
\format{
A data frame
\describe{
\item{b12}{Trust in Armed Forces}
\item{b18}{Trust in National Police}
\item{ing4}{Support for Democracy}
\item{pn4}{Satisfaction with Democracy}
\item{vb21n}{Influence Political Change}
\item{q14f}{Migration Intentions}
\item{edre}{Education}
\item{wealth}{Wealth}
\item{q1tc_r}{Gender}
\item{upm}{Primary Sampling Unit}
\item{strata}{Stratification}
\item{wave}{Survey round year for regional or multi-country data}
\item{pais}{Country of survey}
\item{year}{Survey round year for single-country data}
\item{weight1500}{Cross-country and cross-time weight}
}
}
\source{
LAPOP AmericasBarometer (https://www.vanderbilt.edu/lapop/)  # Ensure this line has a valid source.
}
\usage{
ym23
}
\description{
A dataset containing the AmericasBarometer Year Merge of 2023.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lpr_ts.R
\name{lpr_ts}
\alias{lpr_ts}
\title{LAPOP Time-Series Line Graph Pre-Processing}
\usage{
lpr_ts(
  data,
  outcome,
  rec = c(1, 1),
  use_wave = FALSE,
  ci_level = 0.95,
  mean = FALSE,
  filesave = "",
  cfmt = "",
  ttest = FALSE,
  keep_nr = FALSE
)
}
\arguments{
\item{data}{A survey object.  The data that should be analyzed.}

\item{outcome}{Character.  Outcome variable of interest to be plotted
across time.}

\item{rec}{Numeric. The minimum and maximum values of the outcome variable that
should be included in the numerator of the percentage.  For example, if the variable
is on a 1-7 scale and rec is c(5, 7), the function will show the percentage who chose
an answer of 5, 6, 7 out of all valid answers.  Can also supply one value only,
to produce the percentage that chose that value out of all other values.
Default: c(1, 1).}

\item{use_wave}{Logical.  If TRUE, will use "wave" for the x-axis; otherwise,
will use "year".  Default: FALSE.}

\item{ci_level}{Numeric. Confidence interval level for estimates.  Default: 0.95}

\item{mean}{Logical.  If TRUE, will produce the mean of the variable rather than
rescaling to percentage.  Default: FALSE.}

\item{filesave}{Character.  Path and file name to save the dataframe as csv.}

\item{cfmt}{Character. changes the format of the numbers displayed above the bars.
Uses sprintf string formatting syntax. Default is whole numbers for percentages
and tenths place for means.}

\item{ttest}{Logical.  If TRUE, will conduct pairwise t-tests for difference
of means between all individual x levels and save them in attr(x,
"t_test_results"). Default: FALSE.}

\item{keep_nr}{Logical.  If TRUE, will convert "don't know" (missing code .a)
and "no response" (missing code .b) into valid data (value = 99) and use them
in the denominator when calculating percentages.  The default is to examine
valid responses only.  Default: FALSE.}
}
\value{
Returns a data frame, with data formatted for visualization by lapop_ts()
}
\description{
This function creates dataframes which can then be input in lapop_ts for
comparing values across time with a line graph using LAPOP formatting.
}
\examples{

require(lapop); data(ym23)

# Set Survey Context
ym23lpr<-lpr_data(ym23)

# Run lpr_ts
lpr_ts(ym23lpr,
outcome = "ing4",
use_wave = TRUE,
mean = TRUE,
ttest = TRUE)

}
\author{
Berta Diaz, \email{berta.diaz.martinez@vanderbilt.edu} & Luke Plutowski, \email{luke.plutowski@vanderbilt.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lapop_map.R
\name{lapop_map}
\alias{lapop_map}
\title{LAPOP World and Americas Map Graph}
\usage{
lapop_map(
  data,
  outcome = "value",
  pais_lab = "pais_lab",
  survey = c("CSES", "AmericasBarometer"),
  zoom = 1,
  main_title = "",
  subtitle = "",
  palette = c("#F2A344", "#D97A1E", "#BF5A00", "#8A3900", "#4A1E00"),
  source_info = "LAPOP",
  lang = "en",
  selected_countries = NULL
)
}
\arguments{
\item{data}{A data frame containing ISO2 country codes and a value to map.}

\item{outcome}{String. Column name containing the numeric or categorical variable to visualize.}

\item{pais_lab}{String. Column name containing ISO2 country codes (e.g., `"US"`, `"BR"`).}

\item{survey}{Either `"CSES"` (full world map) or `"AmericasBarometer"` (Americas only).}

\item{zoom}{Numeric (0–1). Controls how tightly the map zooms when `survey = "AmericasBarometer"`. Default is `1`.}

\item{main_title}{Character. Title of graph.  Default: None.}

\item{subtitle}{Character.  Describes the values/data shown in the graph, e.g., "percentage of Mexicans who say...)".
Default: None.}

\item{palette}{Vector of up to 5 colors for continuous and factor variables.}

\item{source_info}{Character. Information on dataset used (country, years, version, etc.),
which is added to the bottom-left corner of the graph. Default: LAPOP ("Source: LAPOP Lab" will be printed).}

\item{lang}{Character.  Changes default subtitle text and source info to either Spanish or English.
Will not translate input text, such as main title or variable labels.  Takes either "en" (English)
or "es" (Spanish).  Default: "en".}

\item{selected_countries}{Character or NULL. ISO2 code of the currently selected country (e.g. from
`input$pais` in Shiny). When not `NULL`, countries with no data are rendered with diagonal stripes
instead of solid gray. Default: `NULL` (solid `"#dddddf"`).}
}
\value{
A `ggplot2` choropleth map object.
}
\description{
The `lapop_map()` generates a stylized choropleth map using ISO2 country codes
from both continuous and factor variables. It is designed to map cross-country
results from `lpr_cc()` and supports either a full world map (`survey = "CSES"`)
# or an Americas-only map (`survey = "AmericasBarometer"`).
}
\examples{
\donttest{
# Standalone — solid gray for no-data countries
lapop_fonts()
 data_cont <- data.frame(
  vallabel = c("US", "AR", "VE", "CH", "EC", "BO"),
  prop = c(37, 52, 80, 17, 69, 94)
)
lapop_map(data_cont, pais_lab = "vallabel", outcome = "prop", zoom = 0.9,
          survey = "AmericasBarometer", main_title = "Latin America and Caribbean Countries",
          subtitle = "\% of respondents")

}
if (interactive()) {
  lapop_map(data_cont, pais_lab = "vallabel", outcome = "prop", zoom = 0.9,
            survey = "AmericasBarometer", selected_countries = "BR")
}
}
\author{
Robert Vidigal, \email{robert.vidigal@vanderbilt.edu}
}
